/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.email.admin;

import java.awt.Component;
import java.awt.Container;
import javax.swing.LayoutFocusTraversalPolicy;

/**
 * This class represent a focus traversal policy based on an array of
 * components; the order is determined by the order of the components in that
 * array.
 *
 * @version $Id: ArrayFocusTraversalPolicy.java,v 1.1 2007/01/19 11:42:36 piter_may Exp $
 */
public class ArrayFocusTraversalPolicy extends LayoutFocusTraversalPolicy {
    
    // ------------------------------------------------------------ Private data
    
    /** components upon which the traversal order is determined */
    private Component[] traversalOrder;
    
    // ------------------------------------------------------------ Constructors
    
    /**
     * Creates a new instance of ArrayFocusTraversalPolicy
     * 
     * 
     * @param traversalOrder components to determine the traversal order
     */
    public ArrayFocusTraversalPolicy( Component[] traversalOrder ) {
        this.traversalOrder = traversalOrder;
    }
    
    // ------------------------------------------------------- Overrides methods
    
    @Override
    public Component getComponentAfter(Container focusCycleRoot, Component aComponent) {
        for ( int k = 0; k < traversalOrder.length - 1; k++ ) {

            // identify aComponent in the list of components            
            if ( traversalOrder[k].equals( aComponent ) ) {
                
                // return the first enabled field that follows aComponent
                for (int i = k + 1; i < traversalOrder.length; i++) {
                    if(!traversalOrder[i].isEnabled()){
                        continue;
                    }
                    return traversalOrder[i];
                }
            }
        }
        return super.getComponentAfter( focusCycleRoot, aComponent );
    }
    
    @Override
    public Component getComponentBefore(Container focusCycleRoot, Component aComponent) {
        for ( int k = 1; k < traversalOrder.length; k++ ) {

            // identify aComponent in the list of components            
            if ( traversalOrder[k].equals( aComponent ) ) {
                
                // return the first enabled field that precedes aComponent
                for (int i = k - 1; i >= 0; i--) {
                    if(!traversalOrder[i].isEnabled()){
                        continue;
                    }
                    return traversalOrder[i];
                }                
            }
        }
        return super.getComponentBefore( focusCycleRoot, aComponent );
    }
}
