/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.email.admin;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JPanel;

import com.funambol.framework.filter.Clause;

import com.funambol.admin.config.AdminConfig;
import com.funambol.admin.util.Log;

import com.funambol.email.admin.dao.WSDAO;
import com.funambol.email.model.MailServerAccount;
import java.awt.Dimension;

/**
 * This is the panel associated with the admin tool node for the email connector
 * module.
 *
 * @version $Id: SearchAccountPanel.java,v 1.12 2007/03/19 16:17:14 gbmiglia Exp $
 */
public class SearchAccountPanel extends JPanel implements ComponentObserver {
    
    // ------------------------------------------------------------ Private data
    
    /** data access object */
    private WSDAO WSDao;
    
    /** clause for filtered search */
    private Clause clause;

    /** controller that menages account processes */
    private AccountController accountController;
    
    /** search clauses */
    private FormSearchAccountPanel formSearchAccountPanel;
    
    /** search results */
    private ResultSearchAccountPanel resultSearchAccountPanel;
    
    /** at least one account search has been performed or not */
    private boolean firstSearchDone = false;    
                
    // ------------------------------------------------------------ Constructors
    
    /**
     * Creates a new instance of a SearchAccountPanel object. A reference to an
     * AccountController object is passed to it, in order to manage account 
     * processing.
     *
     * @param accountController the AccountController object reference
     */
    public SearchAccountPanel(AccountController accountController){
        this.accountController = accountController;
        try {
            initWSDAO();
            initGui();
            
        } catch (Exception e) {
            Log.error(e);
        }
    }
    
    
    
    /**
     * Set size of the panel
     * @return dimension of the panel
     */
    public Dimension getPreferredSize() {
        return new Dimension(350, 550);
    }
    
    // --------------------------------------------------------- Private methods
    
    /**
     * Initializes data access object.
     */
    private void initWSDAO() {

        String endPoint = "";
        String user = "";
        String pwd = "";
        int reloadingTime = 0;
        
        try {
            
            StringBuilder sb = new StringBuilder();
            sb.append("http://")
                .append(AdminConfig.getAdminConfig().getHostName())
                .append(':')
                .append(AdminConfig.getAdminConfig().getServerPort())
                .append(AdminConfig.getAdminConfig().getContextPath())
                .append("/services/email");
            endPoint = sb.toString();

            user = AdminConfig.getAdminConfig().getUser();
            pwd = AdminConfig.getAdminConfig().getPassword();
            reloadingTime = 1;

            WSDao = new WSDAO(endPoint, user, pwd);
            
        } catch (Exception e) {
            Log.error(e);
        }
    }
        
    /**
     * Initializes visual elements.
     */
    private void initGui() {
        
        //
        // search clauses
        //
        formSearchAccountPanel = new FormSearchAccountPanel();
                
        formSearchAccountPanel.addFormActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
            
                // determine which button has been pressed
                String command = event.getActionCommand();                               
                
                //
                // search button pressed
                //
                if (command == Constants.ACTION_COMMAND_SEARCH_ACCOUNT){

                    clause = formSearchAccountPanel.getClause();

                    try {

                        MailServerAccount[] displaiedAccount = WSDao.getAccounts(clause);
                        resultSearchAccountPanel.loadAccounts(displaiedAccount);                                                
                        
                    } catch (Exception e) {
                        Log.error(e);
                    }
                    
                    if (!firstSearchDone){
                        firstSearchDone = true;
                    }
                }

            }
        });
        
        //
        // search results
        //
        resultSearchAccountPanel = new ResultSearchAccountPanel();
                
        setLayout(new BorderLayout());
        add(formSearchAccountPanel,BorderLayout.PAGE_START);
        add(resultSearchAccountPanel,BorderLayout.CENTER);
        
        resultSearchAccountPanel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                                
                // determine shich button has been pressed
                String command = event.getActionCommand();
                
                String description = null;
                try {
                    accountController.setParentFrame(getOwner());
                    if (command.equals(Constants.ACTION_COMMAND_ADD_ACCOUNT)){
                        accountController.selectUser();
                    } else 
                    if (command.equals(Constants.ACTION_COMMAND_EDIT_ACCOUNT)){
                        
                        //
                        // 1 - gets selected account
                        // 2 - modify account
                        //
                        MailServerAccount selectedAccount = resultSearchAccountPanel.getSelectedAccount();
                        
                        // update the search account result table, if not empty

                        MailServerAccount account = accountController.modifyAccount(selectedAccount);                                                
                        resultSearchAccountPanel.updateRowInTable(account);
    
                    } else 
                    if (command.equals(Constants.ACTION_COMMAND_DELETE_ACCOUNT)){
                        
                        MailServerAccount selectedAccount = resultSearchAccountPanel.getSelectedAccount();
                        int numberRowsDeleted = accountController.deleteAccount(selectedAccount);
                        
                        if (numberRowsDeleted > 0){
                            
                            // update the search account result table, if not empty
                            resultSearchAccountPanel.deleteRowInTable();   
                        }                     
                    } else 
                    if (command.equals(Constants.ACTION_COMMAND_SHOW_CACHE)){
                        
                        MailServerAccount selectedAccount = 
                                resultSearchAccountPanel.getSelectedAccount();
                        accountController.showCache(
                                selectedAccount.getUsername(),
                                selectedAccount.getMailServer().getProtocol());
                    }   
                    
                } catch (Exception e) {
                    Log.error("Error handling Accounts" + e.getMessage());
                }
            }
        });
    }    
    
    /**
     * Updates the search result table. View upadate applies only if at least
     * one search has been performed (that is: flag firstSearchDone is true).
     */
    public void update() {
        
        if (!firstSearchDone){
            return;
        }
        
        Clause clause = formSearchAccountPanel.getClause();
        
        clause = formSearchAccountPanel.getClause();
        
        try {
            
            MailServerAccount[] displaiedAccount = WSDao.getAccounts(clause);
            resultSearchAccountPanel.loadAccounts(displaiedAccount);
            
        } catch (Exception e) {
            Log.error(e);
        }
        
    }

    // --------------------------------------------------------- Private methods
    
    /*
     * Returns the frame owner of this panel
     * @return Frame
     */
    private Frame getOwner() {
        Container parent = this;
        while ( (parent = parent.getParent()) != null) {
            if (parent instanceof Frame) {
                return (Frame)parent;
            }
        }
        return null;
    }

}
