/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.email.admin.cache;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import com.funambol.admin.ui.GuiFactory;
import com.funambol.admin.util.Log;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.model.SyncItemInfoAdmin;
import com.funambol.email.admin.dao.WSDAO;

/**
 * Panel to show each cache entry for a given (username, protocol).
 * @version $Id: CacheDetailsPanel.java,v 1.5 2007/01/26 13:40:32 piter_may Exp $
 */
public class CacheDetailsPanel extends JPanel {
    
    // ------------------------------------------------------------ Private data
        
    private String username;
    private String protocol;
    private SyncItemInfoAdmin[] cacheEntries;
    
    //
    // Visual elements
    //
    
    private JLabel panelName = new JLabel();

    /** table for displaying mail server data */
    private JTable table;
    
    private JButton jBClose = new JButton();
    
    // -------------------------------------------------------------- Properties
    
    /** data access object */
    private WSDAO WSDao;
    
    /**
     * Sets the WSDao property.
     * @param WSDao data access object
     */
    public void setWSDao(WSDAO WSDao) {
        this.WSDao = WSDao;
    }

    // ------------------------------------------------------------ Constructors
    
    /**
     * Creates a new instance of CacheDetailsPanel
     * @param username username these entries refer to
     * @param protocol protocol these entries refer to
     */
    public CacheDetailsPanel(String username, String protocol) {
        
        this.username = username;
        this.protocol = protocol;
        initGui();
    }
    
    // --------------------------------------------------------- Private methods
    
    /**
     * Initialize visual elements.
     */
    private void initGui(){
        
        setLayout(null);
        
        //
        // Configure visual elements
        //
        
        // title
        
        panelName.setFont(GuiFactory.titlePanelFont);
        
        StringBuilder sb = new StringBuilder();
        sb.append("Cache for ").append(username).append(',').append(protocol);
        panelName.setText(sb.toString());
        
        panelName.setBounds(new Rectangle(14, 5, 316, 28));
        panelName.setAlignmentX(SwingConstants.CENTER);
        panelName.setBorder(new TitledBorder(""));

        // cache entries table

        table = new JTable(new CacheEntriesTableModel());
        setBorder(BorderFactory.createEmptyBorder());
        table.setShowGrid(true);
        table.setAutoscrolls(true);
        //table.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        JScrollPane tableScrollPane = new JScrollPane(table);
        table.setPreferredScrollableViewportSize(new Dimension(500, 200));
        table.setFont(GuiFactory.defaultTableFont);
        table.getTableHeader().setFont(GuiFactory.defaultTableHeaderFont);
        tableScrollPane.setBounds(new Rectangle(0, 53, 794, 350));        

        // table columns
        
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        
        int tableWidth = this.getWidth();
        int columnWidth = 100;
        TableColumnModel columnModel = table.getColumnModel();
        int numCol = columnModel.getColumnCount();
        for (int i = 0; i < numCol; i++) {
            TableColumn column = columnModel.getColumn(i);
            switch (i){
                case 0:
                    column.setPreferredWidth(80);
                    break;
                case 5:
                    
                    int width = tableScrollPane.getWidth() - (80 + 100*4);
                    if (width > 0){
                        column.setPreferredWidth(width);                        
                    } else {
                        column.setPreferredWidth(100);
                    }
                    break;
                default:
                    column.setPreferredWidth(100);
            }
        }
        
        int vColIndex = 0;
        TableColumn col = table.getColumnModel().getColumn(vColIndex);
        int width = 20;
        col.setPreferredWidth(width);
        
        // buttons
        
        jBClose.setText("Close");        
        jBClose.setBounds(new Rectangle(350, 450, 100, 20));

        //
        // Add elements to this panel.
        //
        
        add(panelName);
        add(tableScrollPane, BorderLayout.WEST);
        add(jBClose);

    }
    
    // ---------------------------------------------------------- Public methods
    
    /**
     * Retrieve cache entries for this panel username and password.
     */
    public void loadCacheEntries(){
        try {
            cacheEntries = WSDao.getCachedInfo(username, protocol);
            
            CacheEntriesTableModel model = 
                    (CacheEntriesTableModel)table.getModel();
            model.loadEntries(cacheEntries);
            
        } catch (InboxListenerException e) {
            Log.error(e);
        }
    }
    
    /**
     * Sets the action bound to the cancel button.
     */
    public void setCloseAction(ActionListener actionListener){
        jBClose.addActionListener(actionListener);
    }    
    
    // ----------------------------------------------------------- Inner classes
    
    /**
     * Model of the table used to display cache entries.
     */
    private class CacheEntriesTableModel extends AbstractTableModel {
                    
            /** column names */
            String[] columnNames = new String[] {
                "n",
                "GUID",
                "Message Id",
                "Header date",
                "Header received",
                "subject"};
            
            /**
             * Container of data in the table.
             * <p/>
             * Must be not null at the time of creation.
             */
            Object[][] rowData = new Object[0][columnNames.length];
            
            public String getColumnName(int col) {
                return columnNames[col];
            }
            public int getColumnCount() {
                return columnNames.length;
            }
            public int getRowCount() {
                return rowData.length;
            }
            public Object getValueAt(int rowIndex, int columnIndex) {
                return rowData[rowIndex][columnIndex];
            }
            public void setValueAt(Object value, int rowIndex, int colIndex) {
                rowData[rowIndex][colIndex] = value;
                fireTableCellUpdated(rowIndex, colIndex);
            }
            
            /**
             * Fill table rows with given cache entries.
             */             
            public void loadEntries(SyncItemInfoAdmin[] entries){
                
                int size = entries.length;
                if (size != 0){
                    cacheEntries = entries;
                    rowData = new Object[size][columnNames.length];
                    for (int i = 0; i < cacheEntries.length; i++) {
                        setValueAt(Integer.toString(i + 1), i, 0);
                        setValueAt(entries[i].getGuid(), i, 1);
                        setValueAt(entries[i].getMessageID(), i, 2);
                        setValueAt(entries[i].getHeaderDate(), i, 3);
                        setValueAt(entries[i].getHeaderReceived(), i, 4);
                        setValueAt(entries[i].getSubject(), i, 5);
                    }
                    table.updateUI();
                }
            }
        };
}
