/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.email.admin.dao;

import java.util.logging.Logger;

import com.funambol.framework.filter.Clause;
import com.funambol.framework.filter.WhereClause;
import com.funambol.framework.tools.beans.BeanException;
import com.funambol.framework.tools.beans.BeanFactory;

import com.funambol.admin.util.Log;

import com.funambol.email.model.SyncItemInfoAdmin;
import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.WSTools;
import com.funambol.email.model.MailServer;
import com.funambol.email.model.MailServerAccount;

/**
 * Contains methods to access to the DB tables.
 * 
 * @version $Id: WSDAO.java,v 1.5 2007/01/26 13:39:18 piter_may Exp $
 */
public class WSDAO {

  // ------------------------------------------------------------ Private data
  
  /** utility object to access web services */
  private WSTools wsTools = null;

  // ------------------------------------------------------------- Constructor

  /**
   * Creates a new instance of WSDAO.
   *
   * @param endPoint 
   * @param user 
   * @param pwd
   * @throws com.funambol.email.exception.InboxListenerException 
   */
  public WSDAO(String endPoint, String user, String pwd) 
    throws InboxListenerException {
      try {
         endPoint = this.emailEndPoint(endPoint);
         this.wsTools  = new WSTools(endPoint,user,pwd);     
      } catch (Exception e) {
          throw new InboxListenerException("Error creating WS tool");
      }
  }

  // ---------------------------------------------------------- Public Methods

  /**
   * Retrieve an array of the MailServerAccount objects that match the given 
   * clause.
   *
   * @throws com.funambol.email.exception.InboxListenerException 
   * @throws com.funambol.framework.tools.beans.BeanException 
   * @return MailServerAccount[] 
   */
  public MailServerAccount[]  getAccounts(Clause clause)
  throws InboxListenerException, BeanException {
      
      MailServerAccount[] accounts = null;
      
      Object[] params = new Object[] { BeanFactory.marshal(clause) } ;
      
      try {
          Object obj = wsTools.invoke("getUsers", params);
          accounts = (MailServerAccount[])obj;
      } catch (Exception e)  {
          throw new InboxListenerException("Error getting all the accounts");
      }
      return accounts;
      
  }

  /**
   * Retrieve an array of the MailServer objects that match the given clause.
   *
   * @throws com.funambol.email.exception.InboxListenerException 
   * @throws com.funambol.framework.tools.beans.BeanException 
   * @return MailServer[] 
   */
  public MailServer[] getMailServers(Clause clause) throws InboxListenerException, BeanException {
      MailServer[] mailServers = null;
      
      Object[] params = new Object[] { BeanFactory.marshal(clause) } ;
      
      try {
          Object obj = wsTools.invoke("getPubMailServers", params);
          mailServers = (MailServer[])obj;
      } catch (Exception e)  {
          throw new InboxListenerException("Error getting all the mail Servers");
      }
      return mailServers;      
  }
  
  /**
   * Retrieve the account for the given username.
   *
   * @return account info MailboxAccount
   * @param username String
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public MailServerAccount getAccount (String username) 
    throws InboxListenerException {      
        MailServerAccount msa = null;        
        Object[] params = new Object[] {username};
        try { 
            msa = (MailServerAccount)wsTools.invoke("getUser", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting the account");
        }        
        return msa;
  }
      
  /**
   * Insert a given account.
   * @param msa the account to be inserted
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public void insertAccount (MailServerAccount msa)
    throws InboxListenerException {      
        Object[] params = new Object[] {msa};
        try { 
           wsTools.invoke("insertUser", params);      
        } catch (Exception e)  {
            throw new InboxListenerException("inserAccount: Error inserting the account");
        }
  }

  /**
   * Insert a given mail server.
   * @param ms the mail server to be inserted
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public void insertMailServer (MailServer ms)
    throws InboxListenerException {      
        Object[] params = new Object[] {ms};
        try { 
           wsTools.invoke("insertPubMailServer", params);      
        } catch (Exception e)  {
            throw new InboxListenerException("Error inserting the mail server");
        }
  }
  
  /**
   * Updates a given account.
   * @param msa account to be updated
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public void updateAccount (MailServerAccount msa)
    throws InboxListenerException {
        Object[] params = new Object[] {msa};
        try { 
            wsTools.invoke("updateUser", params);                
        } catch (Exception e)  {
            throw new InboxListenerException("updateAccount: Error inserting the account");
        }
  }  
  
  /**
   * Updates a given mail server.
   * @param ms mail server to be updated
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public void updateMailServer (MailServer ms)
    throws InboxListenerException {      
        Object[] params = new Object[] {ms};
        try { 
            wsTools.invoke("updatePubMailServer", params);                
        } catch (Exception e)  {
            throw new InboxListenerException("Error inserting the mail server");
        }
  }
  
  
  /**
   * Delete account with the given ID.
   * @param accountID id for the account to be deleted.
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public void deleteAccount (String accountID)
    throws InboxListenerException {
        Object[] params = new Object[] {accountID};
        try {
            wsTools.invoke("deleteUser", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error deleting the account");
        }        
  }
  
  /**
   * Delete mail server with the given ID.
   * @param mailserverID id for the mail server to be deleted.
   * @throws com.funambol.email.exception.InboxListenerException
   */
  public int deleteMailServer (String mailserverID)
    throws InboxListenerException {
        Object[] params = new Object[] {mailserverID};
        int num = 0;
        try {
            num = ((Integer)wsTools.invoke("deletePubMailServer", params)).intValue();
        } catch (Exception e)  {
            throw new InboxListenerException("Error deleting the mail server");
        }    
        return num;
  }
  
    /**
     * Retrieve all the cache entries for given username,protocol.
     * @param username 
     * @param protocol 
     * @throws com.funambol.email.exception.InboxListenerException 
     * @return the array of cache entries
     */
  public SyncItemInfoAdmin[] getCachedInfo(String username, String protocol)
  throws InboxListenerException {
      SyncItemInfoAdmin[] info = null;
      Object[] params = new Object[] {username, protocol};
      try {
          Object obj = wsTools.invoke("getCachedInfo", params);
          info = (SyncItemInfoAdmin[])obj;
      } catch (Exception e)  {
          throw new InboxListenerException("Error getting cached info (" + e.getMessage() + ")");
      }
      return info;
  }

  
  // ----------------------------------------------------------- Private methods
  
  /**
     * Creates the email service end point given the one for the admin service.
     * @param adminEndPoint the admin service endpoint
     * @return the email service endpoint
     */
  private String emailEndPoint(String adminEndPoint) {
      String ADMIN = "admin";
      String EMAIL = "email";
      
      String tmp = adminEndPoint.substring(0,(adminEndPoint.length() - ADMIN.length()));
      tmp = tmp + EMAIL;
      
      return tmp;
      
  }

  
}
