/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.email.admin.mailservers;

import java.util.ArrayList;
import java.util.List;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JPanel;

import com.funambol.framework.filter.Clause;

import com.funambol.admin.util.Log;
import com.funambol.email.model.MailServer;

import com.funambol.email.admin.ComponentObserver;
import com.funambol.email.admin.dao.WSDAO;
import com.funambol.email.admin.MailServerController;

/**
 * Panel for mail server handling search and selection.
 *
 * @version $Id: SearchMailServerPanel.java,v 1.9 2007/01/26 13:39:28 piter_may Exp $
 */
public class SearchMailServerPanel extends JPanel implements ComponentObserver {
    
    // ------------------------------------------------------------ Private data
    
    /** list of all the components whose view needs to be updated */
    private List<ComponentObserver> observers = 
            new ArrayList<ComponentObserver>();

    //
    // Visual elements.
    //
    
    /** search clause panel */
    private FormSearchMailServerPanel formPanel;
    
    /** search result panel */
    private ResultSearchMailServerPanel resultPanel;

    // -------------------------------------------------------------- Properties
        
    /** controller */
    private MailServerController controller;
        
    /** Data access object. */
    private WSDAO WSDao;
    
    /** 
     * Sets the WSDao property 
     */
    public void setWSDao(WSDAO WSDao) {
        this.WSDao = WSDao;
        controller.setWSDao(this.WSDao);
    }
        
    // ------------------------------------------------------------ Constructors
    
    /** 
     * Creates a SearchMailServerPanel object. 
     */
    public SearchMailServerPanel() {
        controller = new MailServerController();
        try {
            initGui();            
        } catch (Exception e) {
            Log.error("Error while initilizing SearchMailServerPanel object", e);
        }
    }
    
    // --------------------------------------------------------- Private methods
    
    /**
     * Initializes visual elements.
     */
    private void initGui(){

        formPanel = new FormSearchMailServerPanel();        
        resultPanel = new ResultSearchMailServerPanel();
        
        formPanel.addSearchAction(new ActionListener() {
            public void actionPerformed(ActionEvent event) {

                Clause clause = formPanel.getClause();
                
                try {
                    
                    MailServer[] displaiedMailServers = WSDao.getMailServers(clause);
                    resultPanel.setMailServers(displaiedMailServers);
                    
                } catch (Exception e) {
                    Log.error(e);
                }
            }
        });

        resultPanel.setAddAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                controller.setParentFrame(getOwner());
                controller.insertMailServer();
            }
        });
        
        resultPanel.setModifyAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                MailServer mailServer = resultPanel.getSelectedMailServer();
                if (mailServer == null){
                    return;
                }
                controller.setParentFrame(getOwner());
                controller.modifyMailServer(mailServer);
                resultPanel.updateMailServer(mailServer);
                update();
            }
        });
        
        resultPanel.setDeleteAction(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                MailServer selectedMailServer = resultPanel.getSelectedMailServer();
                controller.setParentFrame(getOwner());
                int numberRowsDeleted = 
                        controller.deleteMailServer(selectedMailServer);
                
                StringBuilder sb = new StringBuilder();
                if (numberRowsDeleted > 0){                                        
                    // update the search account result table, if not empty                    
                    resultPanel.deleteRowInTable(selectedMailServer);                    
                }                       
            }
        });
        
        setLayout(new BorderLayout());
        add(formPanel,BorderLayout.PAGE_START);
        add(resultPanel,BorderLayout.CENTER);

    }

    // ---------------------------------------------------------- Public methods
    
    /**
     * Adds a component to the components that need to be notified for updates.
     * @param observer component to be added
     */
    public void addObserver(ComponentObserver observer) {
        observers.add(observer);
    }

    /**
     * Updates this component.
     */
    public void update() {
        for (ComponentObserver observer : observers) {
            observer.update();
        }
    }

    // --------------------------------------------------------- Private methods
    /*
     * Returns the frame owner of this panel
     * @return Frame
     */
    private Frame getOwner() {
        Container parent = this;
        while ( (parent = parent.getParent()) != null) {
            if (parent instanceof Frame) {
                return (Frame)parent;
            }
        }
        return null;
    }    
}













