/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.console.dao;

import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import javax.sql.DataSource;
import com.funambol.framework.tools.DBTools;
import com.funambol.email.exception.EntityException;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.MailServer;
import com.funambol.email.model.SyncItemInfoAdmin;
import com.funambol.email.util.Def;
import com.funambol.email.util.Query;
import com.funambol.email.util.TokenUtility;
import com.funambol.email.util.Utility;
import com.funambol.framework.filter.AllClause;
import com.funambol.framework.filter.Clause;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.server.store.PreparedWhere;
import com.funambol.framework.server.store.SQLHelperClause;
import com.funambol.framework.tools.DataSourceTools;
import java.util.ArrayList;

/**
 * This class implements methods to access data
 * in Connector DB Schema.
 *
 * @version $Id: ConsoleDAO.java,v 1.29 2007/03/19 14:25:52 gbmiglia Exp $
 */
public class ConsoleDAO {

    // --------------------------------------------------------------- Constants
    
    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    // -------------------------------------------------------------- Properties

    /**   */
    protected String jndiDataSourceName = null;
    
    /**   */
    protected String encryptionKey     = null;
    
    // ------------------------------------------------------------- Constructor
    
    /**
     *
     */
    private ConsoleDAO(){
    }

    /**
     *
     * @param _jndiDataSourceName String
     * @param _encryptionKey String
     */
    public ConsoleDAO(String _jndiDataSourceName, String _encryptionKey){
        this.jndiDataSourceName = _jndiDataSourceName;
        this.encryptionKey      = _encryptionKey;
    }


    // -----------------------------------------------------------Public Methods

    
    /**
     *
     * insert user and the mail server configuration
     *
     * @param msa MailServerAccount
     * @return result of insert operation int
     * @throws EntityException
     */
    public int insertUser(MailServerAccount msa)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       String username       = null;
       int result            = 0;

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          // insert into fnbl_email_enable_account
          
          ps = connection.prepareStatement(Query.MS_ACCOUNT_INSERT_USER);

          // handling password
          String plain_text = msa.getMsPassword();
          String encryptedPassword = TokenUtility.encryptPassword(plain_text,
                                                                     this.encryptionKey);          
          
          // account
          ps.setString(1, msa.getAccountId());
          ps.setString(2, msa.getUsername());
          ps.setString(3, msa.getMsLogin());
          ps.setString(4, encryptedPassword);          
          ps.setString(5, msa.getMsAddress());          
          ps.setString(6, msa.getListenerId());
          ps.setString(7, Utility.booleanToString(msa.getActivation()));
          ps.setString(8, Utility.booleanToString(msa.getPush()));
          ps.setInt   (9, msa.getRefreshTime());
          ps.setInt   (10, msa.getMaxEmailNumber());
          ps.setInt   (11, msa.getMaxImapEmail());

          // mail server
          ps.setString(12, msa.getMailServer().getMailServerId());
          ps.setString(13, msa.getMailServer().getMailServerType());
          ps.setString(14, msa.getMailServer().getDescription());
          ps.setString(15, msa.getMailServer().getProtocol());
          ps.setString(16, msa.getMailServer().getOutServer());
          ps.setInt   (17, msa.getMailServer().getOutPort());
          ps.setString(18, Utility.booleanToString(msa.getMailServer().getOutAuth()));
          ps.setString(19, msa.getMailServer().getInServer());
          ps.setInt   (20, msa.getMailServer().getInPort());
          ps.setString(21, Utility.booleanToString(msa.getMailServer().getIsSSLIn()));
          ps.setString(22, Utility.booleanToString(msa.getMailServer().getIsSSLOut()));
          ps.setString(23, msa.getMailServer().getKeystorePath());
          ps.setString(24, msa.getMailServer().getKeystorePwd());
          
          ps.setString(25, "Inbox");
          ps.setString(26, "y");
          ps.setString(27, msa.getMailServer().getOutboxPath());
          ps.setString(28, Utility.booleanToString(msa.getMailServer().getOutboxActivation()));
          ps.setString(29, msa.getMailServer().getSentPath());
          ps.setString(30, Utility.booleanToString(msa.getMailServer().getSentActivation()));
          ps.setString(31, msa.getMailServer().getDraftsPath());
          ps.setString(32, Utility.booleanToString(msa.getMailServer().getDraftsActivation()));
          ps.setString(33, msa.getMailServer().getTrashPath());
          ps.setString(34, Utility.booleanToString(msa.getMailServer().getTrashActivation()));
          
          ps.setString(35, Utility.booleanToString(msa.getMailServer().getIsSoftDelete()));          
          
          ps.setString(36, Utility.booleanToString(msa.getMailServer().getIsPublic()));          
                    
          result = ps.executeUpdate();

          
          // insert into fnbl_email_enable_account
          
          ps = connection.prepareStatement(Query.MS_ACCOUNT_ENABLER);
          
          ps.setString(1, msa.getAccountId());
          ps.setString(2, msa.getUsername());
          
          result = ps.executeUpdate();
                    
          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;
       
    }
    
    /**
     *
     * delete user 
     *
     * @param accountId String
     * @return int
     * @throws EntityException
     */
    public int deleteUser(String accountId)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;

       try {

          String username = this.getUsername(accountId);
           
          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          // delete from fnbl_email_account
          ps = connection.prepareStatement(Query.MS_ACCOUNT_DELETE_USER);
          ps.setString(1, accountId);
          result = ps.executeUpdate();
          
          // delete from fnbl_email_enable_account
          ps = connection.prepareStatement(Query.MS_ACCOUNT_DELETE_ENABLE_USER);
          ps.setString(1, accountId);
          result = result + ps.executeUpdate();
          
          // delete from fnbl_email_inbox
          ps = connection.prepareStatement(Query.INBOX_DELETE_EMAILS);
          ps.setString(1, username);
          ps.executeUpdate();

          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }
       
       return result;
       
    }
    
    /**
     *
     * disable account
     *
     * @param accountId String
     * @return int
     * @throws EntityException
     */
    public int disableUser(String accountId)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          // delete from fnbl_email_account
          ps = connection.prepareStatement(Query.MS_ACCOUNT_DISABLE_USER);
          ps.setString(1, accountId);
          result = ps.executeUpdate();
                    
          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;
       
    }
    
    /**
     *
     * disable account
     *
     * @param accountId String
     * @return int
     * @throws EntityException
     */
    public int enableUser(String accountId)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          // delete from fnbl_email_account
          ps = connection.prepareStatement(Query.MS_ACCOUNT_ENABLE_USER);
          ps.setString(1, accountId);
          result = ps.executeUpdate();
                    
          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;
       
    }
    
    /**
     *
     * update user regarding the mail server connection
     *
     * @param msa MailServerAccount
     * @return int
     * @throws EntityException
     */
    public int updateUser(MailServerAccount msa)
    throws EntityException {
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          ps = connection.prepareStatement(Query.MS_ACCOUNT_UPDATE_USER);

          // handling password
          String plain_text = msa.getMsPassword();
          String encryptedPassword = TokenUtility.encryptPassword(plain_text, 
                                                                     this.encryptionKey);          
          
          // account
          ps.setString(1, msa.getMsLogin());
          ps.setString(2, encryptedPassword);          
          ps.setString(3, msa.getMsAddress());          
          ps.setString(4, msa.getListenerId());
          ps.setString(5, Utility.booleanToString(msa.getActivation()));
          ps.setString(6, Utility.booleanToString(msa.getPush()));
          ps.setInt   (7, msa.getRefreshTime());
          ps.setInt   (8, msa.getMaxEmailNumber());
          ps.setInt   (9, msa.getMaxImapEmail());

          // mail server
          ps.setString(10, msa.getMailServer().getMailServerId());
          ps.setString(11, msa.getMailServer().getMailServerType());
          ps.setString(12, msa.getMailServer().getDescription());
          ps.setString(13, msa.getMailServer().getProtocol());
          ps.setString(14, msa.getMailServer().getOutServer());
          ps.setInt   (15, msa.getMailServer().getOutPort());
          ps.setString(16, Utility.booleanToString(msa.getMailServer().getOutAuth()));
          ps.setString(17, msa.getMailServer().getInServer());
          ps.setInt   (18, msa.getMailServer().getInPort());
          ps.setString(19, Utility.booleanToString(msa.getMailServer().getIsSSLIn()));
          ps.setString(20, Utility.booleanToString(msa.getMailServer().getIsSSLOut()));
          ps.setString(21, msa.getMailServer().getKeystorePath());
          ps.setString(22, msa.getMailServer().getKeystorePwd());
          
          ps.setString(23, "Inbox");
          ps.setString(24, "y");
          ps.setString(25, msa.getMailServer().getOutboxPath());
          ps.setString(26, Utility.booleanToString(msa.getMailServer().getOutboxActivation()));
          ps.setString(27, msa.getMailServer().getSentPath());
          ps.setString(28, Utility.booleanToString(msa.getMailServer().getSentActivation()));
          ps.setString(29, msa.getMailServer().getDraftsPath());
          ps.setString(30, Utility.booleanToString(msa.getMailServer().getDraftsActivation()));
          ps.setString(31, msa.getMailServer().getTrashPath());
          ps.setString(32, Utility.booleanToString(msa.getMailServer().getTrashActivation()));
          
          ps.setString(33, Utility.booleanToString(msa.getMailServer().getIsSoftDelete()));          
          
          ps.setString(34, Utility.booleanToString(msa.getMailServer().getIsPublic()));          

          ps.setString(35, msa.getAccountId());          
          ps.setString(36, msa.getUsername());          
          
          result = ps.executeUpdate();

          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;
       
    }
    
    
    /**
     *
     * get users in the filter
     *
     * @param clause Clause
     * @return list with all the user in the filter ArrayList
     * @throws EntityException
     */
    public MailServerAccount[] getUsers(Clause clause)
    throws EntityException {
        
        ArrayList msal         = new ArrayList();
        MailServerAccount[] msalOut = null;
        Connection connection  = null;
        PreparedStatement stmt = null;
        ResultSet rs           = null;
        MailServerAccount msa  = null;
        MailServer        pms  = null;
        String encryptedPassword = null;
        String plain_text        = null;

        try {

           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();

           String dbProductName = connection.getMetaData().getDatabaseProductName();           
           SQLHelperClause sqlHelperClause = new SQLHelperClause(dbProductName);
           
           String query = Query.MS_ACCOUNT_LIST_USER;
           if (clause != null){
               PreparedWhere where = sqlHelperClause.getPreparedWhere(clause);
               if ( (clause != null) && (! (clause instanceof AllClause))) {
                   if (where.sql.length() > 0){
                       query += " where " + where.sql;
                   }
               }    
               stmt = connection.prepareStatement(query);
               if ( (clause != null) && (! (clause instanceof AllClause))) {
                   for (int i=0; i<where.parameters.length; ++i) {
                       stmt.setObject(i+1, where.parameters[i]);
                   }
               }    
           } else {
               stmt = connection.prepareStatement(query);
           }
                                              
           rs = stmt.executeQuery();
                                 
           while(rs.next()) {
               
               msa = new MailServerAccount();
               pms = new MailServer();
               // account
               msa.setAccountId(rs.getString(1));   
               msa.setUsername(rs.getString(2));    
               msa.setMsLogin(rs.getString(3));     
               
               encryptedPassword = rs.getString(4);
               try {
                   plain_text = 
                        TokenUtility.decryptPassword(encryptedPassword, this.encryptionKey);
               } catch (Exception e){
                   throw new Exception("Error handling password decryption");
               }
               msa.setMsPassword(plain_text);  
               
               msa.setMsAddress(rs.getString(5));  
               msa.setListenerId(rs.getString(6));  
               msa.setActivation(Utility.booleanFromString(rs.getString(7))); 
               msa.setPush(Utility.booleanFromString(rs.getString(8))); 
               
               // in the UI we show the minutes but in the DB we save the seconds
               int sec = rs.getInt(9);
               msa.setRefreshTime(Utility.setMinutes(sec));
               
               msa.setMaxEmailNumber(rs.getInt(10)); 
               msa.setMaxImapEmail(rs.getInt(11)); 
               
               // mail server
               pms.setMailServerId(rs.getString(12));   
               pms.setMailServerType(rs.getString(13)); 
               pms.setDescription(rs.getString(14));    
               pms.setProtocol(rs.getString(15));       
               pms.setOutServer(rs.getString(16)); 
               pms.setOutPort(rs.getInt(17)); 
               pms.setOutAuth(Utility.booleanFromString(rs.getString(18))); 
               pms.setInServer(rs.getString(19)); 
               pms.setInPort(rs.getInt(20)); 
               pms.setIsSSLIn(Utility.booleanFromString(rs.getString(21))); 
               pms.setIsSSLOut(Utility.booleanFromString(rs.getString(22))); 
               pms.setKeystorePath(rs.getString(23)); 
               pms.setKeystorePwd(rs.getString(24)); 
               pms.setInboxPath(rs.getString(25));  
               pms.setInboxActivation(Utility.booleanFromString(rs.getString(26))); 
               pms.setOutboxPath(rs.getString(27)); 
               pms.setOutboxActivation(Utility.booleanFromString(rs.getString(28))); 
               pms.setSentPath(rs.getString(29));   
               pms.setSentActivation(Utility.booleanFromString(rs.getString(30)));  
               pms.setDraftsPath(rs.getString(31));   
               pms.setDraftsActivation(Utility.booleanFromString(rs.getString(32))); 
               pms.setTrashPath(rs.getString(33));   
               pms.setTrashActivation(Utility.booleanFromString(rs.getString(34))); 
               pms.setIsSoftDelete(Utility.booleanFromString(rs.getString(35))); 
               pms.setIsPublic(Utility.booleanFromString(rs.getString(36))); 
                              
               msa.setMailServer(pms);               
               
               msal.add(msa);
               
           }
           
           msalOut =  (MailServerAccount[])msal.toArray(new MailServerAccount[0]) ;

           stmt.close() ;
           stmt = null  ;
           
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, stmt, null);
        }

        return msalOut;
    }
    
    /**
     *
     * get user
     *
     * @param username String
     * @return all the info about the user and the related mail server MailServerAccount
     * @throws EntityException
     */
    public MailServerAccount getUser(String username)
    throws EntityException {
        
        MailServerAccount msa = null;
        MailServer pms        = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;
        String encryptedPassword = null;
        String plain_text        = null;
        
        try {

           String accountId = getAccountId(username);
           
           if (accountId != null){
               
               DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

               connection = dataSource.getConnection();

               ps = connection.prepareStatement(Query.MS_ACCOUNT_GET_USER);

               ps.setString (1, accountId ) ;

               rs = ps.executeQuery();

               while(rs.next()) {
                   msa = new MailServerAccount();
                   pms = new MailServer();

                   // account
                   msa.setAccountId(rs.getString(1));      
                   msa.setUsername(rs.getString(2));    
                   msa.setMsLogin(rs.getString(3));  
                   
                   encryptedPassword = rs.getString(4);
                   try {
                       plain_text = 
                            TokenUtility.decryptPassword(encryptedPassword, this.encryptionKey);
                   } catch (Exception e){
                       throw new Exception("Error handling password decryption");
                   }
                   msa.setMsPassword(plain_text);  
                   
                   msa.setMsAddress(rs.getString(5));  
                   msa.setListenerId(rs.getString(6)); 
                   msa.setActivation(Utility.booleanFromString(rs.getString(7))); 
                   msa.setPush(Utility.booleanFromString(rs.getString(8)));
                   
                   // in the UI we show the minutes but in the DB we save the seconds
                   int sec = rs.getInt(9);
                   msa.setRefreshTime(Utility.setMinutes(sec));
                   
                   msa.setMaxEmailNumber(rs.getInt(10));
                   msa.setMaxImapEmail(rs.getInt(11)); 

                   // mail server
                   pms.setMailServerId(rs.getString(12));   
                   pms.setMailServerType(rs.getString(13)); 
                   pms.setDescription(rs.getString(14));   
                   pms.setProtocol(rs.getString(15));      
                   pms.setOutServer(rs.getString(16)); 
                   pms.setOutPort(rs.getInt(17)); 
                   pms.setOutAuth(Utility.booleanFromString(rs.getString(18))); 
                   pms.setInServer(rs.getString(19)); 
                   pms.setInPort(rs.getInt(20)); 
                   pms.setIsSSLIn(Utility.booleanFromString(rs.getString(21))); 
                   pms.setIsSSLOut(Utility.booleanFromString(rs.getString(22))); 
                   pms.setKeystorePath(rs.getString(23)); 
                   pms.setKeystorePwd(rs.getString(24)); 
                   pms.setInboxPath(rs.getString(25));  
                   pms.setInboxActivation(Utility.booleanFromString(rs.getString(26))); 
                   pms.setOutboxPath(rs.getString(27)); 
                   pms.setOutboxActivation(Utility.booleanFromString(rs.getString(28))); 
                   pms.setSentPath(rs.getString(29));   
                   pms.setSentActivation(Utility.booleanFromString(rs.getString(30))); 
                   pms.setDraftsPath(rs.getString(31));   
                   pms.setDraftsActivation(Utility.booleanFromString(rs.getString(32))); 
                   pms.setTrashPath(rs.getString(33));   
                   pms.setTrashActivation(Utility.booleanFromString(rs.getString(34))); 
                   pms.setIsSoftDelete(Utility.booleanFromString(rs.getString(35))); 
                   pms.setIsPublic(Utility.booleanFromString(rs.getString(36))); 

                   msa.setMailServer(pms);               

               }

               ps.close() ;
               ps = null  ;
               
           }
           
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return msa;
    }
     
    /**
     *
     * get user
     *
     * @param username String
     * @return all the info about the user and the related mail server MailServerAccount
     * @throws EntityException
     */
    public SyncItemInfoAdmin[] getCachedInfo(String username, String protocol)
    throws EntityException {
        
        SyncItemInfoAdmin[] out        = null;
        ArrayList           infos      = new ArrayList();
        SyncItemInfoAdmin   record     = null;
        Connection          connection = null;
        PreparedStatement   ps         = null;
        ResultSet           rs         = null;
        
        try {
                          
           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

           connection = dataSource.getConnection();

           ps = connection.prepareStatement(Query.INBOX_SELECT_INFO_4_ADMIN);

           ps.setString (1, username ) ;
           ps.setString (2, protocol ) ;
           
           rs = ps.executeQuery();

           while(rs.next()) {
               
               record = new SyncItemInfoAdmin();
              
               record.setGuid(rs.getString(1)) ;        
               record.setMessageID(rs.getString(2)) ;  
               record.setHeaderDate(Utility.UtcToDate(rs.getString(3))) ; 
               record.setHeaderReceived(Utility.UtcToDate(rs.getString(4))) ;
               record.setSubject(rs.getString(5)) ;           
               
               infos.add(record);

           }

           int len = infos.size();
           out = new SyncItemInfoAdmin[len];
           for (int i=0; i<len; i++){
               out[i] = (SyncItemInfoAdmin)infos.get(i);
           }
           
           //out = (SyncItemInfoAdmin[])infos.toArray(new SyncItemInfoAdmin[0]) ;
           
           ps.close() ;
           ps = null  ;
               
           
           
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return out;
    }
    
    
    /**
     *
     * get user
     *
     * @param accountId String
     * @return all the info about the user and the related mail server MailServerAccount
     * @throws EntityException
     */
    public MailServerAccount getUserFromID(String accountId)
    throws EntityException {
        MailServerAccount msa = null;
        MailServer pms        = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;
        String encryptedPassword = null;
        String plain_text        = null;
        
        try {
           
           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.MS_ACCOUNT_GET_USER);
           
           ps.setString (1, accountId ) ;

           rs = ps.executeQuery();
           
           while(rs.next()) {
               msa = new MailServerAccount();
               pms = new MailServer();
               
               // account
               msa.setAccountId(rs.getString(1));      
               msa.setUsername(rs.getString(2));    
               msa.setMsLogin(rs.getString(3));   
               
               encryptedPassword = rs.getString(4);
               try {
                   plain_text = 
                        TokenUtility.decryptPassword(encryptedPassword, this.encryptionKey);
               } catch (Exception e){
                   throw new Exception("Error handling password decryption");
               }
               msa.setMsPassword(plain_text);  
               
               msa.setMsAddress(rs.getString(5));  
               msa.setListenerId(rs.getString(6)); 
               msa.setActivation(Utility.booleanFromString(rs.getString(7))); 
               msa.setPush(Utility.booleanFromString(rs.getString(8)));

               // in the UI we show the minutes but in the DB we save the seconds
               int sec = rs.getInt(9);
               msa.setRefreshTime(Utility.setMinutes(sec));               
               
               msa.setMaxEmailNumber(rs.getInt(10));
               msa.setMaxImapEmail(rs.getInt(11)); 
               
               // mail server
               pms.setMailServerId(rs.getString(12));   
               pms.setMailServerType(rs.getString(13)); 
               pms.setDescription(rs.getString(14));   
               pms.setProtocol(rs.getString(15));      
               pms.setOutServer(rs.getString(16)); 
               pms.setOutPort(rs.getInt(17)); 
               pms.setOutAuth(Utility.booleanFromString(rs.getString(18))); 
               pms.setInServer(rs.getString(19)); 
               pms.setInPort(rs.getInt(20)); 
               pms.setIsSSLIn(Utility.booleanFromString(rs.getString(21))); 
               pms.setIsSSLOut(Utility.booleanFromString(rs.getString(22))); 
               pms.setKeystorePath(rs.getString(23)); 
               pms.setKeystorePwd(rs.getString(24)); 
               pms.setInboxPath(rs.getString(25));  
               pms.setInboxActivation(Utility.booleanFromString(rs.getString(26))); 
               pms.setOutboxPath(rs.getString(27)); 
               pms.setOutboxActivation(Utility.booleanFromString(rs.getString(28))); 
               pms.setSentPath(rs.getString(29));   
               pms.setSentActivation(Utility.booleanFromString(rs.getString(30))); 
               pms.setDraftsPath(rs.getString(31));   
               pms.setDraftsActivation(Utility.booleanFromString(rs.getString(32))); 
               pms.setTrashPath(rs.getString(33));   
               pms.setTrashActivation(Utility.booleanFromString(rs.getString(34))); 
               pms.setIsSoftDelete(Utility.booleanFromString(rs.getString(35))); 
               pms.setIsPublic(Utility.booleanFromString(rs.getString(36))); 
               
               msa.setMailServer(pms);               
               
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return msa;
    }    

    /**
     *
     * insert public mail server configuration
     *
     * @param pms PublicMailServer
     * @return int
     * @throws EntityException
     */
    public int insertPubMailServer(MailServer pms)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          ps = connection.prepareStatement(Query.MS_INSERT_MAILSERVER);
          
          ps.setString(1, pms.getMailServerId());
          ps.setString(2, pms.getMailServerType());
          ps.setString(3, pms.getDescription());
          ps.setString(4, pms.getProtocol());
          ps.setString(5, pms.getOutServer());
          ps.setInt   (6, pms.getOutPort());
          ps.setString(7, Utility.booleanToString(pms.getOutAuth()));
          ps.setString(8, pms.getInServer());
          ps.setInt   (9, pms.getInPort());
          ps.setString(10, Utility.booleanToString(pms.getIsSSLIn()));
          ps.setString(11, Utility.booleanToString(pms.getIsSSLOut()));
          ps.setString(12, pms.getKeystorePath());
          ps.setString(13, pms.getKeystorePwd());
          
          ps.setString(14, "Inbox");
          ps.setString(15, "y");
          ps.setString(16, pms.getOutboxPath());
          ps.setString(17, Utility.booleanToString(pms.getOutboxActivation()));
          ps.setString(18, pms.getSentPath());
          ps.setString(19, Utility.booleanToString(pms.getSentActivation()));
          ps.setString(20, pms.getDraftsPath());
          ps.setString(21, Utility.booleanToString(pms.getDraftsActivation()));
          ps.setString(22, pms.getTrashPath());
          ps.setString(23, Utility.booleanToString(pms.getTrashActivation()));
          
          ps.setString(24, Utility.booleanToString(pms.getIsSoftDelete()));
          
          ps.setString(25, Utility.booleanToString(true));

          result = ps.executeUpdate();

          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;

    }
    
    /**
     *
     * delete mail server 
     *
     * @param mailServerId String
     * @return int
     * @throws EntityException
     */
    public int deletePubMailServer(String mailServerId)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;
       
       if (log.isTraceEnabled()) {
           log.trace("ConsoleDAO - deletePubMailServer " + mailServerId);
       }                

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          ps = connection.prepareStatement(Query.MS_DELETE_MAILSERVER);

          ps.setString(1, mailServerId);

          result = ps.executeUpdate();

          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;

    }
    
    /**
     *
     * update public mail server
     *
     * @param pms PublicMailServer
     * @return int
     * @throws EntityException
     */
    public int updatePubMailServer(MailServer pms)
    throws EntityException {
        
       Connection connection = null;
       PreparedStatement  ps = null;
       int result            = 0;

       try {

          DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);

          connection = dataSource.getConnection();
         
          
          // update the mail server
          ps = connection.prepareStatement(Query.MS_UPDATE_MAILSERVER);
          
          ps.setString(1, pms.getMailServerType());
          ps.setString(2, pms.getDescription());
          ps.setString(3, pms.getProtocol());          
          ps.setString(4, pms.getOutServer());
          ps.setInt   (5, pms.getOutPort());
          ps.setString(6, Utility.booleanToString(pms.getOutAuth()));
          ps.setString(7, pms.getInServer());
          ps.setInt   (8, pms.getInPort());          
          ps.setString(9, Utility.booleanToString(pms.getIsSSLIn()));
          ps.setString(10, Utility.booleanToString(pms.getIsSSLOut()));
          ps.setString(11, pms.getKeystorePath());
          ps.setString(12, pms.getKeystorePwd());
          
          ps.setString(13, "Inbox");
          ps.setString(14, "y");
          ps.setString(15, pms.getOutboxPath());
          ps.setString(16, Utility.booleanToString(pms.getOutboxActivation()));
          ps.setString(17, pms.getSentPath());
          ps.setString(18, Utility.booleanToString(pms.getSentActivation()));
          ps.setString(19, pms.getDraftsPath());
          ps.setString(20, Utility.booleanToString(pms.getDraftsActivation()));
          ps.setString(21, pms.getTrashPath());
          ps.setString(22, Utility.booleanToString(pms.getTrashActivation()));
          
          ps.setString(23, Utility.booleanToString(pms.getIsSoftDelete()));
          
          ps.setString(24, Utility.booleanToString(true));
          
          ps.setString(25, pms.getMailServerId());

          result = ps.executeUpdate();

          
          // update all users that use this mail server
          
          ps = connection.prepareStatement(Query.MS_ACCOUNT_UPDATE_MAILSERVER_4_USERS);                   
          
          ps.setString(1, pms.getMailServerType());
          ps.setString(2, pms.getDescription());
          ps.setString(3, pms.getProtocol());          
          ps.setString(4, pms.getOutServer());
          ps.setInt   (5, pms.getOutPort());
          ps.setString(6, Utility.booleanToString(pms.getOutAuth()));
          ps.setString(7, pms.getInServer());
          ps.setInt   (8, pms.getInPort());          
          ps.setString(9, Utility.booleanToString(pms.getIsSSLIn()));
          ps.setString(10, Utility.booleanToString(pms.getIsSSLOut()));
          ps.setString(11, pms.getKeystorePath());
          ps.setString(12, pms.getKeystorePwd());
          ps.setString(13, Utility.booleanToString(pms.getIsSoftDelete()));
          ps.setString(14, Utility.booleanToString(true));
          
          ps.setString(15, pms.getMailServerId());

          result = ps.executeUpdate();
                    
          ps.close() ;
          ps = null  ;

       } catch (Exception e) {
           throw new EntityException(e);
       } finally {
           DBTools.close(connection, ps, null);
       }

       return result;

    }
    
    
    /**
     *
     * get public mail servers
     *
     * @return list with all the mail server in the filter ArrayList
     * @throws EntityException
     */
    public MailServer[] getPubMailServers(Clause clause) throws EntityException {
        
        ArrayList msl          = new ArrayList();
        MailServer[] mslOut    = null;
        Connection connection  = null;
        PreparedStatement stmt = null;
        ResultSet rs           = null;
        MailServer pms   = null;

        try {

           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
                      
           String dbProductName = connection.getMetaData().getDatabaseProductName();           
           SQLHelperClause sqlHelperClause = new SQLHelperClause(dbProductName);
           
           String query = Query.MS_LIST_MAILSERVER;
           if (clause != null){
               PreparedWhere where = sqlHelperClause.getPreparedWhere(clause);
               if ( (clause != null) && (! (clause instanceof AllClause))) {
                   if (where.sql.length() > 0){
                       query += " where " + where.sql;
                   }
               }    
               stmt = connection.prepareStatement(query);
               if ( (clause != null) && (! (clause instanceof AllClause))) {
                   for (int i=0; i<where.parameters.length; ++i) {
                       stmt.setObject(i+1, where.parameters[i]);
                   }
               }               
           } else {
               stmt = connection.prepareStatement(query);
           }
                                 
           rs = stmt.executeQuery();
                      
           while(rs.next()) {
               
               pms = new MailServer();
               
               pms.setMailServerId(rs.getString(1));                
               pms.setMailServerType(rs.getString(2));              
               pms.setDescription(rs.getString(3));                 
               pms.setProtocol(rs.getString(4));                    
               pms.setIsPublic(true);                
               pms.setOutServer(rs.getString(5));                   
               pms.setOutPort(rs.getInt(6));                        
               pms.setOutAuth(Utility.booleanFromString(rs.getString(7))); 
               pms.setInServer(rs.getString(8));                    
               pms.setInPort(rs.getInt(9));                         
               pms.setIsSSLIn(Utility.booleanFromString(rs.getString(10)));  
               pms.setIsSSLOut(Utility.booleanFromString(rs.getString(11)));  
               pms.setKeystorePath(rs.getString(12));               
               pms.setKeystorePwd(rs.getString(13));  
               
               pms.setInboxPath(rs.getString(14));                  
               pms.setInboxActivation(Utility.booleanFromString(rs.getString(15)));  
               pms.setOutboxPath(rs.getString(16));                 
               pms.setOutboxActivation(Utility.booleanFromString(rs.getString(17)));  
               pms.setSentPath(rs.getString(18));                   
               pms.setSentActivation(Utility.booleanFromString(rs.getString(19))); 
               pms.setDraftsPath(rs.getString(20));                 
               pms.setDraftsActivation(Utility.booleanFromString(rs.getString(21))); 
               pms.setTrashPath(rs.getString(22));                 
               pms.setTrashActivation(Utility.booleanFromString(rs.getString(23))); 
               
               pms.setIsSoftDelete(Utility.booleanFromString(rs.getString(24)));     
               pms.setIsPublic(Utility.booleanFromString(rs.getString(25)));     
               
               msl.add(pms);
               
           }

           mslOut = (MailServer[])msl.toArray(new MailServer[0]) ;
           
           stmt.close() ;
           stmt = null  ;
           
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, stmt, null);
        }

        return mslOut;
    }
    
    /**
     *
     * get public mail server
     *
     * @param mailServerId String
     * @return all the info about the public mail server MailServer
     * @throws EntityException
     */
    public MailServer getPubMailServer(String mailServerId) 
    throws EntityException {
        
        MailServer pms        = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.MS_GET_MAILSERVER);

           ps.setString (1, mailServerId ) ;

           rs = ps.executeQuery();
           
           while(rs.next()) {
               
               pms = new MailServer();
               
               pms.setMailServerId(rs.getString(1));                
               pms.setMailServerType(rs.getString(2));              
               pms.setDescription(rs.getString(3));                 
               pms.setProtocol(rs.getString(4));                    
               pms.setOutServer(rs.getString(5));                   
               pms.setOutPort(rs.getInt(6));                        
               pms.setOutAuth(Utility.booleanFromString(rs.getString(7))); 
               pms.setInServer(rs.getString(8));                    
               pms.setInPort(rs.getInt(9));                         
               pms.setIsSSLIn(Utility.booleanFromString(rs.getString(10))); 
               pms.setIsSSLOut(Utility.booleanFromString(rs.getString(11))); 
               pms.setKeystorePath(rs.getString(12));               
               pms.setKeystorePwd(rs.getString(13));                
               pms.setInboxPath(rs.getString(14));                  
               pms.setInboxActivation(Utility.booleanFromString(rs.getString(15)));                        
               pms.setOutboxPath(rs.getString(16));                 
               pms.setOutboxActivation(Utility.booleanFromString(rs.getString(17)));                       
               pms.setSentPath(rs.getString(18));                   
               pms.setSentActivation(Utility.booleanFromString(rs.getString(19)));  
               pms.setDraftsPath(rs.getString(20));                 
               pms.setDraftsActivation(Utility.booleanFromString(rs.getString(21))); 
               pms.setTrashPath(rs.getString(22));                 
               pms.setTrashActivation(Utility.booleanFromString(rs.getString(23)));
               pms.setIsSoftDelete(Utility.booleanFromString(rs.getString(24)));   
               pms.setIsPublic(Utility.booleanFromString(rs.getString(25))); 
               
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return pms;
        
    }
    
    
    /**
     *
     * check if there are related users to the specific Mail Server
     *
     * @param mailServerId String
     * @return number of user for the specific mail server
     * @throws EntityException
     */
    public int checkUsersForMailServer(String mailServerId)
    throws EntityException {
        int resultNum         = 0;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.MS_ACCOUNT_CHECK_USER);
           
           ps.setString (1, mailServerId) ;

           rs = ps.executeQuery();
           
           while(rs.next()) {
               resultNum = rs.getInt(1);
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return resultNum;
    }
    
    
    //---------------------------------------------------------- Private Methods
    
    /**
     *
     * get userid
     *
     * @param username ds-server user String
     * @return serId
     * @throws EntityException
     */
    private String getAccountId(String username)
    throws EntityException {
        String userId = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.MS_ACCOUNT_GET_USER_ID);
           
           ps.setString (1, username ) ;

           rs = ps.executeQuery();
           
           while(rs.next()) {
               userId = (rs.getString(1));    
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return userId;
    }

    
    /**
     *
     * get userid
     *
     * @param username ds-server user String
     * @return serId
     * @throws EntityException
     */
    private String getUsername(String accountId)
    throws EntityException {
        String username = null;
        Connection connection = null;
        PreparedStatement  ps = null;
        ResultSet rs          = null;

        try {

           DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
           
           connection = dataSource.getConnection();
           
           ps = connection.prepareStatement(Query.MS_ACCOUNT_GET_USER_NAME);
           
           ps.setString (1, accountId ) ;

           rs = ps.executeQuery();
           
           while(rs.next()) {
               username = (rs.getString(1));    
           }

           ps.close() ;
           ps = null  ;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        return username;
    }
    
}
