/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.console.manager;


import com.funambol.email.exception.EntityException;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.MailServer;
import com.funambol.email.util.Def;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.filter.Clause;
import com.funambol.email.console.dao.ConsoleDAO;
import com.funambol.email.engine.source.EmailConnectorConfig;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.model.SyncItemInfoAdmin;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.util.TokenUtility;
import com.funambol.email.util.Utility;
import com.funambol.framework.tools.DataSourceTools;
import com.funambol.server.tools.id.DBIDGenerator;
import com.funambol.server.tools.id.DBIDGeneratorException;
import com.funambol.server.tools.id.DBIDGeneratorFactory;
import javax.naming.NamingException;

/**
 *
 * @version $Id: ConsoleManager.java,v 1.28 2007/02/05 13:30:14 gbmiglia Exp $
 */
public class ConsoleManager {


    // ------------------------------------------------------------ Private data

    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    /**     */
    protected static DBIDGenerator idMailServerSpace = null;

    /**     */
    protected static DBIDGenerator idUserSpace = null;
    
    /**     */
    protected ConsoleDAO cdao = null;

    /**     */
    private String jndiDataSourceName = null;
            
    /**     */
    private String encryptionKey   = null;
    
    // -------------------------------------------------------------- Properties


    /**
     *
     */
    public ConsoleManager(){
                
        try {
            this.jndiDataSourceName = EmailConnectorConfig.getConfigInstance().getDataSource();
        } catch (EmailConfigException e) {
            log.error("Error Getting Connector Parameters ", e);
        }        
                
        this.encryptionKey = TokenUtility.getToken();
        
        try {
            idMailServerSpace   = DBIDGeneratorFactory.getDBIDGenerator(
                    Def.ID_COUNTER_MAILSERVER,
                    DataSourceTools.lookupDataSource(this.jndiDataSourceName));
            idUserSpace = DBIDGeneratorFactory.getDBIDGenerator(
                    Def.ID_COUNTER_USER,
                    DataSourceTools.lookupDataSource(this.jndiDataSourceName));
        } catch (NamingException e) {
             log.error("Error Getting ID Generator ", e);
        } catch (Exception e) {
             log.error("Error Getting ID Generator ", e);
        }
        
        cdao = new ConsoleDAO(this.jndiDataSourceName, this.encryptionKey);
    }
    

    // -----------------------------------------------------------Public Methods

    
    /**
     *
     * insert user and the mail server configuration
     *
     * @param msa MailServerAccount
     * @return int
     * @throws EntityException
     */
    public int insertUser(MailServerAccount msa)
    throws EntityException {
        int result = 0;
        try {
                                        
            if (msa.getMailServer().getMailServerId() == null ||
                msa.getMailServer().getMailServerId().equals("") ){

                msa.getMailServer().setMailServerId(""+this.idMailServerSpace.next());

            } else {

                String msId = msa.getMailServer().getMailServerId();
                
                if (msa.getMailServer().getIsPublic()) {

                    MailServer msFromDB = cdao.getPubMailServer(msId);

                    // replace the imput info with the db info
                    msa.getMailServer().setMailServerId(msId);
                    msa.getMailServer().setMailServerType(msFromDB.getMailServerType());
                    msa.getMailServer().setDescription(msFromDB.getDescription());
                    msa.getMailServer().setIsSoftDelete(msFromDB.getIsSoftDelete());
                    msa.getMailServer().setIsPublic(msFromDB.getIsPublic());
                    msa.getMailServer().setProtocol(msFromDB.getProtocol());
                    msa.getMailServer().setOutServer(msFromDB.getOutServer());
                    msa.getMailServer().setOutPort(msFromDB.getOutPort());
                    msa.getMailServer().setOutAuth(msFromDB.getOutAuth());
                    msa.getMailServer().setInServer(msFromDB.getInServer());
                    msa.getMailServer().setInPort(msFromDB.getInPort());
                    msa.getMailServer().setIsSSLIn(msFromDB.getIsSSLIn());
                    msa.getMailServer().setIsSSLOut(msFromDB.getIsSSLOut());
                    msa.getMailServer().setKeystorePath(msFromDB.getKeystorePath());
                    msa.getMailServer().setKeystorePwd(msFromDB.getKeystorePwd());
                    msa.getMailServer().setInboxPath("Inbox");
                    msa.getMailServer().setInboxActivation(Utility.booleanFromString("y"));
                    msa.getMailServer().setOutboxPath(msFromDB.getOutboxPath());
                    msa.getMailServer().setOutboxActivation(msFromDB.getOutboxActivation());
                    msa.getMailServer().setSentPath(msFromDB.getSentPath());
                    msa.getMailServer().setSentActivation(msFromDB.getSentActivation());
                    msa.getMailServer().setDraftsPath(msFromDB.getDraftsPath());
                    msa.getMailServer().setDraftsActivation(msFromDB.getDraftsActivation());
                    msa.getMailServer().setTrashPath(msFromDB.getTrashPath());
                    msa.getMailServer().setTrashActivation(msFromDB.getTrashActivation()); 

                }

            }
                                                                 
            // check listener ID
            if (msa.getListenerId() == null || 
                msa.getListenerId().equals("")){
                msa.setListenerId("1");
            } 
            
            // from UI we get the minutes but in the DB we save the seconds
            int min = msa.getRefreshTime();
            msa.setRefreshTime(Utility.setSeconds(min));
            
            // check serverType
            if (msa.getMailServer().getMailServerType() == null ||
                msa.getMailServer().getMailServerType().equals("")){
                msa.getMailServer().setMailServerType(Def.SERVER_TYPE_OTHER);
            } 
            
            // check description
            if (msa.getMailServer().getDescription() == null ||
                msa.getMailServer().getDescription().equals("")){
                msa.getMailServer().setDescription(Def.SERVER_TYPE_OTHER);
            }

            
            // check folder names
            if (msa.getMailServer().getOutboxPath() == null ||
                msa.getMailServer().getOutboxPath().equals("")){
                msa.getMailServer().setOutboxPath(Def.FOLDER_OUTBOX_ENG);
                msa.getMailServer().setOutboxActivation(true);
            }
            
            if (msa.getMailServer().getSentPath() == null ||
                msa.getMailServer().getSentPath().equals("")){
                if (msa.getMailServer().getMailServerType().equals(Def.SERVER_TYPE_EXCHANGE)){
                    msa.getMailServer().setOutboxPath(Def.FOLDER_SENT_ENG_EXC);
                } else {
                    msa.getMailServer().setOutboxPath(Def.FOLDER_SENT_ENG);
                }
            }
            
            if (msa.getMailServer().getDraftsPath() == null ||
                msa.getMailServer().getDraftsPath().equals("")){
                msa.getMailServer().setDraftsPath(Def.FOLDER_DRAFTS_ENG);
            }
            
            if (msa.getMailServer().getTrashPath() == null ||
                msa.getMailServer().getTrashPath().equals("")){
                if (msa.getMailServer().getMailServerType().equals(Def.SERVER_TYPE_EXCHANGE)){
                    msa.getMailServer().setOutboxPath(Def.FOLDER_TRASH_ENG_EXC);
                } else {
                    msa.getMailServer().setOutboxPath(Def.FOLDER_TRASH_ENG);
                }
            }                        
            
            // check user 
            boolean toAdd = toInsert(msa.getUsername());
            
            // insert or update
            if (toAdd){
               msa.setAccountId(""+this.idUserSpace.next());
               result = cdao.insertUser(msa);
            } else {
               result = cdao.updateUser(msa);
            }
            
        } catch (DBIDGeneratorException ee){
            throw new EntityException("Error handling ID ", ee);
        } catch (EntityException ee){
            throw new EntityException("Error updating Account", ee);
        }      
        return result;       
    }
    
    /**
     *
     * delete user 
     *
     * @param accountID String
     * @return int
     * @throws EntityException
     */
    public int deleteUser(String accountID) throws EntityException {
        int result = 0;
        try {
            result = cdao.deleteUser(accountID);
        } catch (EntityException ee){
            throw new EntityException("Error deleting Account ", ee);
        }
        return result;
    }
    
    /**
     *
     * disable account 
     *
     * @param accountID String
     * @return int
     * @throws EntityException
     */
    public int disableUser(String accountID) throws EntityException {
        int result = 0;
        try {
            result = cdao.disableUser(accountID);
        } catch (EntityException ee){
            throw new EntityException("Error disabling Account ", ee);
        }
        return result;
    }
    
    /**
     *
     * enable account 
     *
     * @param accountID String
     * @return int
     * @throws EntityException
     */
    public int enableUser(String accountID) throws EntityException {
        int result = 0;
        try {
            result = cdao.enableUser(accountID);
        } catch (EntityException ee){
            throw new EntityException("Error enabling Account ", ee);
        }
        return result;
    }
    
    /**
     *
     * update user regarding the mail server connection
     *
     * @param msa MailServerAccount
     * @return int
     * @throws EntityException
     */
    public int updateUser(MailServerAccount msa)  throws EntityException {
        
        int result = 0;
        
        try {

            if (msa.getMailServer().getMailServerId() == null ||
                msa.getMailServer().getMailServerId().equals("") ){

                // this case happens when there is a swith from public to custom in the UI
                msa.getMailServer().setMailServerId(""+this.idMailServerSpace.next());

            } else {

                // update custom or update public
                String msId = msa.getMailServer().getMailServerId();
                
                if (msa.getMailServer().getIsPublic()) {

                    // update public; get info from DB
                    MailServer msFromDB = cdao.getPubMailServer(msId);

                    // replace the imput info with the db info
                    msa.getMailServer().setMailServerId(msId);
                    msa.getMailServer().setMailServerType(msFromDB.getMailServerType());
                    msa.getMailServer().setDescription(msFromDB.getDescription());
                    msa.getMailServer().setIsSoftDelete(msFromDB.getIsSoftDelete());
                    msa.getMailServer().setIsPublic(msFromDB.getIsPublic());
                    msa.getMailServer().setProtocol(msFromDB.getProtocol());
                    msa.getMailServer().setOutServer(msFromDB.getOutServer());
                    msa.getMailServer().setOutPort(msFromDB.getOutPort());
                    msa.getMailServer().setOutAuth(msFromDB.getOutAuth());
                    msa.getMailServer().setInServer(msFromDB.getInServer());
                    msa.getMailServer().setInPort(msFromDB.getInPort());
                    msa.getMailServer().setIsSSLIn(msFromDB.getIsSSLIn());
                    msa.getMailServer().setIsSSLOut(msFromDB.getIsSSLOut());
                    msa.getMailServer().setKeystorePath(msFromDB.getKeystorePath());
                    msa.getMailServer().setKeystorePwd(msFromDB.getKeystorePwd());
                    
                } else {
                    
                    // update custom; get info from input
                    
                    
                }

            }
                                       
            // check listener ID
            if (msa.getListenerId() == null || 
                msa.getListenerId().equals("")){
                msa.setListenerId("1");
            } 
            
            // from UI we get the minutes but in the DB we save the seconds
            int min = msa.getRefreshTime();
            msa.setRefreshTime(Utility.setSeconds(min));
            
            // check serverType
            if (msa.getMailServer().getMailServerType() == null ||
                msa.getMailServer().getMailServerType().equals("")){
                msa.getMailServer().setMailServerType(Def.SERVER_TYPE_OTHER);
            }    
            
            // check description
            if (msa.getMailServer().getDescription() == null ||
                msa.getMailServer().getDescription().equals("")){
                msa.getMailServer().setDescription(Def.SERVER_TYPE_OTHER);
            }
            
            // check folder names
            if (msa.getMailServer().getOutboxPath() == null ||
                msa.getMailServer().getOutboxPath().equals("")){
                msa.getMailServer().setOutboxPath(Def.FOLDER_OUTBOX_ENG);
            }
            if (msa.getMailServer().getSentPath() == null ||
                msa.getMailServer().getSentPath().equals("")){
                msa.getMailServer().setSentPath(Def.FOLDER_SENT_ENG);
            }
            if (msa.getMailServer().getDraftsPath() == null ||
                msa.getMailServer().getDraftsPath().equals("")){
                msa.getMailServer().setDraftsPath(Def.FOLDER_DRAFTS_ENG);
            }
            if (msa.getMailServer().getTrashPath() == null ||
                msa.getMailServer().getTrashPath().equals("")){
                msa.getMailServer().setTrashPath(Def.FOLDER_TRASH_ENG);
            }                                 
                        
            // update
            result = cdao.updateUser(msa);
            
        } catch (DBIDGeneratorException ee){
            throw new EntityException("Error handling ID ", ee);
        } catch (EntityException ee){
            throw new EntityException("Error updating Account", ee);
        }   
        
        return result;     
        
    }
    
    
    /**
     *
     * get users in the filter
     *
     * @param clause Clause
     * @return list with all the user in the filter ArrayList
     * @throws EntityException
     */
    public MailServerAccount[] getUsers(Clause clause) throws EntityException {        
        
        MailServerAccount[] msal = null;
        try {
            msal = cdao.getUsers(clause);
        } catch (EntityException ee){
            throw new EntityException("Error getting Account List", ee);
        }
        return msal;
        
    }
    
    /**
     *
     * get user
     *
     * @param username String
     * @return all the info about the user and the related mail server MailServerAccount
     * @throws EntityException
     */
    public MailServerAccount getUser(String username) throws EntityException {
        MailServerAccount msa = null;
        try {            
            msa = cdao.getUser(username);                                   
        } catch (EntityException ee){
            throw new EntityException("Error getting Account ", ee);
        }        
        return msa;
    }
    
    /**
     *
     * get user
     *
     * @param accountID String
     * @return all the info about the user and the related mail server MailServerAccount
     * @throws EntityException
     */
    public MailServerAccount getUserFromID(String accountID)
    throws EntityException {
        MailServerAccount msa = null;
        try {
            msa = cdao.getUserFromID(accountID);            
        } catch (EntityException ee){
            throw new EntityException("Error getting Account ", ee);
        }        
        return msa;
    }
    

    /**
     *
     * insert public mail server configuration
     *
     * @param ms PublicMailServer
     * @return int
     * @throws EntityException
     */
    public int insertPubMailServer(MailServer ms) throws EntityException {
        
        int result = 0;
        
        try {
            
            ms.setMailServerId(""+this.idMailServerSpace.next());
            
            // check serverType
            if (ms.getMailServerType() == null || ms.getMailServerType().equals("")){
                ms.setMailServerType(Def.SERVER_TYPE_OTHER);
            } 
            
            // check description
            if (ms.getDescription() == null || ms.getDescription().equals("")){
                ms.setDescription(Def.SERVER_TYPE_OTHER);
            }

            
            // check folder names
            if (ms.getOutboxPath() == null || ms.getOutboxPath().equals("")){
                ms.setOutboxPath(Def.FOLDER_OUTBOX_ENG);
                ms.setOutboxActivation(true);
            }
            
            if (ms.getSentPath() == null || ms.getSentPath().equals("")){
                if (ms.getMailServerType().equals(Def.SERVER_TYPE_EXCHANGE)){
                    ms.setOutboxPath(Def.FOLDER_SENT_ENG_EXC);
                } else {
                    ms.setOutboxPath(Def.FOLDER_SENT_ENG);
                }
            }
            
            if (ms.getDraftsPath() == null || ms.getDraftsPath().equals("")){
                ms.setDraftsPath(Def.FOLDER_DRAFTS_ENG);
            }
            
            if (ms.getTrashPath() == null || ms.getTrashPath().equals("")){
                if (ms.getMailServerType().equals(Def.SERVER_TYPE_EXCHANGE)){
                    ms.setOutboxPath(Def.FOLDER_TRASH_ENG_EXC);
                } else {
                    ms.setOutboxPath(Def.FOLDER_TRASH_ENG);
                }
            }
            
            result = cdao.insertPubMailServer(ms);
            
        } catch (DBIDGeneratorException ee){
            throw new EntityException("Error handling ID ", ee);
        } catch (EntityException ee){
            throw new EntityException("Error inserting Mail Server ", ee);
        }        
        return result;
    }
    
    /**
     *
     * delete mail server 
     *
     * @param mailServerId String
     * @return int
     * @throws EntityException
     */
    public int deletePubMailServer(String mailServerId)
    throws EntityException {
        int result = 0;
        try {
            if (noUsers(mailServerId)){
                result = cdao.deletePubMailServer(mailServerId);
            } else {
                // There are some Users related to this Mail Server 
                result = Def.ERR_SERVER_DELETION;
            }
        } catch (EntityException ee){
            throw new EntityException("Error deleting MailServer ", ee);
        }
        return result;
    }
    
    /**
     *
     * update public mail server
     *
     * @param ms PublicMailServer
     * @return int
     * @throws EntityException
     */
    public int updatePubMailServer(MailServer ms) throws EntityException {
        
        int result = 0;
        
        try {
            
            if (ms.getMailServerId() == null || ms.getMailServerId().equals("") ){

                throw new EntityException("Error: Mail Server ID is null in an update operation");

            } else {
                
                // check serverType
                if (ms.getMailServerType() == null || ms.getMailServerType().equals("")){
                    ms.setMailServerType(Def.SERVER_TYPE_OTHER);
                } 

                // check description
                if (ms.getDescription() == null || ms.getDescription().equals("")){
                    ms.setDescription(Def.SERVER_TYPE_OTHER);
                }

                // check folder names
                if (ms.getOutboxPath() == null || ms.getOutboxPath().equals("")){
                    ms.setOutboxPath(Def.FOLDER_OUTBOX_ENG);
                    ms.setOutboxActivation(true);
                }
                if (ms.getSentPath() == null || ms.getSentPath().equals("")){
                    if (ms.getMailServerType().equals(Def.SERVER_TYPE_EXCHANGE)){
                        ms.setSentPath(Def.FOLDER_SENT_ENG_EXC);
                    } else {
                        ms.setSentPath(Def.FOLDER_SENT_ENG);
                    }
                }
                
                if (ms.getDraftsPath() == null || ms.getDraftsPath().equals("")){
                    ms.setDraftsPath(Def.FOLDER_DRAFTS_ENG);
                }
                
                if (ms.getTrashPath() == null || ms.getTrashPath().equals("")){
                    if (ms.getMailServerType().equals(Def.SERVER_TYPE_EXCHANGE)){
                        ms.setTrashPath(Def.FOLDER_TRASH_ENG_EXC);
                    } else {
                        ms.setTrashPath(Def.FOLDER_TRASH_ENG);
                    }
                }
            
            }
                        
            result = cdao.updatePubMailServer(ms);
            
        } catch (EntityException ee){
            throw new EntityException("Error updating Mail Server ", ee);
        }        
        return result;
    }
    
    
    /**
     *
     * get public mail servers
     *
     * @return list with all the mail server in the filter ArrayList
     * @throws EntityException
     */
    public MailServer[] getPubMailServers(Clause clause) throws EntityException {        
        MailServer[] msl = null;
        try {
            msl = cdao.getPubMailServers(clause);
        } catch (EntityException ee){
            throw new EntityException("Error getting Mail Server List", ee);
        }        
        return msl;
    }
    
    /**
     *
     * get public mail server
     *
     * @param mailServerId String
     * @return all the info about the public mail server MailServer
     * @throws EntityException
     */
    public MailServer getPubMailServer(String mailServerId) 
    throws EntityException {        
        MailServer pms        = null;
        try {
            pms = cdao.getPubMailServer(mailServerId);
        } catch (EntityException ee){
            throw new EntityException("Error getting Mail Server ", ee);
        }        
        return pms;        
    }
    
    /**
     *
     * get the cached info for the specified username
     *
     * @param username String
     * @param protocol String
     * @return array qith all the info
     * @throws EntityException
     */
    public SyncItemInfoAdmin[] getCachedInfo(String username, String protocol)
      throws EntityException {
        SyncItemInfoAdmin[] infos = null;
        try {
            infos = cdao.getCachedInfo(username, protocol);
        } catch (EntityException ee){
            throw new EntityException("Error getting Cache Info for Administration", ee);
        }        
        return infos;        
     
    }    
    
    
    /**
     *
     * check the account on the mail server
     * ok                           = 0
     * invalid protocol             = 1
     * invalid username or password = 2
     * cannection failed            = 3
     *
     * @param msa MailServerAccount
     * @return true if the user is a mail server account boolean
     * @throws EntityException
     */
    public int checkAccount(MailServerAccount msa) 
    throws EntityException {        
        
        int errorCode = Def.ERR_OK;
        
        try {
            
            if (msa != null){

                String login              = msa.getMsLogin();
                String password           = msa.getMsPassword();
                String host               = msa.getMailServer().getInServer();
                String protocol           = msa.getMailServer().getProtocol();
                int port                  = msa.getMailServer().getInPort();
                boolean isSSLIn           = msa.getMailServer().getIsSSLIn();
                String keystorePath       = msa.getMailServer().getKeystorePath();
                String keystorePassphrase = msa.getMailServer().getKeystorePwd();

                IMailServerWrapper mswf =
                       MailServerWrapperFactory.getMailServerWrapper(protocol);

                if (mswf != null){
                    mswf.checkUser(host, "" + port, login, password, 
                                   isSSLIn, keystorePath, keystorePassphrase);
                } else {
                   errorCode = Def.ERR_INVALID_PROTOCOL; 
                }
                
            } else {
                errorCode = Def.ERR_INVALID_INPUT;
            }
            
        } catch (EmailAccessException dae) {
            log.error("Error getting connection", dae);
            if (dae != null){
                 errorCode = ExceptionHandler(dae.getMessage());
            } else {
                 errorCode = Def.ERR_GENERIC_MAILSERVER; 
            }     
        }

        return errorCode;        
    }
        
    
    //---------------------------------------------------------- Private Methods
    
    
    /**
     * try to create acode frm description error
     */
    private int ExceptionHandler(String description) throws EntityException {    
           
        int errorCode = Def.ERR_CONNECTION_FAILED;
        
        // check "invalid username/password"
        int indexInvalid = description.indexOf("invalid");
        int indexUser    = description.indexOf("username");
        int indexPwd     = description.indexOf("password");
        if (indexInvalid != -1 && indexUser != -1 && indexPwd != -1){
           return Def.ERR_INVALID_CREDENTIALS;
        }
           
        // cehck "no domain specified or invalid domain specified" (i.e. mail.funambol.it)
        int indexDomain = description.indexOf("domain");
        if (indexDomain != -1){
            return Def.ERR_INVALID_DOMAIN;
        }
        
        // "Connect failed" (i.e. error in port)
        int indexConnect      = description.indexOf("Connect");
        int indexConnect_m    = description.indexOf("connect");
        int indexConnection   = description.indexOf("Connection");
        int indexConnection_m = description.indexOf("connection");
        int indexFailed       = description.indexOf("failed");
        int indexFailed_m     = description.indexOf("Failed");
        if ( (indexConnect != -1 || 
              indexConnect_m != -1 ||
              indexConnection != -1 || 
              indexConnection_m != -1)
             &&
             (indexFailed != -1  ||
              indexFailed_m != -1)
            ){
            errorCode = Def.ERR_CONNECTION_FAILED;
        }
        
        return errorCode;
        
    }
    
    
    /**
     *
     */
    private boolean noUsers(String mailServerId) throws EntityException {    
        boolean check = false;
        int userNum   = 0;
        try {
            userNum = cdao.checkUsersForMailServer(mailServerId);
            if (userNum == 0){
                check = true;
            } 
        } catch (EntityException ee){
            throw new EntityException("Error checking Users for the Mail Server ", ee);
        }        
        return check;
    }
    
    /**
     *
     */
    private boolean toInsert(String userId) throws EntityException {    
        boolean check = false;
        MailServerAccount msa = null;
        try {
            msa = cdao.getUser(userId);
            if (msa == null){
                check = true;
            } 
        } catch (EntityException ee){
            throw new EntityException("Error checking Users for the Mail Server ", ee);
        }        
        return check;
    }

}
