/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.source;


import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import com.funambol.email.util.Def;
import com.funambol.email.util.Query;
import com.funambol.email.util.Utility;
import com.funambol.email.exception.EntityException;
import com.funambol.email.model.SendResult;
import com.funambol.email.model.SyncItemInfo;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.tools.DBTools;
import com.funambol.framework.tools.DataSourceTools;

/**
 * Contains methods to access to the table with the state of the items.
 *
 * @version $Id: CacheDAO.java,v 1.4 2007/03/19 14:25:36 gbmiglia Exp $
 *
 */
public class CacheDAO {

    // --------------------------------------------------------------- Constants

    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);

    // ------------------------------------------------------------ Private data

    /**         */
    private String jndiDataSourceName;

    /**         */
    private DataSource dataSource;


    // ------------------------------------------------------------- Constructor

    /**
     *
     */
    private CacheDAO() {
    }

    /**
     *
     *
     * @param jndiDataSourceName String
     */
    public CacheDAO(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }


    // ---------------------------------------------------------- Public Methods

    /**
     *
     * @throws Exception
     */
    public void init() throws Exception {
        dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
    }

    /**
     *
     *
     * @return String
     */
    public String getJndiDataSourceName() {
        return jndiDataSourceName;
    }

    /**
     *
     * @param jndiDataSourceName String
     */
    public void setJndiDataSourceName(String jndiDataSourceName) {
        this.jndiDataSourceName = jndiDataSourceName;
    }



    /**
     * Delete local items
     *
     * @param sourceURI
     * @param principalId
     * @throws EntityException
     */
    public void deleteSentPop(String sourceURI, long principalId)
     throws EntityException {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isTraceEnabled()) {
            log.trace("ItemStateDAO delete fnbl_email_sentpop "
                       + principalId+ " - " +sourceURI);
        }
        try {
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_DELETE_POP);
            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principalId ) ;
            ps.executeUpdate();
        } catch (Exception e) {
            throw new EntityException("Error deleting fnbl_email_sentpop", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Delete local items
     *
     * @param username
     * @param protocol
     *
     * @throws Exception
     **/
    public void updateDeletedStatus(String username, String protocol)
     throws EntityException {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isTraceEnabled()) {
            log.trace("ItemStateDAO reset the D status in fnbl_email_inbox "
                       + username + " - " + protocol);
        }
        try {
            connection = dataSource.getConnection();
            
            ps = connection.prepareStatement(Query.INBOX_REMOVE_DELETED_STATUS);
            
            ps.setString  (1, username ) ;
            ps.setString  (2, protocol ) ;
            
            ps.executeUpdate();
            
        } catch (Exception e) {
            throw new EntityException("Error deleting fnbl_email_sentpop", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }    
    
    /**
     * Delete local items
     *
     *
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void deleteLocalItems(String sourceURI, long principal)
     throws EntityException {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isTraceEnabled()) {
            log.trace("ItemStateDAO delete start "+ principal+ " - " +sourceURI);
        }
        try {
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.CACHE_DELETE_LOCAL_ITEMS);
            ps.setString  (1, sourceURI ) ;
            ps.setLong    (2, principal ) ;
            ps.executeUpdate();
        } catch (Exception e) {
            throw new EntityException("Error deleting local items", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }

    /**
     * Delete local items
     *
     * @param GUID String
     * @param username String
     * @param protocol String
     * @throws EntityException
     */
    public void deleteLocalItem(String GUID, String username, String protocol)
     throws EntityException {

        Connection         connection  = null ;
        PreparedStatement  ps          = null ;

        if (log.isTraceEnabled()) {
            log.trace("ItemStateDAO delete id " + GUID + " - " + 
                    username+ " - " +protocol);
        }
        try {
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.INBOX_DELETE_EMAIL);
            ps.setString  (1, username ) ;
            ps.setString  (2, protocol ) ;
            ps.setString  (3, GUID ) ;
            ps.executeUpdate();
        } catch (Exception e) {
            throw new EntityException("Error deleting local item", e);
        } finally {
            DBTools.close(connection, ps, null);
        }
    }    
    
    /**
     * Update local items
     *
     * @param items map with all the mail server CrcSyncItemInfo
     * @param itemsSent ArrayList
     * @param invalidItems CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    public void updateLocalItems(HashMap items,
                                 ArrayList itemsSent,
                                 SyncItemInfo[] invalidItems,
                                 String sourceURI,
                                 long principal)
    throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("ItemStateDAO update start");
         }         
         
         this.deleteLocalItems (sourceURI, principal);
         
         this.addLocalItems (items, itemsSent, invalidItems, sourceURI, principal);
         
        if (log.isTraceEnabled()) {
            log.trace("ItemStateDAO update end");
         }
    }

    /**
     * Gets local items
     *
     *
     * @param sourceURI String
     * @param principalId String
     * @return map with CrcSyncItemInfo
     * @throws Exception
     */
    public HashMap getLocalItems (String sourceURI, long principalId)
        throws EntityException {

         Connection connection   = null;
         PreparedStatement ps    = null;
         ResultSet rs            = null;
         HashMap items           = null;
         String GUID             = null;
         long   lastCRC          = 0;
         String isInvalid        = null;
         String isInternal       = null;
         String isEmail          = null;
         String messageID        = null;
         java.util.Date headerDate = null;
         java.util.Date received = null;
         String subject          = null;
         String sender           = null;

         try {

             connection = dataSource.getConnection();

             items = new HashMap();

             ps = connection.prepareStatement(Query.CACHE_SELECT_ITEMS_BY_PRINCIPAL_BY_SOURCE);

             ps.setString(1, sourceURI);
             ps.setLong(2, principalId);

             rs = ps.executeQuery();

             while(rs.next()) {
                 
                 GUID      = rs.getString(1);
                 lastCRC   = rs.getLong(2);
                 isInvalid = rs.getString(3);
                 isInternal = rs.getString(4);
                 messageID  = rs.getString(5);
                 headerDate = Utility.UtcToDate(rs.getString(6));
                 received   = Utility.UtcToDate(rs.getString(7));
                 subject    = rs.getString(8);
                 sender     = rs.getString(9);                 
                 isEmail    = rs.getString(10);
                 
                 items.put(GUID, new SyncItemInfo(GUID, lastCRC, 
                                                  messageID, headerDate, 
                                                  received, subject, sender,
                                                  isInvalid, isInternal,  
                                                  isEmail, null));
             }

         } catch (Exception e) {
             throw new EntityException("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }
    
    
    /**
     * Gets local items
     *
     *
     * @param sourceURI String
     * @param principalId String
     * @return CrcSyncItemInfo[]
     * @throws Exception
     */
    public SyncItemInfo[] getInvalidItems (String sourceURI,
                                           long principalId)
        throws EntityException {

         Connection connection   = null;
         PreparedStatement ps    = null;
         ResultSet rs            = null;
         ArrayList rows          = null;
         SyncItemInfo[] items = null;

         try {

             connection = dataSource.getConnection();

             rows = new ArrayList();

             ps = connection.prepareStatement(Query.CACHE_SELECT_INVALID_ITEMS);

             ps.setString(1, sourceURI);
             ps.setLong(2, principalId);

             rs = ps.executeQuery();

             String guid = null;
             long lastCrc = 0;
             String messageID = null;
             java.util.Date headerDate = null;
             java.util.Date received = null;
             String subject = null;
             String sender = null;
             String invalid = null;
             String internal = null;
             String isemail = null;
                          
             while(rs.next()) {
                 
                 guid       = rs.getString(1);
                 lastCrc    = rs.getLong(2);
                 invalid    = rs.getString(3);
                 internal   = rs.getString(4);
                 messageID  = rs.getString(5);
                 headerDate = Utility.UtcToDate(rs.getString(6));
                 received   = Utility.UtcToDate(rs.getString(7));
                 subject    = rs.getString(8);
                 sender     = rs.getString(9);
                 isemail    = rs.getString(10);
                                                   
                 rows.add(new SyncItemInfo(guid, lastCrc, messageID, headerDate,
                                           received, subject, sender,
                                           invalid, internal, isemail, null));
                 
             }

             items = (SyncItemInfo[]) rows.toArray(new SyncItemInfo[0]);

         } catch (Exception e) {
             throw new EntityException("Error getting local items", e);
         } finally {
             DBTools.close(connection, ps, rs);
         }

         return items;
    }
    

    // --------------------------------------------------------- Private Methods

    /**
     * Lookups the datasource
     *
     * JBOSS
     * private String jndiDataSourceName = "java:jdbc/sync4j";
     *
     * TOMCAT
     * private String jndiDataSourceName = "java:comp/env/jdbc/sync4j";
     *
     * @throws Exception if a error occurs during lookup
     */
    /*
    private void lookupDataSource() throws Exception {

        InitialContext ctx = new InitialContext();
        if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: datasource lookup");
        }

        try {
          // try tomcat  i.e. java:comp/env/jdbc/sync4j
          dataSource = (DataSource) ctx.lookup("java:comp/env/"+jndiDataSourceName) ;
          if (log.isLoggable(Level.FINEST)) {
             log.finest("CachedSyncSource: datasource lookup done using java:comp/env/ prefix");
          }
        }
        catch (NamingException ne) {
          if (log.isLoggable(Level.FINEST)) {
              log.finest("CachedSyncSource: No tomcat datasource lookup!");
          }
          try {
              // try jboss i.e. java:jdbc/sync4j
              dataSource = (DataSource) ctx.lookup("java:"+jndiDataSourceName) ;
              if (log.isLoggable(Level.FINEST)) {
                 log.finest("CachedSyncSource: datasource lookup done using java: prefix");
              }
          }
          catch (Exception ex){
            if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No jboss datasource lookup!");
            }
            try {
                // try othser j2ee server jdbc/sync4j
                dataSource = (DataSource) ctx.lookup(""+jndiDataSourceName) ;
                if (log.isLoggable(Level.FINEST)) {
                   log.finest("CachedSyncSource: datasource lookup done using - prefix");
                }
            }
            catch (Exception e){
              if (log.isLoggable(Level.FINEST)) {
                log.finest("CachedSyncSource: No j2ee server datasource lookup!");
              }
              throw new Exception("Data source "+ jndiDataSourceName
                                  + " not found " + e.toString());
            }
          }
        }
    }
    */
    
    /**
     * Adds local items
     *
     *
     * @param items map with all the mail server CrcSyncItemInfo
     * @param itemsSent ArrayList
     * @param invalidItems CrcSyncItemInfo[]
     * @param sourceURI String
     * @param principal String
     * @throws Exception
     */
    private void addLocalItems(HashMap items,
                               ArrayList itemsSent,
                               SyncItemInfo[] invalidItems,
                               String sourceURI,
                               long principal) 
     throws EntityException {

         SendResult         sr            ;
         Connection         connection    = null;
         PreparedStatement  ps            = null;
         String             id            = null;
         String             GUID          = null;
         String             LUID          = null;
         String             isInvalid     = null;
         String             isInternal    = null;
         String             isEmail       = null;
         String             messageId     = null;
         java.util.Date     headerDate    = null;
         java.util.Date     received      = null;
         String             subject       = null;
         String             sender        = null;
         long               lastCrc       ;
         boolean            isSent        ;
         boolean            isPOP         ;
         

         try {

            connection = dataSource.getConnection();

            if (log.isTraceEnabled()) {
                log.trace("ItemStateDAO add start " + items.size());
            }

            int sentLen    = itemsSent.size();
            int invalidLen = invalidItems.length;

            
            Iterator values = items.values().iterator();
            
            SyncItemInfo scrcii = null;
            while (values.hasNext()) {
                
                scrcii = (SyncItemInfo)values.next();
                
                ps = connection.prepareStatement(Query.CACHE_ADD_LOCAL_ITEM);

                // get id
                id       = scrcii.getKey().getKeyAsString();
                
                // get CRC
                lastCrc  = scrcii.getLastCrc() ;

                // set the invalid flag
                isInvalid = null;
                for (int k=0; k<invalidLen; k++){
                    if (invalidItems[k].getKey().getKeyAsString().equals(id)) {
                        isInvalid = invalidItems[k].getInvalid();
                        break;
                    }
                }
                
                // change the id for Sent and Outbox folder
                if (id.startsWith(Def.FOLDER_SENT_ID)){
                    for (int y=0; y<sentLen; y++) {
                        sr = (SendResult) itemsSent.get(y);
                        GUID   = sr.getGUID();
                        isSent = sr.isSent();
                        if (GUID.equals(id) && isSent){
                            id = Utility.convertSentIDToOutboxID(id);
                            break;
                        }
                    }
                }

                // messageID
                messageId  = scrcii.getMessageID();
                
                // header date
                headerDate = scrcii.getHeaderDate();
                String headerDateS = null;
                if (headerDate != null){
                    headerDateS = Utility.UtcFromDate(headerDate);
                }
                
                // received
                received = scrcii.getHeaderReceived();
                String receivedS = null;
                if (received != null){
                    receivedS = Utility.UtcFromDate(received);
                }
                
                subject = scrcii.getSubject();
                
                sender = scrcii.getSender();
                
                // internal email
                isInternal = scrcii.getInternal();

                // flag for email or folder
                isEmail = scrcii.getIsEmail();
                
                
                ps.setString(1, id       );
                ps.setLong  (2, lastCrc  );
                ps.setString(3, isInvalid);
                ps.setString(4, isInternal);
                ps.setString(5, sourceURI);
                ps.setLong  (6, principal);
                ps.setString(7, messageId);
                ps.setString(8, headerDateS);                
                ps.setString(9, receivedS);                
                ps.setString(10, subject);                
                ps.setString(11, sender);                                
                ps.setString(12, isEmail);                

                ps.executeUpdate();

                ps.close() ;
                ps = null  ;
                
            }   
                        
         } catch (Exception e) {
             throw new EntityException("Error adding local items", e);
         } finally {
             DBTools.close(connection, ps, null);
         }
         
    }
    
}
