/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.source;


import java.io.Serializable;
import java.util.TimeZone;
import java.util.ArrayList;
import java.sql.Timestamp;
import java.util.HashMap;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.AbstractSyncSource;
import com.funambol.framework.engine.source.ContentType;
import com.funambol.framework.engine.source.FilterableSyncSource;
import com.funambol.framework.engine.source.SyncContext;
import com.funambol.framework.engine.source.SyncSourceException;
import com.funambol.framework.engine.source.SyncSourceInfo;
import com.funambol.framework.tools.beans.BeanInitializationException;
import com.funambol.framework.tools.beans.LazyInitBean;
import com.funambol.framework.engine.SyncItem;
import com.funambol.email.console.dao.ConsoleDAO;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.EmailConfigException;
import com.funambol.email.exception.EntityException;
import com.funambol.email.util.Def;
import com.funambol.email.model.SyncItemInfo;
import com.funambol.email.model.DefaultFolder;
import com.funambol.email.model.EmailFilter;
import com.funambol.email.model.MailServer;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.util.TokenUtility;
import com.funambol.email.util.Utility;
import com.funambol.framework.core.AlertCode;
import com.funambol.framework.engine.source.ServerWinsSyncSource;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.server.Sync4jUser;
import com.funambol.framework.server.store.PersistentStore;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.server.config.Configuration;
import java.security.Principal;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.tools.DataSourceTools;
import com.funambol.server.tools.id.DBIDGenerator;
import com.funambol.server.tools.id.DBIDGeneratorFactory;
import javax.naming.NamingException;

/**
 * <p>This class implements common methods of MultiEmailSyncSource<p/>
 *
 * version $Id: EmailSyncSource.java,v 1.71 2007/03/21 10:51:48 gbmiglia Exp $
 */
public class EmailSyncSource
        extends AbstractSyncSource
        implements FilterableSyncSource, ServerWinsSyncSource, Serializable, LazyInitBean {
    
    // --------------------------------------------------------------- Constants
    
    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    // -------------------------------------------------------------- Properties
    
    /**
     *
     */
    protected String jndiDataSourceName = null;
    
    /**
     * @return jndiDataSourceName String
     */
    public String getJndiDataSourceName(){
        return this.jndiDataSourceName;
    }
    
    /**
     *
     */
    protected String encryptionKey = null;
    
    /**
     *
     */
    protected String keystoreHome = null;
    
    
    /**
     *
     */
    protected String funambolSignature = Def.SIGNATURE;
    
    /**
     * @return funambolSignature String
     */
    public String getFunambolSignature(){
        return this.funambolSignature;
    }
    
    /**
     * @param _funambolSignature String
     */
    public void setFunambolSignature(String _funambolSignature){
        this.funambolSignature = _funambolSignature;
    }
    
    /**
     *
     */
    protected boolean saveOnlyHeader = false;
    
    
    /**
     *
     */
    protected long principalId ;
    
    /**
     *
     */
    protected int syncMode;
    
    /**
     *
     */
    protected String userName;
    
    /**
     *
     */
    protected String firstname;
    
    /**
     *
     */
    protected String lastname;
    
    /**
     * Property for sync session
     */
    protected TimeZone deviceTimeZone = null;
    
    /**
     * Property for sync session
     */
    protected String deviceCharset = null;
    
    /**
     * wrapper for the Mail Server
     */
    protected IMailServerWrapper MSWrapper = null;
    
    /**
     *
     * @return IMailServerWrapper
     */
    public IMailServerWrapper getMSWrapper(){
        return this.MSWrapper;
    }
    
    /**
     *
     */
    protected EmailFilter filter = new EmailFilter();
    
    /**
     * wrapper for the Email Helper
     */
    protected IEmailGeneric emailhelper = null;
    
    /**
     * wrapper for the Mail Server Account Wrapper
     */
    protected MailServerAccount mailServerAccount = null;
    
    /**
     *
     * @return MailServerAccount
     */
    public MailServerAccount getMailServerAccount(){
        return this.mailServerAccount;
    }
    
    /**
     *
     */
    protected CacheDAO cacheDao = null;
    
    /**
     *
     */
    protected static DBIDGenerator idSentSpace = null;
    
    /**
     *
     */
    protected static DBIDGenerator idFolderSpace = null;
    
    /**
     *
     */
    protected ArrayList newItemKeys     = null;
    
    /**
     *
     */
    protected ArrayList deletedItemKeys = null;
    
    /**
     *
     */
    protected ArrayList updatedItemKeys = null;
    
    /**
     * keep the items id and the result of send operation
     */
    protected ArrayList sentItems = new ArrayList();
    
    /**
     * list of local cached Items
     */
    protected HashMap localItems  = null;
    
    /**
     * list of server Items
     */
    protected HashMap serverItems = null;
    
    /**
     * save the invalid items during the cache updating procedure
     */
    protected SyncItemInfo[] invalidItems = null;
    
    /**
     * Should allMailbox sync?
     */
    protected boolean allMailboxActivation = false;
    
    /**
     * Should allMailbox sync?
     */
    protected boolean isSlowOrRefresh = false;
    
    /**
     *
     */
    protected DefaultFolder defaultFolder = new DefaultFolder(Def.FOLDER_INBOX_ENG,
            Def.FOLDER_OUTBOX_ENG,
            Def.FOLDER_SENT_ENG,
            Def.FOLDER_DRAFTS_ENG,
            Def.FOLDER_TRASH_ENG);
    
    /**
     *
     */
    {
        //this.setType("application/vnd.omads-email+xml,application/vnd.omads-folder+xml");
        
        this.setInfo(new SyncSourceInfo(
                new ContentType[]{
            new ContentType("application/vnd.omads-email+xml", "1.2"),
            new ContentType("application/vnd.omads-folder+xml", "1.2")
        }, 0));
        
    }
            
    // -------------------------------------------------------------- Constrctor

    /**
     *
     */
    public EmailSyncSource(){
    }

    // ---------------------------------------------------------- Public Methods


    /**
     *
     * @throws com.funambol.framework.tools.beans.BeanInitializationException
     */
    public void init() throws BeanInitializationException {

        try {
            this.jndiDataSourceName =
                    EmailConnectorConfig.getConfigInstance().getDataSource();
            this.keystoreHome       =
                    EmailConnectorConfig.getConfigInstance().getKeystoreHome();
            this.saveOnlyHeader     =
                    EmailConnectorConfig.getConfigInstance().getSaveOnlyHeader();
        } catch (EmailConfigException e) {
            log.error("Error Getting Connector Parameters ", e);
        }

        this.encryptionKey = TokenUtility.getToken();
        
        try {
            cacheDao = new CacheDAO(jndiDataSourceName);
            cacheDao.init();
        } catch (Exception e) {
            throw new BeanInitializationException("Error initializing ItemStateDAO", e);
        }

        try {
            idSentSpace   = DBIDGeneratorFactory.getDBIDGenerator(
                    Def.ID_COUNTER_SENT,
                    DataSourceTools.lookupDataSource(this.jndiDataSourceName)
                    );
            idFolderSpace = DBIDGeneratorFactory.getDBIDGenerator(
                    Def.ID_COUNTER_FOLDER,
                    DataSourceTools.lookupDataSource(this.jndiDataSourceName)
                    );
        } catch (NamingException e) {
            throw new BeanInitializationException("Error initializing ID Generator", e);
        }
    }


    /**
     * Begin Sync Method
     * before starting sync the method sets the filter
     * <p>
     * main folder filter
     * <ul>
     * <li> T D S O I
     * <li> 0 0 0 0 1 -->  1   inbox
     * <li> 0 0 0 1 1 -->  3   inbox + outbox
     * <li> 0 0 1 1 1 -->  7   inbox + outbox + sent
     * <li> 0 1 1 1 1 -->  15  inbox + outbox + sent + draft
     * <li> 1 1 1 1 1 -->  31  inbox + outbox + sent + draft + trash
     * </ul>
     * <p>
     * time filter (timeFilter)
     * <ul>
     * <li> null              --> all
     * <li> "i.e. 20-10-2005" --> filter
     * </ul>
     * <p>
     * size filter (sizeFilter / numbytes)
     * <ul>
     * <li> 000001 --> 1  FILTER_SIZE_H
     * <li> 000011 --> 3  FILTER_SIZE_H_PERC
     * <li> 000101 --> 5  FILTER_SIZE_H_B
     * <li> 001111 --> 15 FILTER_SIZE_H_B_PERC
     * <li> 010101 --> 21 FILTER_SIZE_H_B_A
     * <li> 111111 --> 63 FILTER_SIZE_H_B_A_PERC
     * </ul>
     *
     * @param context SyncContext
     * @throws SyncSourceException
     */
    public void beginSync(SyncContext context) throws SyncSourceException {

        if (log.isTraceEnabled()) {
            log.trace("EmailSyncSource - Start beginSync [" + this.getSourceURI() + "]");
        }

        // set common properties
        Principal principal  = context.getPrincipal();
        this.syncMode    = context.getSyncMode();
        this.principalId = ((Sync4jPrincipal) principal).getId();
        this.userName    = ((Sync4jPrincipal) principal).getUsername();
        Sync4jUser user  = ((Sync4jPrincipal) principal).getUser();
        String password  = user.getPassword();
        this.firstname   = user.getFirstname();
        this.lastname    = user.getLastname();

        try {
            // set device info
            setDeviceInfo(context);

            // set session mode
            setSyncType();

            // set the MailServerAccount Object
            setMailServerAccount();

            // set filter
            this.filter = HelperForFilter.setFilter(context,
                    this.mailServerAccount.getMaxEmailNumber(),
                    this.mailServerAccount.getMaxImapEmail(),
                    this.mailServerAccount.getMailServer().getInboxActivation(),
                    this.mailServerAccount.getMailServer().getOutboxActivation(),
                    this.mailServerAccount.getMailServer().getSentActivation(),
                    this.mailServerAccount.getMailServer().getDraftsActivation(),
                    this.mailServerAccount.getMailServer().getTrashActivation());

            // main folders
            setDefaultFolder(this.mailServerAccount.getMailServer());


        } catch (EntityException e) {
            throw new SyncSourceException("Error Setting Preliminary Info: " +
                    e.getMessage(), e);
        }


        String protocol = this.mailServerAccount.getMailServer().getProtocol();
        
        boolean isDraftsActive = this.mailServerAccount.getMailServer().getDraftsActivation();

        if (log.isTraceEnabled()) {
            log.trace(this.printIntro());
        }

        try {

            //
            this.MSWrapper = MailServerWrapperFactory.getMailServerWrapper(protocol);

            // imap or pop3 helper
            EmailSyncSourceWrapper essc = new EmailSyncSourceWrapper();
            essc.setEmailSyncSource(this);
            this.emailhelper = EmailFactory.getEmailSyncSource(protocol, essc);

            // open Mail Server connection
            this.MSWrapper.openConnection(this.mailServerAccount.getMailServer().getMailServerType(),
                    this.mailServerAccount.getMailServer().getOutServer(),
                    ""+this.mailServerAccount.getMailServer().getOutPort(),
                    this.mailServerAccount.getMailServer().getOutAuth(),
                    this.mailServerAccount.getMailServer().getInServer(),
                    ""+this.mailServerAccount.getMailServer().getInPort(),
                    getDefaultFolder(),
                    this.mailServerAccount.getMsLogin(),
                    this.mailServerAccount.getMsPassword(),
                    this.mailServerAccount.getMailServer().getIsSSLIn(),
                    this.mailServerAccount.getMailServer().getIsSSLOut(),
                    this.mailServerAccount.getMailServer().getKeystorePath(),
                    this.mailServerAccount.getMailServer().getKeystorePwd());

        } catch (EmailAccessException e) {
            try {
                this.MSWrapper.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }
            throw new SyncSourceException("Error Connecting Mail Server:" +
                    e.getMessage(), e);
        }


        // standard operation
        try {

            // insert the 5 main folder if needed
            insertDefaultFolder();

            // only for IMAP-based syncsource
            // check item without Message-ID in the draft folder
            // the draft created with a client i.e. outlook has no message-id
            if (protocol.equals(Def.PROTOCOL_IMAP) && isDraftsActive){
                checkMessageIDforDrafts();
            }
            
        } catch (EntityException e) {
            try {
                this.MSWrapper.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }
            throw new SyncSourceException("Error inserting Default Folder: " +
                    e.getMessage(), e);
        }


        // call cache
        try {
            cacheAtBegin();
        } catch (EntityException e) {
            try {
                this.MSWrapper.releaseConnection();
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }
            throw new SyncSourceException("Error Handling the Cache Info: " +
                    e.getMessage(), e);
        }

        if (log.isTraceEnabled()) {
            log.trace("EmailSyncSource - End beginSync [" + this.getSourceURI() + "]");
        }

    }

    /**
     * End Sync Method
     *
     * @throws SyncSourceException
     */
    public void endSync() throws SyncSourceException {

        if (log.isTraceEnabled()) {
            log.trace("EmailSyncSource - start EndSync [" + this.getSourceURI() + "]");
        }

        // reset the cache
        try {
            cacheAtEnd();
        } catch (EntityException e) {
            try {
                if (this.MSWrapper != null) {
                    this.MSWrapper.releaseConnection();
                }
            } catch (EmailAccessException e1){
                throw new SyncSourceException("Error closing store connection: " +
                        e1.getMessage(), e1);
            }
            throw new SyncSourceException("Error Handling the Cache Info:" +
                    e.getMessage(), e);
        }

        // close Mail Server connection
        try {
            if (this.MSWrapper != null) {
                this.MSWrapper.releaseConnection();
            }    
        } catch (EmailAccessException e) {
            throw new SyncSourceException("Error Closing Email:" +
                    e.getMessage(), e);
        }

        if (log.isTraceEnabled()) {
            log.trace("EmailSyncSource - end EndSync [" + this.getSourceURI() + "]");
        }

    }


    /**
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {
        return emailhelper.getAllSyncItemKeys();
    }


    /**
     * @param syncItemKey SyncItemKey
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey) throws SyncSourceException {
        SyncItem si = emailhelper.getSyncItemFromId(syncItemKey);
        if ( si == null){
            // remove from cache
            String GUID = syncItemKey.getKeyAsString();
    /*
    try {
    cacheDao.deleteLocalItem(GUID, this.userName,
                             this.mailServerAccount.getMailServer().getProtocol());
    } catch (EntityException ee){
        log.error("Error deleting Item in Ibox Cache; method getSyncItemFromId ", ee);
    }
     */
            // throws an exception
        }
        return si;
    }


    /**
     * @param syncItem SyncItem
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem) throws SyncSourceException {
        return emailhelper.getSyncItemKeysFromTwin(syncItem);
    }


    /**
     * @param syncItem SyncItem
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItem syncItem) throws SyncSourceException {
        return emailhelper.isSyncItemInFilterClause(syncItem);
    }


    /**
     * @param syncItemKey SyncItemKey
     * @return boolean
     * @throws SyncSourceException
     */
    public boolean isSyncItemInFilterClause(SyncItemKey syncItemKey) throws SyncSourceException {
        return emailhelper.isSyncItemInFilterClause(syncItemKey);

    }


    /**
     * @param syncItemKey SyncItemKey
     * @param s           Timestamp
     * @param engineSoftDelete boolean
     * @throws SyncSourceException
     */
    public void removeSyncItem(SyncItemKey syncItemKey,
            Timestamp s,
            boolean engineSoftDelete)
            throws SyncSourceException {
        emailhelper.removeSyncItem(syncItemKey, s, engineSoftDelete);
    }

    /**
     * @param operation  String
     * @param statusCode int
     * @param keys       SyncItemKey[]
     */
    public void setOperationStatus(String operation,
            int statusCode,
            SyncItemKey[] keys) {
        emailhelper.setOperationStatus(operation, statusCode, keys);
    }



    /**
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem updateSyncItem(SyncItem syncItem) throws SyncSourceException {
        return emailhelper.updateSyncItem(syncItem);
    }

    /**
     *
     * @param syncItem SyncItem
     * @return SyncItem
     * @throws SyncSourceException
     */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {
        return emailhelper.addSyncItem(syncItem);
    }



    /**
     *
     *
     * @param since Timestamp
     * @param to Timestamp
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp to)
    throws SyncSourceException {
        return emailhelper.getDeletedSyncItemKeys(since, to);
    }


    /**
     *
     *
     * @param since Timestamp
     * @param to Timestamp
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp to)
    throws SyncSourceException {
        return emailhelper.getNewSyncItemKeys(since, to);
    }

    /**
     *
     *
     * @param since Timestamp
     * @param to Timestamp
     * @return SyncItemKey[]
     * @throws SyncSourceException
     */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp to)
    throws SyncSourceException {
        return emailhelper.getUpdatedSyncItemKeys(since, to);
    }


    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @return char
     * @throws SyncSourceException
     */
    public char getSyncItemStateFromId(SyncItemKey syncItemKey)
    throws SyncSourceException {
        return emailhelper.getSyncItemStateFromId(syncItemKey);
    }


    //---------------------------------------------------------- Private Methods

    /**
     *
     */
    private void cacheAtBegin() throws EntityException {

        try {

            // refresh the inbox; remove the D status for Slow or refresh sync session
            if (this.syncMode == AlertCode.SLOW ||
                    this.syncMode == AlertCode.REFRESH_FROM_SERVER){
                this.cacheDao.updateDeletedStatus(this.userName,
                        this.mailServerAccount.getMailServer().getProtocol());
            }

            this.serverItems = this.emailhelper.getAllSyncItemInfo();

            if (this.syncMode == AlertCode.SLOW){

                this.cacheDao.deleteLocalItems(this.getSourceURI(),this.principalId);
                this.cacheDao.deleteSentPop(this.getSourceURI(),this.principalId);
                this.localItems = new HashMap();

            } else if (this.syncMode == AlertCode.REFRESH_FROM_SERVER){

                this.cacheDao.deleteLocalItems(this.getSourceURI(),this.principalId);
                this.cacheDao.deleteSentPop(this.getSourceURI(),this.principalId);
                this.localItems = new HashMap();
                setItemKeys();

            } else {

                this.localItems = this.cacheDao.getLocalItems(this.getSourceURI(), this.principalId);
                setItemKeys();

            }

            if (log.isTraceEnabled()) {
                String tmp = "";
                if (this.newItemKeys != null){
                    log.trace("NEW num:" + this.newItemKeys.size());
                    for (int i=0; i<this.newItemKeys.size(); i++){
                        tmp = tmp + " " + (this.newItemKeys.get(i));
                    }
                    log.trace("NEW List : [" + tmp + "]");
                } else {
                    log.trace("NO NEW ITEMS");
                }

                tmp = "";
                if (this.updatedItemKeys != null){
                    log.trace("UPDATED num:" + this.updatedItemKeys.size());
                    for (int i=0; i<this.updatedItemKeys.size(); i++){
                        tmp = tmp + " " + (this.updatedItemKeys.get(i));
                    }
                    log.trace("UPDATED List : [" + tmp + "]");
                } else {
                    log.trace("NO UPDATED ITEMS");
                }

                tmp = "";
                if (this.deletedItemKeys != null){
                    log.trace("DELETED num:" + this.deletedItemKeys.size());
                    for (int i=0; i<this.deletedItemKeys.size(); i++){
                        tmp = tmp + " " + (this.deletedItemKeys.get(i));
                    }
                    log.trace("DELETED List : [" + tmp + "]");
                } else {
                    log.trace("NO DELETED ITEMS");
                }
            }

        } catch (Exception e) {
            throw new EntityException("Error handling Items In Cache: " +
                    e.getMessage(), e);
        }

    }


    /**
     *
     */
    private void setSyncType() throws EntityException {

        try {
            if        (this.syncMode == AlertCode.SLOW){
                this.isSlowOrRefresh = true;
            } else if (this.syncMode == AlertCode.REFRESH_FROM_SERVER){
                this.isSlowOrRefresh = true;
            } else {
                this.isSlowOrRefresh = false;
            }
        } catch (Exception e){
            throw new EntityException("Error setting SyncSession Mode");
        }
    }

    /**
     *
     *
     * @param syncItemKey SyncItemKey
     * @return char
     * @throws SyncSourceException
     */
    private void setItemKeys() throws EntityException {

        if (this.filter.getFilterType().equalsIgnoreCase(Def.FILTER_TYPE_INC)){
            // inclusive
            if (this.filter.getId() != null){
                // id filter
                String idfilter = this.filter.getId();
                this.newItemKeys     = new ArrayList();
                this.updatedItemKeys = HelperForGetter.setUpdatedIDFilter(
                        idfilter, this.serverItems);
                this.deletedItemKeys = new ArrayList();
            } else {
                if (this.filter.getTime() != null) {
                    // time filter
                    java.util.Date timefilter = this.filter.getTime();
                    int comparison = Utility.getComparisonTerm(this.filter.getTimeClause());
                    this.newItemKeys     = new ArrayList();
                    this.updatedItemKeys = HelperForGetter.setUpdatedTimeFilter(
                            this.filter.getTime(), comparison,
                            this.serverItems);
                    this.deletedItemKeys = new ArrayList();
                } else {
                    // no filter
                    this.newItemKeys     = HelperForGetter.setNewItemKeys(
                            this.localItems, this.serverItems);
                    this.updatedItemKeys = HelperForGetter.setUpdatedItemKeys(
                            this.localItems, this.serverItems);
                    this.deletedItemKeys = HelperForGetter.setDeletedItemKeys(
                            this.localItems, this.serverItems);
                }
            }
        } else {
            // exclusive
            if (this.filter.getId() != null){
                // id filter
                String idfilter = this.filter.getId();
                this.newItemKeys     = HelperForGetter.setNewItemKeysWithIdFilter(
                        idfilter, this.localItems, this.serverItems);
                this.updatedItemKeys = HelperForGetter.setUpdatedItemKeysWithIdFilter(
                        idfilter, this.localItems, this.serverItems);
                this.deletedItemKeys = HelperForGetter.setDeletedItemKeysWithIdFilter(
                        idfilter, this.localItems, this.serverItems);
            } else {
                if (this.filter.getTime() != null) {
                    // time filter
                    java.util.Date timefilter = this.filter.getTime();
                    int comparison = Utility.getComparisonTerm(this.filter.getTimeClause());
                    this.newItemKeys     = HelperForGetter.setNewItemKeysWithTimeFilter(
                            timefilter, comparison, this.localItems, this.serverItems);
                    this.updatedItemKeys = HelperForGetter.setUpdatedItemKeysWithTimeFilter(
                            timefilter, comparison, this.localItems, this.serverItems);
                    this.deletedItemKeys = HelperForGetter.setDeletedItemKeysWithTimeFilter(
                            timefilter, comparison, this.localItems, this.serverItems);
                } else {
                    // no filter
                    this.newItemKeys     = HelperForGetter.setNewItemKeys(
                            this.localItems, this.serverItems);
                    this.updatedItemKeys = HelperForGetter.setUpdatedItemKeys(
                            this.localItems, this.serverItems);
                    this.deletedItemKeys = HelperForGetter.setDeletedItemKeys(
                            this.localItems, this.serverItems);
                }
            }
        }
    }


    /**
     * refresh the cache in the End Sync Session
     * Note: we add and remove items from the serverItems
     * hashmap during the syncsession
     *
     * @throws SyncSourceException
     */
    private void cacheAtEnd() throws EntityException {

        try {

            HashMap endServerItems = this.emailhelper.getAllSyncItemInfo();

            this.invalidItems = this.cacheDao.getInvalidItems(this.getSourceURI(),
                    this.principalId);

            this.cacheDao.updateLocalItems(endServerItems,
                    this.sentItems,
                    this.invalidItems,
                    this.getSourceURI(),
                    this.principalId);

        } catch (Exception e) {
            throw new EntityException("Error caching items:" +
                    e.getMessage(), e);
        }

    }


    /**
     * set the device info for the session
     *
     * @param context
     * @throws EntityException
     */
    private void setDeviceInfo(SyncContext context) throws EntityException {

        String deviceId = null;
        String timezone = null;
        String deviceTimeZoneDescr = null;
        Sync4jDevice device = null;
        try {
            deviceId = context.getPrincipal().getDeviceId();
            device   = getDevice(deviceId);
            timezone = device.getTimeZone();
            if (device.getConvertDate()) {
                if (timezone != null && timezone.length() > 0) {
                    deviceTimeZoneDescr = timezone;
                    this.deviceTimeZone = TimeZone.getTimeZone(deviceTimeZoneDescr);
                }
            }
            this.deviceCharset = device.getCharset();
        } catch (PersistentStoreException ex) {
            throw new EntityException("Error setting device '" + deviceId + "'", ex);
        }

    }

    /**
     * set the mail server account object
     */
    private void setMailServerAccount() throws EntityException {

        ConsoleDAO consoleDao = new ConsoleDAO(this.jndiDataSourceName,
                this.encryptionKey);

        MailServerAccount msa = consoleDao.getUser(this.userName);

        if (msa != null){

            // set the ssl path if needed
            if (msa.getMailServer() != null){
                if (msa.getMailServer().getIsSSLIn() || msa.getMailServer().getIsSSLOut() ){
                    msa.getMailServer().setKeystorePath(
                            this.keystoreHome + "\\" + msa.getMailServer().getKeystorePath());
                }
            }

            this.mailServerAccount = msa;

            // add the prefix 'recent:' for a gmail serrver
            String type  = this.mailServerAccount.getMailServer().getMailServerType();
            String login = this.mailServerAccount.getMsLogin();
            if (type.equalsIgnoreCase(Def.SERVER_TYPE_GMAIL)) {
                this.mailServerAccount.setMsLogin("recent:" + login);
            }

        } else {

            throw new EntityException("Error getting Account information ");

        }
    }


    /**
     * insert the default folder if needed
     *
     * @throws EntityException
     */
    private void insertDefaultFolder() throws EntityException {

        try {
            this.emailhelper.insertDefaultFolder(this.defaultFolder,
                                                 this.filter,
                                                 this.mailServerAccount.getMailServer().getProtocol(),
                                                 this.getSourceURI(),
                                                 this.principalId);
        } catch (EntityException e) {
            throw new EntityException("Error inserting default folders", e);
        }

    }


    /**
     * check if in the Drafts folder therse is an item
     * without message-id
     *
     * @throws SyncSourceException
     */
    private void checkMessageIDforDrafts()
    throws EntityException {
        try {
            this.emailhelper.checkMessageIDforDrafts(this.getSourceURI(),
                    this.principalId);
        } catch (EntityException e) {
            throw new EntityException("Error inserting default folders", e);
        }

    }

    /**
     * Return the device with the given deviceId
     *
     * @param deviceId String
     * @return Sync4jDevice
     * @throws PersistentStoreException
     */
    private Sync4jDevice getDevice(String deviceId) throws PersistentStoreException {
        Sync4jDevice device = new Sync4jDevice(deviceId);
        PersistentStore store = Configuration.getConfiguration().getStore();
        store.read(device);
        return device;
    }

    /**
     * Getter for property defaultFolder.
     *
     * @return Value of property defaultFolder.
     */
    private DefaultFolder getDefaultFolder() {
        return this.defaultFolder;
    }

    /**
     * Setter for property defaultFolder.
     *
     * @param defaultFolder DefaultFolder
     */
    private void setDefaultFolder(MailServer ms)
    throws EntityException {
        try {
            this.defaultFolder.setInboxName(ms.getInboxPath());
            this.defaultFolder.setOutboxName(ms.getOutboxPath());
            this.defaultFolder.setSentName(ms.getSentPath());
            this.defaultFolder.setDraftsName(ms.getDraftsPath());
            this.defaultFolder.setTrashName(ms.getTrashPath());
        } catch (Exception e) {
            throw new EntityException("Error setting default folders paths", e);
        }
    }

    /**
     *
     */
    private String printFolders() {
        String out = this.defaultFolder.getInboxName() + " (" + 
                this.mailServerAccount.getMailServer().getInboxActivation() + "); " +
                     this.defaultFolder.getOutboxName() + " (" +
                this.mailServerAccount.getMailServer().getOutboxActivation() + "); " +
                     this.defaultFolder.getSentName()   + " (" +
                this.mailServerAccount.getMailServer().getSentActivation() + "); " +
                     this.defaultFolder.getDraftsName() + " (" +
                this.mailServerAccount.getMailServer().getDraftsActivation() + "); " +
                     this.defaultFolder.getTrashName()  + " (" +
                this.mailServerAccount.getMailServer().getTrashActivation() + "); ";                     
        return out;
    }
    
    /**
     *
     */
    private String printIntro() {
        String out = 
                "\nCOORDS " +
                "\nprincipalId  " + this.principalId +
                "\nlogin        " + this.userName +
                "\ninHost       " + this.mailServerAccount.getMailServer().getInServer() +
                "\noutHost      " + this.mailServerAccount.getMailServer().getOutServer() +
                "\nprotocol     " + this.mailServerAccount.getMailServer().getProtocol() +
                "\nfolders      [" + this.printFolders() + "]" +
                "\nFILTER\n" + this.filter;
        return out;
    }    
}
