/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.source;

import com.funambol.email.exception.EntityException;
import com.funambol.email.model.SyncItemInfo;
import java.util.ArrayList;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import java.util.HashMap;
import java.util.Iterator;
import javax.mail.search.ComparisonTerm;


/**
 * This class includes the methods to set the New Items List, 
 * Updated Items List and Daleted Items List, in according with
 * the filter settings
 *
 * @version $Id: HelperForGetter.java,v 1.6 2007/01/15 16:37:39 nichele Exp $
 *
 */
public class HelperForGetter {

    
    // ---------------------------------------------------------- Public Methods

    /**
     * Creates new items id array
     *
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all server SyncItemInfo
     * @return new items id array ArrayList
     * @throws EntityException
     */
    public static ArrayList setNewItemKeys(HashMap localItems,
                                           HashMap serverItems)
     throws EntityException {

        ArrayList       newRows = new ArrayList(); 
        SyncItemInfo scrcii  = null;
        SyncItemInfo lcrcii  = null;
        String          sGUID   = null;
        
        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (SyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (SyncItemInfo)localItems.get(sGUID);
               if (lcrcii == null){
                  newRows.add(scrcii.getKey());
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting New Item List " + e.getMessage());
        }
        
        return newRows;
        
    }
    
    /**
     * Creates new items id array
     *
     * @param timefilter Date
     * @param comparison Ge or LT
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all server SyncItemInfo
     * @return new items id array
     * @throws EntityException
     */
    public static ArrayList setNewItemKeysWithTimeFilter(java.util.Date timefilter,
                                                         int comparison, 
                                                         HashMap localItems,
                                                         HashMap serverItems) 
     throws EntityException {
                                                        
        ArrayList       newRows = new ArrayList(); 
        SyncItemInfo scrcii  = null;
        SyncItemInfo lcrcii  = null;
        String          sGUID   = null;
        java.util.Date  demail  = null;

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (SyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (SyncItemInfo)localItems.get(sGUID);
               if (lcrcii == null){
                    if (comparison == ComparisonTerm.GE){
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Afterd2(demail, timefilter)){
                                newRows.add(scrcii.getKey());
                            }                        
                        }
                    } else {
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Befored2(demail, timefilter)){
                                newRows.add(scrcii.getKey());
                            }                        
                        }
                    }                        
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting NewItem List " + e.getMessage());
        }
        
        
        return newRows;

    }
    
    /**
     *
     * Creates new items id array
     *
     * @param idfilter String
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all server SyncItemInfo
     * @return new items id array ArrayList
     * @throws EntityException
     */
    public static ArrayList setNewItemKeysWithIdFilter(String idfilter,
                                                       HashMap localItems,
                                                       HashMap serverItems) 
     throws EntityException {                                                       
        
        ArrayList       newRows = new ArrayList(); 
        SyncItemInfo scrcii  = null;
        SyncItemInfo lcrcii  = null;
        String          sGUID   = null;

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (SyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (SyncItemInfo)localItems.get(sGUID);
               if (lcrcii == null){
                   if (idfilter.equals(sGUID)){
                       newRows.add(scrcii.getKey());
                   }   
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting New Item List " + e.getMessage());
        }
        
        
        return newRows;
                
    }

    /**
     * Creates deleted items id array
     *
     * @param localItems map with all local CrcSyncItemInfo
     * @param serverItems map with all Mail Server CrcSyncItemInfo
     * @return deleted items id array
     */
    public static ArrayList setDeletedItemKeys(HashMap localItems,
                                               HashMap serverItems)      
      throws EntityException {

        ArrayList       deleteRows = new ArrayList();
        SyncItemInfo scrcii     = null;
        SyncItemInfo lcrcii     = null;
        String          lGUID      = null;

        try {
            Iterator values = localItems.values().iterator();

            while (values.hasNext()) {
               lcrcii = (SyncItemInfo)values.next();
               lGUID = lcrcii.getKey().getKeyAsString();
               scrcii = (SyncItemInfo)serverItems.get(lGUID);
               if (scrcii == null){
                   deleteRows.add(lcrcii.getKey());
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting Deleted Item List " + e.getMessage());
        }
        
                
        return deleteRows;
        
    }

    /**
     * Creates deleted items id array
     *
     * @param timefilter Date
     * @param comparison GE or LT
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all Mail Server SyncItemInfo
     * @return deleted items id array ArrayList
     * @throws EntityException
     */
    public static ArrayList setDeletedItemKeysWithTimeFilter(java.util.Date timefilter,
                                                             int comparison,
                                                             HashMap localItems,
                                                             HashMap serverItems) 
     throws EntityException {

        ArrayList       deleteRows = new ArrayList();
        SyncItemInfo scrcii     = null;
        SyncItemInfo lcrcii     = null;
        String          lGUID      = null;
        java.util.Date  demail     = null;

        try {
            Iterator values = localItems.values().iterator();

            while (values.hasNext()) {
               lcrcii = (SyncItemInfo)values.next();
               lGUID = lcrcii.getKey().getKeyAsString();
               scrcii = (SyncItemInfo)serverItems.get(lGUID);
               if (scrcii == null){
                    if (comparison == ComparisonTerm.GE){
                        demail = lcrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Afterd2(demail, timefilter)){
                                deleteRows.add(lcrcii.getKey());
                            }                                                                            
                        }
                    } else {
                        demail = lcrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Befored2(demail, timefilter)){
                                deleteRows.add(lcrcii.getKey());
                            }                                                                            
                        }
                    }                
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Deleted Item List " + e.getMessage());
        }
        
                
        return deleteRows;
                
    }
    
    /**
     * Creates deleted items id array
     *
     * @param idfilter String
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all Mail Server SyncItemInfo
     * @return deleted items id array
     * @throws EntityException
     */
    public static ArrayList setDeletedItemKeysWithIdFilter(String idfilter, 
                                                           HashMap localItems,
                                                           HashMap serverItems) 
      throws EntityException {

        ArrayList       deleteRows = new ArrayList();
        SyncItemInfo scrcii     = null;
        SyncItemInfo lcrcii     = null;
        String          lGUID      = null;

        try {
            Iterator values = localItems.values().iterator();

            while (values.hasNext()) {
               lcrcii = (SyncItemInfo)values.next();
               lGUID = lcrcii.getKey().getKeyAsString();
               scrcii = (SyncItemInfo)serverItems.get(lGUID);
               if (scrcii == null){
                    if (idfilter.equals(lGUID)){
                       deleteRows.add(lcrcii.getKey());
                    }
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Deleted Item List " + e.getMessage());
        }
        
                
        return deleteRows;
        
    }    
    
    /**
     * Creates updated items id array
     *
     * @param localItems map with all local CrcSyncItemInfo
     * @param serverItems map with all Mail Server CrcSyncItemInfo
     * @return updated items id array
     */
    public static ArrayList setUpdatedItemKeys(HashMap localItems,
                                               HashMap serverItems)      
     throws EntityException {

        ArrayList updateRows         = new ArrayList();
        long serverItemLastCrc ;
        long localItemLastCrc  ;        
        SyncItemInfo scrcii = null;
        SyncItemInfo lcrcii = null;
        String sGUID = null;

        try {
            Iterator values = serverItems.values().iterator();
            while (values.hasNext()) {
               scrcii = (SyncItemInfo)values.next();
               sGUID = scrcii.getKey().getKeyAsString();
               lcrcii = (SyncItemInfo)localItems.get(sGUID);
               if (lcrcii != null){
                  serverItemLastCrc = scrcii.getLastCrc () ;
                  localItemLastCrc  = lcrcii.getLastCrc () ;
                  if (serverItemLastCrc != localItemLastCrc) {
                      updateRows.add(scrcii.getKey());
                  }
               }
            }   
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        return updateRows ;
    }

    /**
     * Creates updated items id array
     *
     * @param timefilter Date
     * @param comparison GE or LT
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all server SyncItemInfo
     * @return updated items id array
     * @throws EntityException
     */
    public static ArrayList setUpdatedItemKeysWithTimeFilter(java.util.Date timefilter,
                                                             int comparison,
                                                             HashMap localItems,
                                                             HashMap serverItems) 
      throws EntityException {
        
        ArrayList       updateRows = new ArrayList();
        SyncItemInfo scrcii     = null;
        SyncItemInfo lcrcii     = null;
        String          sGUID      = null;
        long serverItemLastCrc ;
        long localItemLastCrc  ;
        java.util.Date demail = null;

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (SyncItemInfo)values.next();
               sGUID  = scrcii.getKey().getKeyAsString();
               lcrcii = (SyncItemInfo)localItems.get(sGUID);
               if (lcrcii != null){
                  serverItemLastCrc = scrcii.getLastCrc () ;
                  localItemLastCrc  = lcrcii.getLastCrc () ;
                  if (serverItemLastCrc != localItemLastCrc) {
                     if (comparison == ComparisonTerm.GE){
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Afterd2(demail, timefilter)){
                                updateRows.add(scrcii.getKey());
                            }                                                    
                        }
                     } else {
                        demail = scrcii.getHeaderDate();
                        if (demail != null){
                            if (Utility.d1Befored2(demail, timefilter)){
                                 updateRows.add(scrcii.getKey());
                            }                                                    
                        }
                     }                        
                  }
               }
            } 
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        
        return updateRows;

    }
    
    
    
    /**
     * Creates updated items id array
     *
     * @param idfilter String
     * @param localItems map with all local SyncItemInfo
     * @param serverItems map with all server SyncItemInfo
     * @return updated items id array
     * @throws EntityException
     */
    public static ArrayList setUpdatedItemKeysWithIdFilter(String idfilter,
                                                           HashMap localItems,
                                                           HashMap serverItems) 
      throws EntityException {
        
        ArrayList       updateRows = new ArrayList();
        SyncItemInfo scrcii     = null;
        SyncItemInfo lcrcii     = null;
        String          sGUID      = null;
        long serverItemLastCrc ;
        long localItemLastCrc  ;        

        try {
            Iterator values = serverItems.values().iterator();

            while (values.hasNext()) {
               scrcii = (SyncItemInfo)values.next();
               sGUID  = scrcii.getKey().getKeyAsString();
               lcrcii = (SyncItemInfo)localItems.get(sGUID);
               if (lcrcii != null){
                  serverItemLastCrc = scrcii.getLastCrc () ;
                  localItemLastCrc  = lcrcii.getLastCrc () ;
                  if (serverItemLastCrc != localItemLastCrc) {
                      if (idfilter.equals(sGUID)){
                         updateRows.add(scrcii.getKey());
                      }
                  }
               }
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        
        return updateRows;
        
    }
    
    /**
     *
     * in the updated list there are just the emails related to the time filter
     *
     * @param timefilter Date
     * @param comparison GE or LT
     * @param serverItems map with  all mail server SyncItemInfo
     * @return ArrayList
     * @throws EntityException
     */
    public static ArrayList setUpdatedTimeFilter(java.util.Date timefilter, 
                                                 int comparison,
                                                 HashMap serverItems) 
     throws EntityException {
       
        ArrayList updateRows   = new ArrayList();
        SyncItemInfo scrcii = null;
        java.util.Date demail  = null;
        
        try {
            
            Iterator values = serverItems.values().iterator();
            
            if (comparison == ComparisonTerm.GE){
                // get all email where time "great/equals"
                while (values.hasNext()) {
                   scrcii = (SyncItemInfo)values.next();
                   demail = scrcii.getHeaderDate();
                   if (demail != null){
                       if (Utility.d1Afterd2(demail, timefilter)){
                           updateRows.add(scrcii.getKey());
                       }                                                                        
                   }
                }   
            } else {
                // get all emails where time "less than"
                while (values.hasNext()) {
                   scrcii = (SyncItemInfo)values.next();
                   demail = scrcii.getHeaderDate();
                   if (demail != null){
                       if (Utility.d1Befored2(demail, timefilter)){
                           updateRows.add(scrcii.getKey());
                       }                                                                        
                   }
                }   
            }
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        return updateRows;
        
    }

   

    
    /**
     *
     * in the updated list there is just the id in the filter
     *
     * @param idfilter String
     * @param serverItems map with all the CrcSyncItemInfo
     * @return ArrayList
     * @throws EntityException
     */
    public static ArrayList setUpdatedIDFilter(String idfilter, 
                                               HashMap serverItems)      
      throws EntityException {
        
        ArrayList       updateRows = new ArrayList();
        SyncItemInfo scrcii     = null ;
        String          key        = null;

        try {
            scrcii = (SyncItemInfo)serverItems.get(idfilter);
            if (scrcii != null){
                updateRows.add(scrcii.getKey());
            }            
        } catch (Exception e){
            throw new EntityException ("Error setting Updated Item List " + e.getMessage());
        }
        
        return updateRows;
        
    }
    
    //---------------------------------------------------------- Private Methods

    /**
     * Checks if item is included in array of items
     *
     * @param items items array
     * @param item item to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private static boolean isItemInArray(SyncItemInfo[] items,
                                         SyncItemInfo   item) {

        return isIdInArray(items, item.getKey().getKeyAsString());

    }    
    
    /**
     * Checks if item id is included in ids from array of items
     *
     * @param items items array
     * @param id id to find
     * @return <p>true</p> if the item is included
     *         <p>false</p> if the item is not included
     **/
    private static boolean isIdInArray(SyncItemInfo[] items, String id) {
        boolean isFind = false;
        int l = items.length;
        for (int i = 0; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                isFind = true;
                break;
            }
        }
        return isFind;
    }


    /**
     * Gets item by id
     *
     *
     * @param items CrcSyncItemInfo[]
     * @param id String
     * @return <p>item</p> if found
     *         <p>null</p> if not found
     */
    private static SyncItemInfo getItemById(SyncItemInfo[] items, String id) {
        SyncItemInfo item = null;

        for (int i = 0, l = items.length; i < l; i++) {
            if (items[i].getKey().getKeyAsString().equals(id)) {
                item = items[i];
                break;
            }
        }

        return item;
    }

    
    
}

