/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.engine.source;

import com.funambol.email.exception.EntityException;
import com.funambol.email.model.DefaultFolder;
import com.funambol.email.model.EmailFilter;
import com.funambol.framework.engine.SyncItem;
import com.funambol.framework.engine.SyncItemKey;
import com.funambol.framework.engine.source.SyncSourceException;
import java.sql.Timestamp;
import java.util.HashMap;


/**
 * 
 * <p>Interface for Email SyncSource Wrapper </p>
 * 
 * @version $Id: IEmailGeneric.java,v 1.6 2007/01/26 17:39:55 gbmiglia Exp $
 */
public interface IEmailGeneric {

    /**
     * 
     * @return SyncItemKey[]
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     */
    public SyncItemKey[] getAllSyncItemKeys() 
      throws SyncSourceException ;
    /**
     * 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return HashMap
     */
    public HashMap getAllSyncItemInfo() 
      throws SyncSourceException ;
    /**
     * 
     * @param syncItemKey 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItem
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey) 
      throws SyncSourceException ;
    /**
     * 
     * @param syncItem 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItemKey[]
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem) 
      throws SyncSourceException ;
    /**
     * 
     * @param syncItem 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return boolean
     */
    public boolean isSyncItemInFilterClause(SyncItem syncItem) 
      throws SyncSourceException ;    
    /**
     * 
     * @param syncItemKey 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return boolean
     */
    public boolean isSyncItemInFilterClause(SyncItemKey syncItemKey) 
      throws SyncSourceException ;
    /**
     * 
     * @param syncItemKey 
     * @param s 
     * @param engineSoftDelete 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     */
    public void removeSyncItem(SyncItemKey syncItemKey, Timestamp s, boolean engineSoftDelete)
      throws SyncSourceException ;
    /**
     * 
     * @param syncItem 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItem
     */
    public SyncItem updateSyncItem(SyncItem syncItem) 
      throws SyncSourceException ;
    /**
     * 
     * @param syncItem 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItem
     */
    public SyncItem addSyncItem(SyncItem syncItem) 
      throws SyncSourceException ;
    /**
     * 
     * @param since 
     * @param to 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItemKey[]
     */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp to)
      throws SyncSourceException ;
    /**
     * 
     * @param since 
     * @param to 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItemKey[]
     */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp to)
      throws SyncSourceException ;    
    /**
     * 
     * @param since 
     * @param to 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return SyncItemKey[]
     */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp to)
      throws SyncSourceException ;
    /**
     * 
     * @param syncItemKey 
     * @throws com.funambol.framework.engine.source.SyncSourceException 
     * @return char
     */
    public char getSyncItemStateFromId(SyncItemKey syncItemKey)
      throws SyncSourceException ;
    /**
     * 
     * @param operation 
     * @param statusCode 
     * @param keys 
     */
    public void setOperationStatus(String operation, int statusCode, SyncItemKey[] keys) ;
    
 
    /**
     * 
     * @param sURI 
     * @param pID 
     * @throws com.funambol.email.exception.EntityException 
     */
    public void checkMessageIDforDrafts(String sURI, long pID)  
      throws EntityException;
    
    /**
     * 
     * @param df defaulFolder object
     * @param protocol 
     * @param sURI sourceURI
     * @param pID principal ID
     * @throws com.funambol.email.exception.EntityException 
     */
    public void insertDefaultFolder(DefaultFolder df, EmailFilter filter, 
                                    String protocol, String sURI,  long pID) 
      throws EntityException;
}
