/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.admin;


import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.framework.tools.beans.BeanException;
import com.funambol.framework.tools.beans.BeanTool;
import org.apache.commons.dbcp.BasicDataSource;

/**
 * This class loads the parameter from the XML configuration file
 *
 * @version $Id: InboxListenerConfig.java,v 1.20 2007/03/19 14:24:59 gbmiglia Exp $
 */
public class InboxListenerConfig {

    // --------------------------------------------------------------- CONSTANTS

    /**     */
    private final static String beanName = "email/email/InboxListener.xml";

    // -------------------------------------------------------------- PROPERTIES

    /**     */
    private String listenerId;
    /**     */
    private boolean startupNotification;
    /**     */
    private boolean saveSubject;
    /**     */
    private boolean saveSender;
    /**     */
    private int reloadingTime;
    /**     */
    private String keystoreHome;
    
    /**     */
    private String wsEndPoint;    
    /**     */
    private String wsUsername;
    /**     */
    private String wsPassword;
    /**     */
    private String wsSyncSource;    
    
    /**     */
    private BasicDataSource basicDataSource;
            
    //------------------------------------------------------------- CONSTRUCTORS

    /**
     *
     */
    public InboxListenerConfig() {
    }

    //----------------------------------------------------------- PUBLIC METHODS


    /**
     * @return InboxListenerConfig
     * @throws InboxListenerConfigException
     */
    public static InboxListenerConfig getConfigInstance()
       throws InboxListenerConfigException {

        InboxListenerConfig configInstance;
        try {
            String path = System.getProperty("funambol.inboxlistener.home");
            path = path + "/config/";
            Object obj =  BeanTool.getBeanTool(path).getBeanInstance(beanName);
            configInstance = (InboxListenerConfig) obj;
        } catch (BeanException bie) {
            throw new InboxListenerConfigException(bie.getMessage());
        }

        return configInstance;

    }

    
    /**
     * @param _listenerId String representing the listener ID.
     */
    public void setListenerId(String _listenerId) {
        this.listenerId = _listenerId;
    }

    /**
     * @return String
     */
    public String getListenerId() {
        return this.listenerId;
    }

    /**
     * @param _startupNotification boolean
     */
    public void setStartupNotification(boolean _startupNotification) {
        this.startupNotification = _startupNotification;
    }

    /**
     * @return boolean
     */
    public boolean getStartupNotification() {
        return this.startupNotification;
    }
    
    /**
     * @param _saveSubject boolean
     */
    public void setSaveSubject(boolean _saveSubject) {
        this.saveSubject = _saveSubject;
    }

    /**
     * @return boolean
     */
    public boolean getSaveSubject() {
        return this.saveSubject;
    }
    
    /**
     * @param _saveSender boolean
     */
    public void setSaveSender(boolean _saveSender) {
        this.saveSender = _saveSender;
    }

    /**
     * @return boolean
     */
    public boolean getSaveSender() {
        return this.saveSender;
    }
    
    /**
     * @param _reloadingTime int
     */
    public void setReloadingTime(int _reloadingTime) {
        this.reloadingTime = _reloadingTime;
    }

    /**
     * @return String
     */
    public int getReloadingTime() {
        return this.reloadingTime;
    }
    
    /**
     * @param _keystoreHome String 
     */
    public void setKeystoreHome(String _keystoreHome) {
        this.keystoreHome = _keystoreHome;
    }

    /**
     * @return String
     */
    public String getKeystoreHome() {
        return this.keystoreHome;
    }

    /**
     * @param _wsEndPoint String
     */
    public void setWsEndPoint(String _wsEndPoint) {
        this.wsEndPoint = _wsEndPoint;
    }

    /**
     * @return String
     */
    public String getWsEndPoint() {
        return this.wsEndPoint;
    }

    /**
     * @param _wsUsername String
     */
    public void setWsUsername(String _wsUsername) {
        this.wsUsername = _wsUsername;
    }

    /**
     * @return String
     */
    public String getWsUsername() {
        return this.wsUsername;
    }

    /**
     * @param _wsPassword String
     */
    public void setWsPassword(String _wsPassword) {
        this.wsPassword = _wsPassword;
    }

    /**
     * @return String
     */
    public String getWsPassword() {
        return this.wsPassword;
    }

    /**
     * @param _wsSyncSource String
     */
    public void setWsSyncSource(String _wsSyncSource) {
        this.wsSyncSource = _wsSyncSource;
    }

    /**
     * @return String
     */
    public String getWsSyncSource() {
        return this.wsSyncSource;
    }
        
    
    /**
     * @param _basicDataSource BasicDataSource
     */
    public void setBasicDataSource(BasicDataSource _basicDataSource) {
        this.basicDataSource = _basicDataSource;
    }

    /**
     * @return basicDataSource
     */
    public BasicDataSource getBasicDataSource() {
        return this.basicDataSource;
    }
    
   
}
