/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.console;


import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.WSTools;
import com.funambol.email.model.MailServer;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.SyncItemInfoAdmin;
import com.funambol.framework.filter.Clause;
import com.funambol.framework.tools.beans.BeanException;
import com.funambol.framework.tools.beans.BeanFactory;

/**
 * Contains methods to access to the DB tables
 *
 * @version $Id: MenuWSDAO.java,v 1.16 2007/03/19 14:25:11 gbmiglia Exp $
 */
public class MenuWSDAO {

  // ------------------------------------------------------------ Private data
  
  /**   */
  private WSTools wsTools = null;

  // ------------------------------------------------------------- Constructor

  /**
   * 
   * @param endPoint 
   * @param user 
   * @param pwd
   * @throws com.funambol.email.exception.InboxListenerException 
   */
  public MenuWSDAO(String endPoint, String user, String pwd) 
    throws InboxListenerException {
      try {
         endPoint = this.emailEndPoint(endPoint);
         this.wsTools  = new WSTools(endPoint,user,pwd);     
      } catch (Exception e) {
          throw new InboxListenerException("Error creating WS tool");
      }
  }



  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   * @throws com.funambol.email.exception.InboxListenerException 
   * @throws com.funambol.framework.tools.beans.BeanException 
   * @return MailServerAccount[] 
   */
  public MailServerAccount[]  getAccounts () 
    throws InboxListenerException, BeanException {
        MailServerAccount[] accounts = null;
        
        Clause wc = null;
        //String userName = "gibi";
        //String value[] = new String[]{userName};
        //WhereClause wc = new WhereClause("username", value, WhereClause.OPT_CONTAINS, true);        
        Object[] params = new Object[] { BeanFactory.marshal(wc) } ; 
        
        try { 
            Object obj = wsTools.invoke("getUsers", params);
            accounts = (MailServerAccount[])obj;
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting all the accounts");
        }                
        return accounts;

  }

  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   * @throws com.funambol.email.exception.InboxListenerException 
   * @throws com.funambol.framework.tools.beans.BeanException 
   * @return MailServer[] 
   */
  public MailServer[]  getMailServers () throws InboxListenerException, BeanException {
        MailServer[] mailServers = null;
        
        Clause wc = null;
        //String tmp = "a";
        //String value[] = new String[]{tmp};
        //WhereClause wc = new WhereClause("description", value, WhereClause.OPT_CONTAINS, true);        
        
        Object[] params = new Object[] { BeanFactory.marshal(wc) } ; 
        
        try { 
            Object obj = wsTools.invoke("getPubMailServers", params);
            mailServers = (MailServer[])obj;
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting all the mail Servers");
        }                
        return mailServers;

  }
  
  /**
     * Get Account; returns the MailAccount Object with 
     * all the info about the user
     * 
     * @return account info MailboxAccount
     * @param username String
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public MailServerAccount getAccount (String username) 
    throws InboxListenerException {      
        MailServerAccount msa = null;        
        Object[] params = new Object[] {username};
        try { 
            msa = (MailServerAccount)wsTools.invoke("getUser", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting the account");
        }        
        return msa;
  }
  
  /**
     * Get Account; returns the MailAccount Object with 
     * all the info about the user
     * 
     * @return account info MailboxAccount
     * @param username String
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public SyncItemInfoAdmin[] getCachedInfo (String username, String protocol) 
    throws InboxListenerException {      
        SyncItemInfoAdmin[] info = null;        
        Object[] params = new Object[] {username, protocol};
        try { 
            Object obj = wsTools.invoke("getCachedInfo", params);
            info = (SyncItemInfoAdmin[])obj;
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting cached info (" + e.getMessage() + ")");
        }        
        return info;
  }
  
  /**
     * Get Account; returns the MailAccount Object with 
     * all the info about the user
     * 
     * @return account info MailboxAccount
     * @param msa 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public int checkAccount (MailServerAccount msa) 
    throws InboxListenerException {      
        int isOK = 0;        
        Object[] params = new Object[] {msa};
        try { 
            isOK = ((Integer)wsTools.invoke("checkAccount", params)).intValue();
        } catch (Exception e)  {
            throw new InboxListenerException("Error checking the account");
        }        
        return isOK;
  }
  
  /**
     * Get Account; returns the MailAccount Object with 
     * all the info about the user
     * 
     * @return account info MailboxAccount
     * @param mailserverID 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public MailServer getMailServerFromID (String mailserverID) 
    throws InboxListenerException {      
        MailServer ms = null;        
        Object[] params = new Object[] {mailserverID};
        try { 
            ms = (MailServer)wsTools.invoke("getPubMailServerFromID", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting the mail server");
        }        
        return ms;
  }
  
  /**
     * Get Account; returns the MailAccount Object with 
     * all the info about the user
     * 
     * @return account info MailboxAccount
     * @param accountID 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public MailServerAccount getAccountFromID (String accountID) 
    throws InboxListenerException {      
        MailServerAccount msa = null;        
        Object[] params = new Object[] {accountID};
        try { 
            msa = (MailServerAccount)wsTools.invoke("getUserFromID", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error getting the account from ID");
        }        
        return msa;
  }

  /**
     * 
     * @param msa 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public void insertAccount (MailServerAccount msa)
    throws InboxListenerException {      
        Object[] params = new Object[] {msa};
        try { 
           wsTools.invoke("insertUser", params);      
        } catch (Exception e)  {
            throw new InboxListenerException("Error inserting the account");
        }
  }

  /**
     * 
     * @param ms 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public void insertMailServer (MailServer ms)
    throws InboxListenerException {      
        Object[] params = new Object[] {ms};
        try { 
           wsTools.invoke("insertPubMailServer", params);      
        } catch (Exception e)  {
            throw new InboxListenerException("Error inserting the mail server");
        }
  }
  
  /**
     * 
     * @param msa 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public void updateAccount (MailServerAccount msa)
    throws InboxListenerException {      
        Object[] params = new Object[] {msa};
        try { 
            wsTools.invoke("updateUser", params);                
        } catch (Exception e)  {
            throw new InboxListenerException("Error inserting the account");
        }
  }
  
  /**
     * 
     * @param accountID 
     * @throws InboxListenerException 
     */
  public void disableAccount (String accountID)
    throws InboxListenerException {      
        Object[] params = new Object[] {accountID};
        try { 
            wsTools.invoke("disableUser", params);                
        } catch (Exception e)  {
            throw new InboxListenerException("Error disabling the account");
        }
  }
  
  /**
     * 
     * @param accountID 
     * @throws InboxListenerException 
     */
  public void enableAccount (String accountID)
    throws InboxListenerException {      
        Object[] params = new Object[] {accountID};
        try { 
            wsTools.invoke("enableUser", params);                
        } catch (Exception e)  {
            throw new InboxListenerException("Error enabling the account");
        }
  }
  
  
  /**
     * 
     * @param ms 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public void updateMailServer (MailServer ms)
    throws InboxListenerException {      
        Object[] params = new Object[] {ms};
        try { 
            wsTools.invoke("updatePubMailServer", params);                
        } catch (Exception e)  {
            throw new InboxListenerException("Error inserting the mail server");
        }
  }
  
  
  /**
     * 
     * @param accountID 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public void deleteAccount (String accountID)
    throws InboxListenerException {
        Object[] params = new Object[] {accountID};
        try {
            wsTools.invoke("deleteUser", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error deleting the account");
        }        
  }
  
  /**
     * 
     * @param mailserverID 
     * @throws com.funambol.email.exception.InboxListenerException 
     */
  public void deleteMailServer (String mailserverID)
    throws InboxListenerException {
        Object[] params = new Object[] {mailserverID};
        try {
            wsTools.invoke("deletePubMailServer", params);
        } catch (Exception e)  {
            throw new InboxListenerException("Error deleting the mail server");
        }        
  }
  
  //----------------------------------------------------------------------------
  
  /**
   *
   */
  private String emailEndPoint(String adminEndPoint) {
      String ADMIN = "admin";
      String EMAIL = "email";
      
      String tmp = adminEndPoint.substring(0,(adminEndPoint.length()-ADMIN.length()));
      tmp = tmp + EMAIL;
      
      return tmp;
      
  }

  
}
