/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.model.MailServerAccount;
import java.util.List;
import java.util.ArrayList;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.email.util.Def;
import com.funambol.email.util.UtilityIL;
import com.funambol.framework.server.Sync4jUser;
import com.funambol.framework.server.store.PersistentStoreException;
import com.funambol.server.admin.UserManager;
import com.funambol.server.config.Configuration;


/**
 *
 *
 * @version $Id: ListenerRegistry.java,v 1.25 2007/03/19 13:20:33 gbmiglia Exp $
 */
public class ListenerRegistry extends Thread {

  //------------------------------------------------------------------ CONSTANTS

    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);

  //----------------------------------------------------------------- PROPERTIES

  /** list wtih all account to monitor  */
  private List accountList = new ArrayList() ;

  /**   */
  private WSAccount wsa = null;

  /**   */
  private String listenerId = null;

  /**   */
  private String keystoreHome = null;
  
  /**   */
  private String encryptionKey = null;
  
  /**   */
  private boolean startupNotification = false;

  /**   */
  private boolean saveSubject = false;
  
  /**   */
  private boolean saveSender = false;
  
  /** 
   * in the XML there is the minutes
   */
  private int reloadingTime;
  
  /**   */
  private ListenerRegistryDAO lrdao = null;

  /** to stop the thread  */
  private boolean go = true ;


  //--------------------------------------------------------------- CONSTRUCTORS

  /**
   *
   */
  public ListenerRegistry(String _listenerId,
                          boolean _startupNotification,
                          boolean _saveSubject,
                          boolean _saveSender,
                          int _reloadingTime,
                          String _keystoreHome,
                          String _encryptionKey,
                          WSAccount _wsa) {
     this.listenerId          = _listenerId;
     this.startupNotification = _startupNotification;
     this.saveSubject         = _saveSubject;
     this.saveSender          = _saveSender;
     this.reloadingTime       = _reloadingTime;
     this.keystoreHome        = _keystoreHome;
     this.encryptionKey       = _encryptionKey;
     this.wsa                 = _wsa;
     this.lrdao               = new ListenerRegistryDAO(this.keystoreHome,
                                                        this.encryptionKey);
  }


  //------------------------------------------------------------- PUBLIC METHODS

  /**
   *
   *
   */
  public void run() {

     while (go) {

        if (log.isTraceEnabled()) {
            log.trace("Read Accounts - DB information max active: " + 
                      DBConnectionProvider.getMaxActive() + " max idle: " + 
                      DBConnectionProvider.getMaxIdle() );
        }
         
        // clear the account table (instead of sql DELETE CASCADE)
        removeAccountsWithoutDSUser();
                
        // get list
        List accountListNew = null;
        try {            
           accountListNew = lrdao.getAccounts(listenerId);
        } catch (Exception e){
           log.error("Error getting the accounts " + e.toString());
        }
        
        List accountListOld = this.accountList;

        if (accountListNew != null){
           manageMailboxListener(accountListNew,accountListOld);
        }

        // sleep for X mins
        try{
           sleep(UtilityIL.setMilliSecondsFromMinutes(this.reloadingTime));
        } catch (Exception e){
           log.error("Error sleeping .. " + e.toString());
        }

     }

  }

  /**
   * close all the MailboxManager before closing the 
   * Inbox-Listener Application
   */
   public void closeMailboxManager() throws InboxListenerException {
       
       if (log.isTraceEnabled()) {
           log.trace("Start Closing all the MailboxManager");
       }
       
       MailServerAccount msaNew  = null;
       String            userNew = null;
       MailboxManager    mm      = null;
       
       for (int i=0; i<accountList.size(); i++){
           
           msaNew  = (MailServerAccount) accountList.get(i);
           userNew = getLogin(msaNew);
           mm      = userListenerGetter(userNew, msaNew.getUsername());
           
           if (mm != null){
               try {
                   mm.closeMailServerConnection();
               } catch (InboxListenerException e) {
                   throw new InboxListenerException(e.getMessage());
               }
           }
           
       }       
       
   }

  //------------------------------------------------------------ PRIVATE METHODS

  /**
   * 
   *
   */
  private void manageMailboxListener(List accountListNew,
                                    List accountListOld){
      
      // remove listener for a deleted user
      accountListRemover(accountListNew, accountListOld);
      
      // add listener for a new user
      accountListAdder(accountListNew, accountListOld);
      
      // update a listener
      accountListUpdater(accountListNew, accountListOld);
                  
      // replace the new account list
      accountListRenewer(accountListNew, accountListOld);
      
  }

  
  /**
   * remove listener for a deleted user.
   * <br>
   * remove the account from the active account list.
   * then it stops the thread
   *
   * @param list of the new accounts
   * @param list of the old accounts
   *
   */
  private void accountListRemover(List accountListNew,
                                  List accountListOld){

      MailServerAccount msaOld    = null;
      String            msuserOld = null;
      String            dsuserOld = null;
      MailboxManager    mm        = null;
      
      for (int i=0; i<accountListOld.size(); i++){
          
          msaOld  = (MailServerAccount) accountListOld.get(i);          
          
          msuserOld = msaOld.getMsLogin();          
          
          dsuserOld = msaOld.getUsername();
          
          if (toBeRemoved(msuserOld, dsuserOld, accountListNew)){
             if (log.isTraceEnabled()) {
                 log.trace("Removing Listener for User " + msuserOld);
             }
             // stop mailbox listener
             msuserOld  = getLogin(msaOld);             
             mm         = userListenerGetter(msuserOld, dsuserOld);
             if (mm != null){
                  mm.stopThread();
             }                 
          }
      }
      
  }  
  
  
  /**
   * add listener for a new user.
   * <br> 
   * add the account into the active account list.
   * then it starts the thread
   * 
   * @param list of the new accounts
   * @param list of the old accounts
   *
   */
  private void accountListAdder(List accountListNew,
                                List accountListOld){

      MailServerAccount msaNew  = null;
      String            msuserNew = null;
      String            dsuserNew = null;
      MailboxManager    mm      = null;
      
      for (int i=0; i<accountListNew.size(); i++){    
          
          msaNew  = (MailServerAccount) accountListNew.get(i);
          
          msuserNew = getLogin(msaNew);
          
          msaNew.setMsLogin(msuserNew);
          
          dsuserNew = msaNew.getUsername();
          
          if (toBeAdded(msuserNew, dsuserNew, accountListOld)){
              
             if (log.isTraceEnabled()) {
                 log.trace("Adding Listener for User " + msuserNew );
             }
             
             // start mailbox listener
             mm = new MailboxManager(this.startupNotification, 
                                     this.saveSubject, this.saveSender,
                                     msaNew, this.wsa);
             
             if (mm != null){
                mm.setName(createThreadName(msuserNew, dsuserNew));
                mm.start();
             }   
             
          }
          
      }
      
  }
  
  
  /**
   * add the account into the active account list.
   * then it starts the thread
   * 
   * @param list of the new accounts
   * @param list of the old accounts
   *
   */
  private void accountListUpdater(List accountListNew,
                                  List accountListOld){

      MailServerAccount msaNew       = null;
      MailServerAccount msaNewLogin  = null;
      String            msuserNew    = null;
      String            dsuserNew    = null;
      MailboxManager    mm           = null;
            
      for (int i=0; i<accountListNew.size(); i++){          
          
          msaNew  = (MailServerAccount) accountListNew.get(i);
          
          msuserNew = getLogin(msaNew);
          
          msaNew.setMsLogin(msuserNew); 
          
          dsuserNew = msaNew.getUsername();
          
          if (toBeUpdated(msuserNew, dsuserNew, accountListOld)){
              
             if (log.isTraceEnabled()) {
                 log.trace("Updating Listener for User " + msuserNew);
             }
             
             // update mailbox listener
             mm          = userListenerGetter(msuserNew, dsuserNew);             
             
             msaNewLogin = (MailServerAccount) accountListNew.get(i);  
             
             // update the listener for a specific user
             // note: inorder to update the refresh_time the admin must
             // disable the account, wait until the thread is down,
             // enable the account with the new refresh_time
             if (mm != null){
                  mm.setMaxEmailNumber(msaNewLogin.getMaxEmailNumber());
                  mm.setPassword      (msaNewLogin.getMsPassword());
                  mm.setPush          (msaNewLogin.getPush());
                  mm.setProtocol      (msaNewLogin.getMailServer().getProtocol());
                  mm.setInServer      (msaNewLogin.getMailServer().getInServer());
                  mm.setInPort        (msaNewLogin.getMailServer().getInPort());
                  mm.setIsSSLIn       (msaNewLogin.getMailServer().getIsSSLIn());
                  mm.setKeystorePath  (msaNewLogin.getMailServer().getKeystorePath());
                  mm.setKeystorePwd   (msaNewLogin.getMailServer().getKeystorePwd());             
             } 
             
          }
          
      }      
      
  }
  
  
  /**
   * renew the account list
   *
   * @param list of the new accounts
   * @param list of the old accounts
   *
   */
  private void accountListRenewer(List accountListNew,
                                  List accountListOld){

      MailServerAccount msaNew  = null;
      
      accountList.clear();
      
      for (int i=0; i<accountListNew.size(); i++){
         msaNew = (MailServerAccount) accountListNew.get(i);
         accountList.add(msaNew);
      }
      
  }  
  
  
  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   *
   * to remove
   * ---------
   * user is the Old user
   * accounts is the New List
   * if the username is not in the list it retrun true, the
   * listener on the account must be deregistered
   *
   *
   */
  private boolean toBeRemoved(String msuser, String dsuser, List accountsNew){
      
      MailServerAccount msaNew            = null;
      String            msuserNew         = null;
      String            dsuserNew         = null;
      String            threadNameOld     = null;
      String            threadNameNew     = null;
      boolean           isAlreadyRegister = false;
      boolean           toRemove          = true;

      for (int i=0; i<accountsNew.size(); i++){
          
         msaNew    = (MailServerAccount) accountsNew.get(i);         
         msuserNew = getLogin(msaNew);         
         dsuserNew = msaNew.getUsername();    
         
         threadNameNew = createThreadName(msuserNew,dsuserNew);                  
         threadNameOld = createThreadName(msuser,dsuser);
         
         if (threadNameOld.equals(threadNameNew)){
            isAlreadyRegister = true;
            break;
         }
      }

      if (isAlreadyRegister){
        toRemove = false;
      }

      return toRemove;
  }

  /**
   * add the prefix 'recent:' for a gmail serrver
   *
   * @param msa MailServerAccount
   * @return username
   *
   */
  private String getLogin(MailServerAccount msa){
      
      String userNew     = "";
      
      String type        = msa.getMailServer().getMailServerType();
      String loginFromDB = msa.getMsLogin();
      
      if (type.equalsIgnoreCase(Def.SERVER_TYPE_GMAIL)) {
          // check the prefix
          if (loginFromDB.startsWith("recent:")){
              userNew = loginFromDB;
          } else {
              userNew = "recent:" + loginFromDB;
          }
      } else {
          userNew = loginFromDB;
      }       
      
      return userNew;
      
  }  
  
  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   * @param user user to check
   * @param accounts is the Old List
   * @return return true if if the username is not in the old account list
   *
   */
  private boolean toBeAdded(String msuser, String dsuser, List accountsOld){
      
     String         msuserOld         = null;
     String         dsuserOld         = null;
     String         threadNameOld     = null;
     String         threadNameNew     = null;
     boolean        isAlreadyRegister = false;
     boolean        toAdd             = true;

     for (int i=0; i<accountsOld.size(); i++){
         
        msuserOld = ((MailServerAccount) accountsOld.get(i)).getMsLogin();
        dsuserOld = ((MailServerAccount) accountsOld.get(i)).getUsername();
        
        threadNameOld = createThreadName(msuserOld,dsuserOld);        
        threadNameNew = createThreadName(msuser,dsuser);
        
        if (threadNameOld.equals(threadNameNew)){
           isAlreadyRegister = true;
           break;
        }
        
     }

     if (isAlreadyRegister){
        toAdd = false;
     }
     return toAdd;
  }

  /**
   *
   * checks if the username is already present in the
   * account list to monitor.
   * @param user user to check
   * @param accounts is the Old List
   * @return return true if if the username is not in the old account list
   *
   */
  private boolean toBeUpdated(String msuser, String dsuser, List accountsOld){
      
     String         msuserOld           = null;
     String         dsuserOld           = null;
     String         threadNameOld     = null;
     String         threadNameNew     = null;
     boolean        isAlreadyRegister = false;
     boolean        toUpdate          = false;

     for (int i=0; i<accountsOld.size(); i++){
         
        msuserOld = ((MailServerAccount) accountsOld.get(i)).getMsLogin();
        dsuserOld = ((MailServerAccount) accountsOld.get(i)).getUsername();
        
        threadNameOld = createThreadName(msuserOld,dsuserOld);        
        threadNameNew = createThreadName(msuser,dsuser);
        
        if (threadNameOld.equals(threadNameNew)){
           isAlreadyRegister = true;
           break;
        }
     }

     if (isAlreadyRegister){
        toUpdate = true;
     }
     return toUpdate;
  }  
  
    
  /**
   * return the listener for a specific user
   */
  private MailboxManager userListenerGetter(String user, String dsuser){

     MailboxManager mm = null;
     Thread tobj       = null;
     String tname      = null;
     
     int activeCount = Thread.activeCount();
     ThreadGroup tg = Thread.currentThread().getThreadGroup();
     
     Thread[] list = new Thread[activeCount];     
     tg.enumerate(list);
     
     for (int i=0; i<list.length; i++){
         tobj = (Thread)list[i];
         if (tobj != null){
            tname = tobj.getName();
            if (tname.startsWith(createThreadName(user, dsuser))){
               mm = (MailboxManager)tobj; 
               break;
            }
         }
     }

     return mm;
  }

  
  /**
   * check if there are some account to delete
   * <p>
   * get all ds user
   * <p>
   * get all account
   * <p>
   * make difference
   * <p>
   * remove accounts without ds user
   *
   */
  private void removeAccountsWithoutDSUser(){   
      
       List accountsToRem = null;
       try {            
          accountsToRem = lrdao.getAccountsToRemove(listenerId);
       } catch (Exception e){
          log.error("Error getting all the account IDS to remove " + e.toString());
       }      
       try {  
           String[] account = null;
           for (int i=0; i<accountsToRem.size(); i++){
               account = (String[])accountsToRem.get(i);
               lrdao.removeAccounts(account[0], account[1]);
           }
       } catch (Exception e){
          log.error("Error deleting account IDS without ds-user " + e.toString());
       }      
       
  }
  
  /**
   * the thread name will be like:
   * prefix + ds-server username + mail server login
   * <br>
   * i.e. : IL-john-john.smith@funambol.com
   *
   * @param mail server username
   * @param ds server username
   * @return thread name
   *
   */
  private String createThreadName(String msuser, String dsuser){
      return Def.IL_PREFIX + dsuser + "-" + msuser;
  }
      
}
