/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import java.util.List;
import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.model.MailServer;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.util.Query;
import com.funambol.email.util.TokenUtility;
import com.funambol.email.util.Utility;
import com.funambol.email.util.UtilityIL;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.tools.DBTools;
import java.sql.SQLException;
import java.util.ArrayList;


/**
 *
 * Contains methods to access to the tables with the
 * information about Mail Server Account.
 *
 * @version $Id: ListenerRegistryDAO.java,v 1.24 2007/03/19 14:25:20 gbmiglia Exp $
 */
public class ListenerRegistryDAO {

  // ------------------------------------------------------------ Private data
    
  /**     */
  private FunambolLogger log   = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);

  /**    */
  private DBTools   dbtools    = null;

  /**    */
  private String keystoreHome  = null;
  
  /**    */
  private String encryptionKey = null;
  
  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public ListenerRegistryDAO(String _keystoreHome, String _encryptionKey) {
       this.keystoreHome  = _keystoreHome;
       this.encryptionKey = _encryptionKey;
       this.dbtools = new DBTools();
  }


  // ---------------------------------------------------------- Public Methods



  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccounts (String listenerId)
    throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accounts         = new ArrayList();

         String  username     = null;
         String  password     = null;
                  
         String  listenerID   = null;
         
         int maxNumEmail      = Def.MAX_INBOX_EMAIL_NUMBER;
         String  maxNumEmailS = ""+Def.MAX_INBOX_EMAIL_NUMBER;

         String encryptedPassword = null;
         String plain_text        = null;
        
         try {

             connection = DBConnectionProvider.getConnection();

             ps = connection.prepareStatement(Query.MS_ACCOUNT_GET_USERS_FROM_LISTENER);

             ps.setString(1, listenerId);
             ps.setString(2, "y");

             rs = ps.executeQuery();

             MailServerAccount msa = null;
             MailServer        pms = null;
             
             while(rs.next()) {

                   msa = new MailServerAccount();
                   pms = new MailServer();
                   // account
                   msa.setAccountId(rs.getString(1));     
                   msa.setUsername(rs.getString(2));    
                   msa.setMsLogin(rs.getString(3));     
                   
                   encryptedPassword = rs.getString(4);
                   
                   try {
                       plain_text = 
                            TokenUtility.decryptPassword(encryptedPassword, this.encryptionKey);
                   } catch (Exception e){
                       log.error("Error handling password decryption");
                       throw new Exception("Error handling password decryption");
                   }
                   
                   msa.setMsPassword(plain_text);  
                   
                   msa.setMsAddress(rs.getString(5));  
                   msa.setListenerId(rs.getString(6));  
                   msa.setActivation(Utility.booleanFromString(rs.getString(7))); 
                   msa.setPush(Utility.booleanFromString(rs.getString(8))); 
                   
                   // in the IL we use the millisec but in the DB we save the seconds
                   int sec = rs.getInt(9);
                   msa.setRefreshTime(UtilityIL.setMilliSecondsFromSeconds(sec));
                   
                   msa.setMaxEmailNumber(rs.getInt(10)); 
                   msa.setMaxImapEmail(rs.getInt(11)); 

                   // mail server
                   pms.setMailServerId(rs.getString(12));  
                   pms.setMailServerType(rs.getString(13));
                   pms.setDescription(rs.getString(14));   
                   pms.setProtocol(rs.getString(15));      
                   pms.setOutServer(rs.getString(16)); 
                   pms.setOutPort(rs.getInt(17)); 
                   pms.setOutAuth(Utility.booleanFromString(rs.getString(18))); 
                   pms.setInServer(rs.getString(19)); 
                   pms.setInPort(rs.getInt(20)); 
                   pms.setIsSSLIn(Utility.booleanFromString(rs.getString(21)));                    
                   pms.setIsSSLOut(Utility.booleanFromString(rs.getString(22)));                    
                   pms.setKeystorePath(rs.getString(23));
                   pms.setKeystorePwd(rs.getString(24)); 
                   pms.setInboxPath(rs.getString(25));  
                   pms.setInboxActivation(Utility.booleanFromString(rs.getString(26))); 
                   pms.setOutboxPath(rs.getString(27)); 
                   pms.setOutboxActivation(Utility.booleanFromString(rs.getString(28))); 
                   pms.setSentPath(rs.getString(29));  
                   pms.setSentActivation(Utility.booleanFromString(rs.getString(30))); 
                   pms.setDraftsPath(rs.getString(31));   
                   pms.setDraftsActivation(Utility.booleanFromString(rs.getString(32))); 
                   pms.setTrashPath(rs.getString(33));   
                   pms.setTrashActivation(Utility.booleanFromString(rs.getString(34))); 
                   pms.setIsSoftDelete(Utility.booleanFromString(rs.getString(35))); 
                   pms.setIsPublic(Utility.booleanFromString(rs.getString(36))); 
                   
                   msa.setMailServer(pms);               

                   
                   // set the ssl path if needed
                   if (msa.getMailServer() != null){
                       if (msa.getMailServer().getIsSSLIn()){
                           msa.getMailServer().setKeystorePath(
                               this.keystoreHome + "\\" + msa.getMailServer().getKeystorePath());
                       }
                   }            
                                      
                   accounts.add(msa);

             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error getting Accounts from DB ", e);
         } catch (Exception e) {
             throw new InboxListenerException("Error getting Accounts from DB ", e);
         } finally {
             try {
                 DBTools.close(connection, ps, rs);
             } catch (Exception e){
                 throw new InboxListenerException("Error getting Accounts from DB (" +
                                                  e.getMessage() + ")" );
             }
         }

         return accounts;
         
  }
  
  /**
   * Gets Accounts
   * returns a list that contains MailAccount
   *
   */
  public List getAccountsToRemove (String listenerId)
    throws InboxListenerException {

         Connection connection = null;
         PreparedStatement ps  = null;
         ResultSet rs          = null;
         List accountsToRem    = new ArrayList();
         String[] accounts     = null;

         try {

             connection = DBConnectionProvider.getConnection();

             ps = connection.prepareStatement(Query.MS_ACCOUNT_SELECT_ACCOUNT_TO_REMOVE);

             ps.setString(1, listenerId);

             rs = ps.executeQuery();
                          
             while(rs.next()) {
                 accounts  = new String[2];
                 accounts[0] = rs.getString(1); // account_id
                 accounts[1] = rs.getString(2); // username
                 accountsToRem.add(accounts);     
             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error getting Account IDS to remove from DB ", e);
         } catch (Exception e) {
             throw new InboxListenerException("Error getting Account IDS to remove from DB ", e);
         } finally {
             try {
                 DBTools.close(connection, ps, rs);
             } catch (Exception e){
                 throw new InboxListenerException("Error getting Account IDS to remove from DB (" +
                                                  e.getMessage() + ")" );
             }
         }

         return accountsToRem;
         
  }
  
    /**
     *
     * delete user 
     *
     * @param accountId String
     * @param username String
     * @throws InboxListenerException
     */
    public void removeAccounts(String accountId, String username) 
      throws InboxListenerException {
        
       Connection connection = null;
       PreparedStatement  ps = null;

       try {
                             
          connection = DBConnectionProvider.getConnection();
         
          // delete from fnbl_email_account
          ps = connection.prepareStatement(Query.MS_ACCOUNT_DELETE_USER);
          ps.setString(1, accountId);
          ps.executeUpdate();
          
          // delete from fnbl_email_enable_account
          ps = connection.prepareStatement(Query.MS_ACCOUNT_DELETE_ENABLE_USER);
          ps.setString(1, accountId);
          ps.executeUpdate();
          
          // delete from fnbl_email_inbox
          ps = connection.prepareStatement(Query.INBOX_DELETE_EMAILS);
          ps.setString(1, username);
          ps.executeUpdate();

          ps.close() ;
          ps = null  ;

        } catch (SQLException e) {
             throw new InboxListenerException("Error getting Accounts from DB ", e);
        } catch (Exception e) {
             throw new InboxListenerException("Error getting Accounts from DB ", e);
        } finally {
             try {
                 DBTools.close(connection, ps, null);
             } catch (Exception e){
                 throw new InboxListenerException("Error getting Accounts from DB (" +
                                                  e.getMessage() + ")" );
             }
        }
       
    }
  
    

}
