/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.model.DefaultFolder;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.ImapMailServerWrapper;
import com.funambol.email.transport.PopMailServerWrapper;
import com.funambol.email.util.Def;
import javax.mail.Folder;
import javax.mail.MessagingException;



/**
 * This class groups some utility methods for Mail Server invoking
 *
 * @version $Id: MSTools.java,v 1.11 2007/02/21 17:42:33 gbmiglia Exp $
 */
public class MSTools {
    
   // ------------------------------------------------------------ Private data

   /**    */
   private MailServerAccount msa = null;

   /**    */
   private Folder  inbox     = null;
    
    // ------------------------------------------------------------ Constructors

   /**
     * Creates a new MSTools with the given MSAccount
     * 
     * 
     * 
     * @param _msa the DBAccount
     */
   public MSTools(MailServerAccount _msa) {
        this.msa = _msa;
   }

   // ---------------------------------------------------------- Public Methods
    
   /**
    * open the Mail Server connection; Note the IL Tool
    * works only the INBOX folder
    *
    * @param mswf reference to the mail server wrapper IMailServerWrapper
    * @throws InboxListenerException
    */
   public void openConnection(IMailServerWrapper mswf) 
    throws InboxListenerException {
        
         try {
              DefaultFolder df = new DefaultFolder();
              
              df.setInboxName(msa.getMailServer().getInboxPath());
              df.setInboxId(Def.FOLDER_INBOX_ID);
                
              mswf.openConnection(msa.getMailServer().getMailServerType(),
                                  msa.getMailServer().getOutServer(),
                                  "25",
                                  false,
                                  msa.getMailServer().getInServer(),
                                  ""+msa.getMailServer().getInPort(),
                                  df,
                                  msa.getMsLogin(),
                                  msa.getMsPassword(),
                                  msa.getMailServer().getIsSSLIn(),
                                  msa.getMailServer().getIsSSLOut(),
                                  msa.getMailServer().getKeystorePath(),
                                  msa.getMailServer().getKeystorePwd()); 
                
              if (mswf instanceof ImapMailServerWrapper){
                  // imap
                  ImapMailServerWrapper imsw = (ImapMailServerWrapper) mswf ;
                  this.inbox = (Folder) imsw.folderInboxOpened;
              } else {
                  // pop3
                  PopMailServerWrapper  pmsw = (PopMailServerWrapper) mswf ;
                  this.inbox = (Folder) pmsw.folderInboxOpened;
              }
                           
              if (!this.inbox.isOpen()) {
                  this.inbox.open(Folder.READ_ONLY);
              }
                                
         } catch (MessagingException e) {
             throw new InboxListenerException(
                     "Error opening the Store " + e.getMessage());
         } catch (EmailAccessException e) {
             throw new InboxListenerException(
                     "Error opening the Mail Server Connection " + e.getMessage());
         }
        
   }
    
   /**
     * close the Mail Server connection; Note the IL Tool
     * works only the INBOX folder
     * 
     * @param mswf reference to the mail server wrapper IMailServerWrapper
     * @throws com.funambol.email.exception.InboxListenerException 
     */
   public void closeConnection(IMailServerWrapper mswf) throws InboxListenerException {
         try {
              if (this.inbox != null){
                  if (this.inbox.isOpen()){
                     this.inbox.close(true);
                  }
              }
              if (mswf != null){
                 mswf.releaseConnection();
              }
         } catch (MessagingException e) {
             throw new InboxListenerException(
                     "Error closing the Folder " + e.getMessage());
         } catch (EmailAccessException e) {
             throw new InboxListenerException(
                     "Error closing the Mail Server Connection " + e.getMessage());
         }
    }
    
    
  /**
     * return the inbox folder
     * @return Folder
     */
   public Folder getInbox() {
       return this.inbox;
   }
  
   
}
