/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import java.sql.PreparedStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.SyncItemInfo;
import com.funambol.email.util.Query;
import com.funambol.email.util.Utility;
import com.funambol.framework.tools.DBTools;
import java.sql.SQLException;
import java.util.Date;
import java.util.HashMap;

/**
 * Contains methods to access to the tables with the
 * information about Mail Server Mailbox.
 *
 * @version $Id: MailboxDBDAO.java,v 1.24 2007/01/16 12:13:10 gbmiglia Exp $
 */
public class MailboxDBDAO {


  // ------------------------------------------------------------ Private data


  // ------------------------------------------------------------- Constructor

  /**
   *
   */
  public MailboxDBDAO() {
  }

  // ---------------------------------------------------------- Public Methods

  /**
   *
   * Set the ServerItems into DB
   *
   */
  public void setItems (String username, String protocol, SyncItemInfo[] infos)
    throws InboxListenerException {

         Connection connection        = null;
         PreparedStatement ps         = null;
         ResultSet rs                 = null;
         SyncItemInfo syncItemInfo    = null;
         long            id           = 0;
         String          GUID         = null;
         long            lastCRC      = 0;
         String          messageID    = null;
         Date            headerDate   = null;
         String          headerDateS  = null;
         Date            received     = null;
         String          receivedS    = null;
         String          subject      = null;
         String          sender       = null;
         String          isInvalid    = null;
         String          isInternal   = null;
         String          status       = null;

         try {

             connection = DBConnectionProvider.getConnection();
             connection.setAutoCommit(false);

             // remove data
             ps = connection.prepareStatement(Query.INBOX_DELETE_EMAILS);
             ps.setString  (1, username) ;
             ps.executeUpdate();

             // insert data
             ps = connection.prepareStatement(Query.INBOX_INSERT_MAILBOX_INFO);
                                       
             for (int i=0; i<infos.length; i++ ){
                 
                 syncItemInfo = infos[i];
                 GUID        = syncItemInfo.getKey().getKeyAsString();
                 lastCRC     = syncItemInfo.getLastCrc();
                 messageID   = syncItemInfo.getMessageID();
                 headerDate  = syncItemInfo.getHeaderDate();
                 if (headerDate != null){
                     headerDateS = Utility.UtcFromDate(headerDate);
                 }
                 received    = syncItemInfo.getHeaderReceived();
                 if (received != null){
                     receivedS = Utility.UtcFromDate(received);
                 }
                 subject     = syncItemInfo.getSubject();
                 sender      = syncItemInfo.getSender();
                 isInvalid   = syncItemInfo.getInvalid();
                 isInternal  = syncItemInfo.getInternal();
                 status      = syncItemInfo.getStatus();

                 ps.setString(1, username);
                 ps.setString(2, protocol);
                 ps.setString(3, GUID);
                 ps.setLong  (4, lastCRC);
                 ps.setString(5, isInvalid);
                 ps.setString(6, isInternal);
                 ps.setString(7, messageID);
                 ps.setString(8, headerDateS);
                 ps.setString(9, receivedS);
                 ps.setString(10, subject);
                 ps.setString(11, sender);
                 ps.setString(12, status);
                 
                 ps.executeUpdate();
                 
             }

             connection.commit();

         } catch (SQLException sqle) {
             try {
                connection.rollback();
             } catch (SQLException sqlee){
                throw new InboxListenerException("Error rollbacking", sqlee);
             }
             throw new InboxListenerException("Error refreshing Local Items", sqle);
         } finally {
             try {
                 DBTools.close(connection, ps, rs);
             } catch (Exception e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }

   }
  
  
  /**
   *
   *
   */
  public boolean hasNewMessages(SyncItemInfo[] serverInfos,
                                HashMap localInfos)
    throws InboxListenerException {

        boolean hasNewEmail    = false;

        try {

            if (localInfos.size() == 0 && serverInfos.length > 0){                
                return true;                
            } else if (localInfos.size() < serverInfos.length){
                return true;
            } else {
                hasNewEmail = checkArray(serverInfos, localInfos);
            }

        } catch (Exception me){
            throw new InboxListenerException("Error checking new messages", me);
        }

        return hasNewEmail;
    }
   


  /**
   *
   * Get the Items from DB
   *
   * @return map with all local CrcSyncItemInfo
   *
   */
  //public HashMap getItems (MailServerAccount msa) 
  public HashMap getItems (String username, String protocol) 
    throws InboxListenerException {

         HashMap           items        = new HashMap();
         Connection        connection   = null;
         PreparedStatement ps           = null;
         ResultSet         rs           = null;
         SyncItemInfo      syncItemInfo = null;
         String            guid         = null;
         long              last_crc     = 0;
         String            isInvalid    = null;
         String            isInternal   = null;
         String            messageid    = null;
         Date              headerdate   = null;
         Date              received     = null;
         String            subject      = null;
         String            sender       = null;
         String            status       = null;

         try {

             connection = DBConnectionProvider.getConnection();

             ps = connection.prepareStatement(Query.INBOX_SELECT_ALL_MAILBOX_INFO);

             ps.setString(1, username);
             ps.setString(2, protocol);

             rs = ps.executeQuery();

             while(rs.next()) {
                 
                 guid       = rs.getString(1);
                 last_crc   = rs.getLong(2);
                 isInvalid  = rs.getString(3);
                 isInternal = rs.getString(4);
                 messageid  = rs.getString(5);
                 headerdate = Utility.UtcToDate(rs.getString(6));
                 received   = Utility.UtcToDate(rs.getString(7));
                 subject    = rs.getString(8);
                 sender     = rs.getString(9);
                 status     = rs.getString(10);

                 items.put(guid, new SyncItemInfo(guid, last_crc,
                                                  messageid, headerdate, 
                                                  received, subject, sender,
                                                  isInvalid, isInternal, "Y", status));
             }

         } catch (SQLException e) {
             throw new InboxListenerException("Error setting ServerItems into DataBase", e);
         } finally {
             try {
                 DBTools.close(connection, ps, rs);
             } catch (Exception e){
                 throw new InboxListenerException("Error setting ServerItems into DataBase (" +
                                                  e.getMessage() + ")" );
             }
         }

         return items;

   }
  
  
  //------------------------------------------------------------ Private Methods
  
  /**
   *
   */
   private boolean checkArray(SyncItemInfo[] serverInfos, HashMap localInfos){
      
       boolean hasNewEmail    = false;

       SyncItemInfo lcrcii = null;
       String  GUIDFromMS     = null;
       String  syncLabel      = null;

       for (int i=0; i<serverInfos.length; i++){
            hasNewEmail = false;
            GUIDFromMS  = serverInfos[i].getKey().getKeyAsString();
            syncLabel   = serverInfos[i].getInternal();

            if (syncLabel == null) {  // external email
               if (GUIDFromMS != null){
                    lcrcii = (SyncItemInfo)localInfos.get(GUIDFromMS);
                    if (lcrcii == null){
                        hasNewEmail = true;
                        break;
                    }
               }
           }
       }
        
       return hasNewEmail;
                    
   }
   
}
