/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.util.UtilityIL;
import java.util.List;
import java.util.ArrayList;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.transport.MailServerWrapperFactory;
import com.funambol.email.util.Def;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.msdao.IMailboxMSDAO;
import com.funambol.email.inboxlistener.msdao.MailboxMSDAOFactory;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.SyncItemInfo;
import java.util.Date;
import java.util.HashMap;

/**
 *
 *
 * @version $Id: MailboxManager.java,v 1.27 2007/02/07 09:45:23 gbmiglia Exp $
 */
public class MailboxManager extends Thread {
    
    //------------------------------------------------------------------ CONSTANTS
    
    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    //----------------------------------------------------------------- PROPERTIES
    
    /** owner of the thread */
    private List messageIDs = new ArrayList();
        
    /**   */
    private boolean startupNotification = false ;
    
    /**   */
    private boolean saveSubject = false ;
    
    /**   */
    private boolean saveSender = false ;
    
    /** owner of the thread */
    private MailServerAccount msa;
            
    /** webservices coords */
    private WSAccount wsa;
    
    /** to stop the thread  */
    private boolean go = true ;
    
    /**   */
    private IMailServerWrapper mswf = null;
    
    /**    */
    private IMailboxMSDAO msdao = null;

    
    //--------------------------------------------------------------- CONSTRUCTORS
    
    /**
     *
     */
    public MailboxManager(boolean _startupNotification,
                          boolean _saveSubject,
                          boolean _saveSender,
                          MailServerAccount _msa,
                          WSAccount _wsa) {
        this.startupNotification = _startupNotification;
        this.saveSubject         = _saveSubject;
        this.saveSender          = _saveSender;
        this.msa                 = _msa;
        this.wsa                 = _wsa;
    }
    
    
    //------------------------------------------------------------- PUBLIC METHODS
    
    /**
     *
     *
     * this thread is not always connected to the mailbox.
     * every X seconds (interval) it starts a connection and read the ids
     * in the inbox folder; then compares the previous time read ids.
     * If there are new mails call the Sync4j AdminManager web service
     * in order to trigger the push message
     *
     *
     */
    public void run() {
        
        if (log.isTraceEnabled()) {
            log.trace("START NEW THREAD for : " + msa.toStringForCommandLine() +
                       " startupNotification : " + startupNotification );
        }
        
        boolean hasNew    = false;
        String  p = null;
        String  t = null;
        String  u = null;
        int     firstLoop = 1;        
        long    timeStart = 0;
        long    timeStop  = 0;
        long    totTimeStart = 0;
        long    totTimeStop  = 0;
                        
        MailboxDBDAO dbdao = new MailboxDBDAO();
        
        while (go) {

            // get info
            totTimeStart = System.currentTimeMillis();
            
            try {
                    
                p = this.msa.getMailServer().getProtocol();
                t = this.msa.getMailServer().getMailServerType();
                u = this.msa.getUsername();
                
                mswf  = MailServerWrapperFactory.getMailServerWrapper(p);
                
                msdao = MailboxMSDAOFactory.getMailboxMSDAO(this.msa, 
                                                            this.saveSubject,
                                                            this.saveSender);
                                
                // get local info; map with (key:GUID, value:SyncItemInfo)
                // the items are order by id: from older to newer
                HashMap localInfos = dbdao.getItems(u, p);
                
                // get info in Mail Server
                SyncItemInfo[] serverInfos = msdao.getEmailsFromInbox(mswf, localInfos);
                
                // check new email
                if (this.msa.getPush()){
                    hasNew = dbdao.hasNewMessages(serverInfos,localInfos);
                } else {
                    hasNew = false;
                }
                
                // set info in DB
                dbdao.setItems(u, p, serverInfos);                
                                
            } catch (InboxListenerException e) {
                log.error("Generic error getting items from mail server " + e.toString());
            }
            
            totTimeStop = System.currentTimeMillis();
            if (log.isTraceEnabled()) {
                log.trace("create Local Cache "+msa.getUsername()+" Execution Time: " +
                                       (totTimeStop - totTimeStart) + " ms");
                log.trace("MailboxListener for : " + msa.toStringForCommandLine() +
                           " send Notification: " + hasNew );
            }
            
            // call notification system
            try {
                if (hasNew){
                    if (firstLoop == 1 && !startupNotification){
                        // do nothing
                    } else {
                        NotificationWrapper nw = new NotificationWrapper(msa, wsa);
                        nw.callNotification();
                    }
                }
            } catch (Exception e){
                log.error("Error checking new email for notification " + e.toString());
            }
            
            hasNew    = false;
            firstLoop = 2;
            
            // in the DB there is a second value
            // but the method ListenerRegistryDAO.getAccounts returns
            // the millisec
            try{
                sleep(msa.getRefreshTime());
            } catch (Exception e){
                log.error("Error sleeping .. " + e.toString());
            }
                        
        }
        
        if (log.isTraceEnabled()) {
            log.trace("STOP EXECUTION THREAD for : " + msa.toStringForCommandLine());
        }
                
    }    
    
    /**
     *
     */
    public void setMaxEmailNumber(int _maxEmailNumber){
        this.msa.setMaxEmailNumber(_maxEmailNumber);
    }
    
    /**
     *
     */
    public void setPassword(String _password){
        this.msa.setMsPassword(_password);
    }
    
    /**
     *
     */
    public void setPush(boolean _push){
        this.msa.setPush(_push);
    }
    
    /**
     *
     */
    public void setProtocol(String _protocol) {
        this.msa.getMailServer().setProtocol(_protocol);
    }
    
    /**
     *
     */
    public void setInServer(String _inServer) {
        this.msa.getMailServer().setInServer(_inServer);
    }
    
    /**
     *
     */
    public void setInPort(int _inPort) {
        this.msa.getMailServer().setInPort(_inPort);
    }
    
    
    /**
     *
     */
    public void setIsSSLIn(boolean _isSSLIn) {
        this.msa.getMailServer().setIsSSLIn(_isSSLIn);
    }
        
    /**
     *
     */
    public void setKeystorePath(String _keystorePath) {
        this.msa.getMailServer().setKeystorePath(_keystorePath);
    }
    
    /**
     *
     */
    public void setKeystorePwd(String _keystorePwd) {
        this.msa.getMailServer().setKeystorePwd(_keystorePwd);
    }
    
    
    /**
     * close the Mail Server Connection
     */
    public void closeMailServerConnection() throws InboxListenerException {
        if (log.isTraceEnabled()) {
            log.trace("Shutting down MailboxListener for : " + 
                    msa.toStringForCommandLine() + "(closeMailServerConnection)");
        }
        msdao.closeConnection(mswf);
    }
           
    
    /**
     *
     */
    public void stopThread(){
        if (log.isTraceEnabled()) {
            log.trace("Shutting down MailboxListener for : " + 
                    msa.toStringForCommandLine() + "(stopThread)");
        }
        this.go = false;
    }          
    
}
