/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;

import java.lang.management.ManagementFactory;
import java.sql.SQLException;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.log4j.xml.DOMConfigurator;

import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;

import com.funambol.email.exception.InboxListenerConfigException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.admin.InboxListenerConfig;
import com.funambol.email.util.Def;
import com.funambol.email.util.TokenUtility;

/**
 *
 *
 * @version $Id: MessageProcessor.java,v 1.24 2007/02/27 10:59:47 gbmiglia Exp $
 */
public class MessageProcessor implements MessageProcessorMBean
{

    //------------------------------------------------------------------ CONSTANTS

    /**     */
    protected static FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);

    /**    */
    private static ListenerRegistry lr = null;

    //--------------------------------------------------------------- CONSTRUCTORS

    public MessageProcessor(){
    }

    //------------------------------------------------------------- PUBLIC METHODS

    /**
     * starting method
     *
     * @param args String[]
     */
    public static void main(String args[]) {
        String path = System.getProperty("funambol.inboxlistener.home");
        path = path + "/config/";        
        DOMConfigurator.configureAndWatch(path+"log4j.xml", 30000); // 30 sec.
        MessageProcessor messageProcessor = new MessageProcessor();
        messageProcessor.registerMBean();
        messageProcessor.startApp();
    }

    /**
     * Starts the messages processor
     */
    public void startApp() {
        // load the listener configuration
        String  listenerId          = null;
        boolean startupNotification = false;
        boolean saveSubject         = false;
        boolean saveSender          = false;
        int     reloadingTime       = 1;
        String  keystoreHome        = null;
        try {
            listenerId          = InboxListenerConfig.getConfigInstance().getListenerId();
            startupNotification = InboxListenerConfig.getConfigInstance().getStartupNotification();
            saveSubject         = InboxListenerConfig.getConfigInstance().getSaveSubject();
            saveSender          = InboxListenerConfig.getConfigInstance().getSaveSender();
            reloadingTime       = InboxListenerConfig.getConfigInstance().getReloadingTime();
            keystoreHome        = InboxListenerConfig.getConfigInstance().getKeystoreHome();
        } catch (InboxListenerConfigException e) {
            log.error("Error getting Inbox Listener Configuration in the XML file " + e.toString());
        }


        // load WebServices Properties
        WSAccount wsa       = null;
        String wsEndPoint   = null;
        String wsUsername   = null;
        String wsPassword   = null;
        String wsSyncSource = null;
        try {
            wsEndPoint   = InboxListenerConfig.getConfigInstance().getWsEndPoint();
            wsUsername   = InboxListenerConfig.getConfigInstance().getWsUsername();
            wsPassword   = InboxListenerConfig.getConfigInstance().getWsPassword();
            wsSyncSource = InboxListenerConfig.getConfigInstance().getWsSyncSource();
            wsa          = new WSAccount(wsEndPoint, wsUsername, wsPassword, wsSyncSource);
        } catch (InboxListenerConfigException e) {
            log.error("Error getting Configuration using WS " + e.toString());
        }

        // load DataBase Properties ad create the DB pool for all threads
        // the DBConnectionProvider is a singleton
        BasicDataSource basicDataSource = null;
        try {
            basicDataSource = InboxListenerConfig.getConfigInstance().getBasicDataSource();
            DBConnectionProvider.configure(basicDataSource);
        } catch (InboxListenerConfigException e) {
            log.error("Error getting Database Configuration " + e.toString());
        } catch (SQLException e) {
            log.error("Error getting Database Configuration " + e.toString());
        }

        String  encryptionKey = TokenUtility.getToken();


        if (log.isTraceEnabled()) {
            log.trace("Start Inbox Listener "                          +
                    "\nInboxListener Parameters"                     +
                    "\nlistener id         : " + listenerId          +
                    "\nstartupNotification : " + startupNotification +
                    "\nsaveSubject         : " + saveSubject         +
                    "\nsaveSender          : " + saveSender          +
                    "\nreloadingTime       : " + reloadingTime       +
                    "\nkeystoreHome        : " + keystoreHome        +
                    "\nWebServices Parameters"                       +
                    "\nwsEndPoint   : " + wsa.getWsEndPoint()        +
                    "\nwsUsername   : " + wsa.getWsUsername()        +
                    "\nwsPassword   : ****"                          +
                    "\nwsSyncSource : " + wsa.getWsSyncSource()      );
        }

        // register the shutdown hook
        Runtime.getRuntime().addShutdownHook(new Thread() {
            public void run() {
                endApp();
            }
        });;

        // start listener registry
        lr = new ListenerRegistry(listenerId,
                startupNotification,
                saveSubject,
                saveSender,
                reloadingTime,
                keystoreHome,
                encryptionKey,
                wsa);
        lr.setName("ListenerRegistry-"+listenerId);
        lr.start();

        System.out.println("Inbox-Listener started ...");

    }

    /**
     * close all the connection
     */
    public static void endApp() {
        if (log.isTraceEnabled()) {
            log.trace("END APP TASKS");
        }
        try {
            lr.closeMailboxManager();
        } catch (InboxListenerException ile) {
            log.error("Error closing application " + ile.toString());
        }
        if (log.isTraceEnabled()) {
            log.trace("End Closing Mail Server Connection");
        }
        try {
            DBConnectionProvider.close();
        } catch (SQLException sqle) {
            log.error("Error closing db connection pool " + sqle.toString());
        }
    }

    /**
     * Stops the push listener
     */
    public void stop() {
        System.exit(-1);
    }

    // --------------------------------------------------------- Private methods

    /**
     * Register the MessageProcessor instance as MBEAN
     */
    private void registerMBean() {
        if (log.isInfoEnabled()) {
            log.info("Registering MBean " + MBEAN_NAME);
        }
        MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
        try {

            // Construct the ObjectName for the MBean we will register
            ObjectName name = new ObjectName(MBEAN_NAME);

            mbs.registerMBean(this, name);
        } catch (Exception ex) {
            log.error("Error registering mbean '" + MBEAN_NAME + "'", ex);
        }
    }

}
