/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.engine;


import com.funambol.email.model.MailServerAccount;
import com.funambol.email.util.Def;
import com.funambol.framework.core.Alert;
import com.funambol.framework.core.Target;
import com.funambol.framework.core.Meta;
import com.funambol.framework.core.Item;
import com.funambol.framework.core.CmdID;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.tools.beans.BeanFactory;


/**
 *
 * @version $Id: NotificationWrapper.java,v 1.10 2007/01/15 16:37:39 nichele Exp $
 */
public class NotificationWrapper
{

  //----------------------------------------------------------------- Properties
    
  /**     */
  protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
  
  /**   */
  private WSAccount wsa         = null;
  
  /**   */
  private MailServerAccount msa = null;
  
  /**   */
  private WSTools wsTools       = null;


  //--------------------------------------------------------------- CONSTRUCTORS


  /**
   *
   *
   * @param msa MailServerAccount
   * @param wsa WSAccount
   */
  public NotificationWrapper(MailServerAccount msa, WSAccount wsa){
       this.msa = msa;
       this.wsa = wsa;
  }

  //------------------------------------------------------------- PUBLIC METHODS

   /**
    *
    *
    * @throws Exception
    */
   public void callNotification() throws Exception {
      wsTools  = new WSTools(wsa.getWsEndPoint(),
                             wsa.getWsUsername(),
                             wsa.getWsPassword());
      sendNotificationMessages(msa.getUsername());
   }


  /**
   *
   *
   * @param deviceId String
   * @throws Exception
   */
  public void sendNotificationMessages(String deviceId) throws Exception  {

        Alert[] alerts = createMailAlerts();
        String xml = getXMLAlerts(alerts);

        if (log.isTraceEnabled()) {
            log.trace(xml);
        }

        Object[] params = new Object[] {deviceId, xml, new Integer(1)};

        wsTools.invoke("sendNotificationMessages", params);
  }


  //------------------------------------------------------------ PRIVATE METHODS

  /**
   *
   *
   * @return Alert[]
   */
  private Alert[] createMailAlerts() {

      Target t1 = new Target(wsa.getWsSyncSource());
      Meta meta1 = new Meta();
      meta1.setType("application/vnd.omads-email+xml");
      Item i1 = new Item(null, t1,meta1, null, false);

      Alert alert1 = new Alert(new CmdID(0), false, null, 206, new Item[]{i1});

      Alert[] alerts = new Alert[1];
      alerts[0] = alert1;

      return alerts;
  }

  /**
   *
   *
   * @param alerts Alert[]
   * @return String
   * @throws Exception
   */
  private String getXMLAlerts(Alert[] alerts) throws Exception  {
      return BeanFactory.marshal(alerts);
  }

}
