/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.msdao;


import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.inboxlistener.engine.MSTools;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.SyncItemInfo;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.util.UtilityDate;
import com.sun.mail.imap.IMAPFolder;
import com.sun.mail.pop3.POP3Folder;
import java.util.HashMap;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.zip.CRC32;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;



/**
 * This class groups some utility methods for Mail Server invoking
 *
 * @version $Id: MailboxMSDAOCommon.java,v 1.8 2007/03/16 13:39:28 gbmiglia Exp $
 */
public abstract class MailboxMSDAOCommon  implements IMailboxMSDAO {

   // --------------------------------------------------------------- Constants

    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);

   // ------------------------------------------------------------ Private data

   /**    */
   protected MailServerAccount msa = null;

   /**    */
   protected MSTools mstools       = null;

   /**    */
   protected boolean saveSubject   = false;
   
   /**    */
   protected boolean saveSender    = false;
   
   // ------------------------------------------------------------ Constructors

   public MailboxMSDAOCommon() {
   }
   
   /**
     * Creates a new MSTools with the given MSAccount
     *
     *
     *
     * @param _msa the DBAccount
     */
   public MailboxMSDAOCommon(MailServerAccount _msa, boolean _saveSubject, boolean _saveSender) {
        this.msa = _msa;
        this.mstools = new MSTools(_msa);
        this.saveSubject = _saveSubject;
        this.saveSender  = _saveSender;
   }

   // ---------------------------------------------------------- Public Methods

   /**
    *
    */
   public void setMaxEmailNumber(int _maxEmailNumber){
       this.msa.setMaxEmailNumber(_maxEmailNumber);
   }

   /**
    *
    */
   public int getMaxEmailNumber(){
       return this.msa.getMaxEmailNumber();
   }
   
   /**
    *
    */
   public void setPassword(String _password){
       this.msa.setMsPassword(_password);
   }
    
    /**
     *  close the Mail Server Connection
     */
    public void closeConnection(IMailServerWrapper mswf)
      throws InboxListenerException {
         mstools.closeConnection(mswf);
    }
   
   /**
     * get messages from inbox folder
     * This method returns all the items in the server
     * inbox folder
     *
     * @param mswf IMailServerWrapper
     * @param localInfos map with all the loacal CrcSyncItemInfo
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
   public abstract SyncItemInfo[] getEmailsFromInbox(IMailServerWrapper mswf,
                                                HashMap localInfos)
      throws InboxListenerException ;
    
    
   //---------------------------------------------------------Protected Methods
        
   /**
    *
    *
    * @param messages Message[]
    * @param localInfos map with all the loacal SyncItemInfo
    * @return list with all the server SyncItemInfo
    * @throws MessagingException
    */
   protected ArrayList createInfoList(Message[] messages, HashMap localInfos) 
    throws MessagingException {
        
        ArrayList    allSyncItemInfo   = new ArrayList();
        SyncItemInfo syncItemInfo      = null;       
        SyncItemInfo localSyncItemInfo = null;
                
        
        // add the new server info
        for (int i = 0; i < messages.length; i++) {
            
            // this method can return null if the email is a SentEmail
            syncItemInfo = createInfo(messages[i], localInfos);
            
            if (syncItemInfo != null){
                // insert in the list if there is some condiftion
                if (!isMessageIDPresent(syncItemInfo.getMessageID(), allSyncItemInfo)){
                    allSyncItemInfo.add(syncItemInfo);                                
                }
            }
                        
        }
         
        return allSyncItemInfo; 
        
    }
    
    /**
     *
     * @param message Message
     * @param localInfos map with all the loacal SyncItemInfo
     * @return SyncItemInfo
     * @throws MessagingException
     */
    protected SyncItemInfo createInfo(Message message, HashMap localInfos) 
    throws MessagingException {
        
        SyncItemInfo lcrcii          = null;       
        SyncItemInfo syncItemInfo    = null;       
        
        String       GUID            = null;
        String       userAddress     = null;
        
        long         lastCRC         = 0;
        String       messageID       = null;
        Date         headerDate      = null;
        Date         received        = null;
        String       subject         = null;
        String       sender          = null;
        String       status          = null;
        
        long         lastCRCold      = 0;
        String       messageIDold    = null;
        Date         headerDateold   = null;
        Date         receivedold     = null;
        String       subjectold      = null;
        String       senderold       = null;
        String       statusold       = null;
        

        GUID        = this.createGUID(message);
        userAddress = this.msa.getMsAddress();

        if (GUID != null){
            
            boolean skipMsg = false;
            String type = this.msa.getMailServer().getMailServerType();
            if (type.equals(Def.SERVER_TYPE_GMAIL) && isSentEmail(userAddress, message)){
                skipMsg = true;
            }
            if (isCorruptedEmail(message)){
                skipMsg = true;
            }
            
            if (!skipMsg){
                // save the old value
                lcrcii = (SyncItemInfo)localInfos.get(GUID);

                if (lcrcii != null){
                    messageIDold  = lcrcii.getMessageID();
                    headerDateold = lcrcii.getHeaderDate();
                    lastCRCold    = lcrcii.getLastCrc();
                    receivedold   = lcrcii.getHeaderReceived();
                    subjectold    = lcrcii.getSubject();
                    senderold     = lcrcii.getSender();
                    statusold     = lcrcii.getStatus();
                }

                // create the new values
                // Message-ID
                try {
                    messageID  = Utility.getHeaderMessageID(message);
                } catch (Exception e){
                    log.error("Error parsing creating Message-ID for Caching System ", e);
                    if (lcrcii != null){
                       messageID  = messageIDold;
                    }
                }
                // Date
                try {
                    // REVIEWED THE FILTER TIME STRATEGY
                    //headerDate = UtilityDate.getHeaderDate(message,null);                    
                    headerDate = UtilityDate.getDateForTimeFilter(message,null);                    
                } catch (Exception e){
                    log.error("Error parsing creating new Date for Caching System ", e);
                    if (lcrcii != null){
                       headerDate = headerDateold;
                    }   
                }
                // CRC
                try {
                    String dateForCRC = Utility.getHeaderDateForCRC(message);
                    lastCRC = Utility.getLastCrc(message, messageID, dateForCRC);
                } catch (Exception e){
                    log.error("Error parsing creatingCRC for Caching System ", e);
                    if (lcrcii != null){
                       lastCRC    = lastCRCold;
                    }   
                }
                // Received
                try {
                    received   = UtilityDate.getHeaderReceived(message);
                } catch (Exception e){
                    log.error("Error parsing creating Received for Caching System " + e.getMessage());
                    if (lcrcii != null){
                       received   = receivedold;
                    }   
                }
                // Subject
                try {
                    if (this.saveSubject){
                        subject    = Utility.getHeaderSubject(message);
                    } else {
                        subject    = "";
                    }    
                } catch (Exception e){
                    log.error("Error parsing creating Subject for Caching System ", e);
                    if (lcrcii != null){
                       subject    = subjectold;
                    }   
                }
                // Sender
                try {
                    if (this.saveSender){
                        sender = Utility.getHeaderSender(message);
                    } else {
                        sender = "";
                    }
                } catch (Exception e){
                    log.error("Error parsing creating Sender for Caching System ", e);
                    if (lcrcii != null){
                       sender     = senderold;
                    }   
                }

                // status
                if (lcrcii != null){
                    if (statusold != null){
                        status = statusold;
                    } else {
                        status = "N";
                    }
                } else {
                    status = "N";
                }  
                               
                syncItemInfo = new SyncItemInfo(GUID, lastCRC, messageID, headerDate, 
                                                received, subject, sender, 
                                                null, null, "Y", status);
                
            }
                        
        }
                 
        return syncItemInfo; 
        
   }    
   
   
   //------------------------------------------------------------Private Methods
    
   /**
     *
     */
    protected Message[] getMessages(int max) 
      throws MessagingException {
        
        if (log.isTraceEnabled()) {
            log.trace("get Messages from Inbox Mailbox number : " + max);
        }        
        
        Message[] messages = null;
        
        int mNumber  = this.mstools.getInbox().getMessageCount();
        
        if (mNumber > max){
            messages = this.mstools.getInbox().getMessages((mNumber-(max-1)),mNumber);            
        } else {
            messages = this.mstools.getInbox().getMessages();
        }
        
        return messages;
    }
    
    /**
     * the message is a sent email if the userAddress is in the From list 
     * and is not in the To list
     *
     * @param userAddress String
     * @param message Message
     * @return true if is a sent email
     */
    protected boolean isSentEmail(String userAddress, Message message) 
     throws MessagingException {
        
        boolean isSentEmail = false;
        boolean isInFROM    = false;
        boolean isInTO      = false;
        int index = -1;
        
        String sbj = "sorry, subject NOT available!";        
        
        Address[] froms = message.getFrom();
        if (froms != null){
            String from = "";
            index = -1;
            for (int i=0; i<froms.length; i++){
                from  = froms[i].toString();
                index = from.indexOf(userAddress);
                if (index != -1){
                    isInFROM = true;
                    break;
                }
            }
        } else {
            // if there is somethig wrong .. anyways it returns the email 
            try {
               sbj = Utility.getHeaderSubject(message);
            } catch (Exception e) {
                // do nothing
            }
            log.error("Error checking FORM list for the email: " + sbj);
            isInFROM = true;
        }
                
        Address[] tos = message.getRecipients(Message.RecipientType.TO);
        if (tos != null){
            String to = "";
            index = -1;
            for (int i=0; i<tos.length; i++){
                to = tos[i].toString();
                index = to.indexOf(userAddress);
                if (index != -1){
                    isInTO = true;
                }
            }
        } else {
            // if there is somethig wrong .. anyways it returns the email 
            try {
               sbj = Utility.getHeaderSubject(message);
            } catch (Exception e) {
                // do nothing
            }
            log.error("Error checking TO list for the email: " + sbj);
            isInTO = false;
        }
                
        if (isInFROM && !isInTO){
            isSentEmail = true;
        }
            
        return isSentEmail;        
    }
    
    /**
     * the message can be all the part empty. it's a problem happened with
     * the Zimbra Mail Server
     *
     * @param message Message
     * @return true if is a corrupted email
     */
    protected boolean isCorruptedEmail(Message message) 
     throws MessagingException {
        
        boolean isCorruptedEmail = false;
        String from    = null;
        String to      = null;
        String id      = null;
        String date    = null;
        String subject = null;
        try {
            
            from    = Utility.getHeaderSender(message);      // ok returns null
             
            to      = Utility.getHeaderReceiver(message);    // ok returns null
            
            id      = Utility.getHeaderMessageID(message);   // ok returns null
            
            date    = Utility.getHeaderDateForCRC(message);  //    returns ""
            if (date != null){
                if (date.equals("")){
                    date = null;
                }
            }
            
            subject = Utility.getHeaderSubject(message);   // ok returns null
            
            if (from    == null &&
                to      == null &&
                id      == null &&
                subject == null && 
                date    == null ){
                isCorruptedEmail = true;
            }
                
        } catch (Exception e){
            // do nothing
        }
        
        return isCorruptedEmail;        
        
    }
    
    /**
     *
     */
    protected String createGUID(Message msg) throws MessagingException {

        String GUID = null;
        String FMID = "";
        String UIDV = "";

        if (this.msa.getMailServer().getProtocol().equals(Def.PROTOCOL_IMAP)){
           FMID = ""+((IMAPFolder)this.mstools.getInbox()).getUID(msg);
           UIDV = ""+((IMAPFolder)this.mstools.getInbox()).getUIDValidity();
           GUID = Utility.createIMAPGUID(Def.FOLDER_INBOX_ID, FMID, UIDV);
        } else {
           FMID = ((POP3Folder)this.mstools.getInbox()).getUID(msg);
           GUID = Utility.createPOPGUID(Def.FOLDER_INBOX_ID, FMID);
        }
        return GUID;
    }
    
    /**
     *
     */
    protected boolean isMessageIDPresent(String messageID, 
                                       List allSyncItemInfo) {
        
        String messageIDTmp = null;
        boolean check = false;
        
        if (messageID != null) {
            for (int i=0; i<allSyncItemInfo.size(); i++){
                messageIDTmp = ((SyncItemInfo)allSyncItemInfo.get(i)).getMessageID();
                if (messageID.equals(messageIDTmp)){
                    check = true;
                    break;
                }
            }
        }        
        
        return check;        
    }    
       
}
