/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.inboxlistener.msdao;


import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.InboxListenerException;
import com.funambol.email.model.MailServerAccount;
import com.funambol.email.model.SyncItemInfo;
import com.funambol.email.transport.IMailServerWrapper;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import com.funambol.email.util.UtilityDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import javax.mail.Address;
import javax.mail.Message;
import javax.mail.MessagingException;



/**
 * This class groups some utility methods for Mail Server invoking
 *
 * @version $Id: MailboxMSDAOGmail.java,v 1.6 2007/02/19 11:41:59 gbmiglia Exp $
 */
public class MailboxMSDAOGmail extends MailboxMSDAOCommon implements IMailboxMSDAO {


   // ------------------------------------------------------------ Constructors

   public MailboxMSDAOGmail() {
   }
   
   /**
     * Creates a new MSTools with the given MSAccount
     *
     * @param _msa the DBAccount
     */
   public MailboxMSDAOGmail(MailServerAccount _msa, boolean _saveSubject, boolean _saveSender) {
       super(_msa, _saveSubject, _saveSender);
   }

   // ---------------------------------------------------------- Public Methods


   
   /**
     * get messages from inbox folder
     * This method returns all the items in the server
     * inbox folder
     *
     * @param mswf IMailServerWrapper
     * @param localInfos map with all the loacal CrcSyncItemInfo
     * @return CrcSyncItemInfo[]
     * @throws EntityException
     */
   
    public SyncItemInfo[] getEmailsFromInbox(IMailServerWrapper mswf,
                                             HashMap localInfos)
      throws InboxListenerException {

        SyncItemInfo syncItemInfo    = null;
        ArrayList       allSyncItemInfo = new ArrayList();


        try {
            
            mstools.openConnection (mswf);

            int max = this.msa.getMaxEmailNumber();

            // get max Messages
            Message[] messages = getMessages(max);

            // create list with all SyncItemInfo
            if (messages != null) {
                allSyncItemInfo = createInfoList(messages, localInfos);
            }

            mstools.closeConnection(mswf);

        } catch (MessagingException e) {
            throw new InboxListenerException(e.getMessage());
        }

        return (SyncItemInfo[]) allSyncItemInfo.toArray(new SyncItemInfo[0]);

    }
    
    
    //---------------------------------------------------------Protected Methods     
     
    /**
     *
     * @param message Message
     * @param localInfos map with all the loacal SyncItemInfo
     * @return SyncItemInfo
     * @throws MessagingException
     */
    /*
    protected SyncItemInfo createInfo(Message message, HashMap localInfos) 
    throws MessagingException {
        
        SyncItemInfo lcrcii          = null;       
        SyncItemInfo syncItemInfo    = null;       
        
        String       GUID            = null;
        String       userAddress     = null;
        
        long         lastCRC         = 0;
        String       messageID       = null;
        Date         headerDate      = null;
        Date         received        = null;
        String       subject         = null;
        String       sender          = null;
        String       status          = null;
        
        long         lastCRCold      = 0;
        String       messageIDold    = null;
        Date         headerDateold   = null;
        Date         receivedold     = null;
        String       subjectold      = null;
        String       senderold       = null;
        String       statusold       = null;
        

        GUID        = this.createGUID(message);
        userAddress = this.msa.getMsAddress();

        if (GUID != null){
            
            if (!isSentEmail(userAddress, message)){
                // it's not a sent email
                // save the old value
                lcrcii = (SyncItemInfo)localInfos.get(GUID);

                if (lcrcii != null){
                    messageIDold  = lcrcii.getMessageID();
                    headerDateold = lcrcii.getHeaderDate();
                    lastCRCold    = lcrcii.getLastCrc();
                    receivedold   = lcrcii.getHeaderReceived();
                    subjectold    = lcrcii.getSubject();
                    senderold     = lcrcii.getSender();
                    statusold     = lcrcii.getStatus();
                }

                // create the new values
                // Message-ID
                try {
                    messageID  = Utility.getHeaderMessageID(message);
                } catch (Exception e){
                    log.error("Error parsing creating Message-ID for Caching System ", e);
                    if (lcrcii != null){
                       messageID  = messageIDold;
                    }
                }
                // Date
                try {
                    headerDate = UtilityDate.getHeaderDate(message,null);                    
                } catch (Exception e){
                    log.error("Error parsing creating new Date for Caching System ", e);
                    if (lcrcii != null){
                       headerDate = headerDateold;
                    }   
                }
                // CRC
                try {
                    lastCRC    = Utility.getLastCrc(message, messageID, ""+headerDate);
                } catch (Exception e){
                    log.error("Error parsing creatingCRC for Caching System ", e);
                    if (lcrcii != null){
                       lastCRC    = lastCRCold;
                    }   
                }
                // Received
                try {
                    received   = UtilityDate.getHeaderReceived(message);
                } catch (Exception e){
                    log.error("Error parsing creating Received for Caching System " + e.getMessage());
                    if (lcrcii != null){
                       received   = receivedold;
                    }   
                }
                // Subject
                try {
                    if (this.saveSubject){
                        subject    = Utility.getHeaderSubject(message);
                    } else {
                        subject    = "";
                    }    
                } catch (Exception e){
                    log.error("Error parsing creating Subject for Caching System ", e);
                    if (lcrcii != null){
                       subject    = subjectold;
                    }   
                }
                // Sender
                try {
                    if (this.saveSender){
                        sender = Utility.getHeaderSender(message);
                    } else {
                        sender = "";
                    }
                } catch (Exception e){
                    log.error("Error parsing creating Sender for Caching System ", e);
                    if (lcrcii != null){
                       sender     = senderold;
                    }   
                }

                // status
                if (lcrcii != null){
                    if (statusold != null){
                        status = statusold;
                    } else {
                        status = "N";
                    }
                } else {
                    status = "N";
                }  

                syncItemInfo = new SyncItemInfo(GUID, lastCRC, messageID, headerDate, 
                                                received, subject, sender, 
                                                null, null, "Y", status);
            
            }

        }
                 
        return syncItemInfo; 
        
    }    
    */    
    
    
}
