/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.dao;


import com.funambol.email.exception.EntityException;
import com.funambol.email.model.EmailFilter;
import com.funambol.email.model.ItemMessage;
import com.funambol.email.util.Def;
import com.funambol.email.util.DescendingComparator;
import com.funambol.email.util.Query;
import com.funambol.email.util.Utility;
import com.funambol.framework.tools.DBTools;
import com.funambol.framework.tools.DataSourceTools;
import com.funambol.server.tools.id.DBIDGenerator;
import com.sun.mail.pop3.POP3Folder;

import javax.mail.Flags;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;
import java.util.TreeMap;

/**
 * This class implements methods to access data
 * in Mail Server datastore.
 *
 * @version $Id: PopEntityDAO.java,v 1.43 2007/02/19 11:42:17 gbmiglia Exp $
 */
public class PopEntityDAO extends EntityDAO {

    // ------------------------------------------------------------ Constructors

    /**
     *
     */
    public PopEntityDAO() {
    }

    /**
     * @param jndiDataSourceName String
     */
    public PopEntityDAO(String jndiDataSourceName, String serverType) {
        this.jndiDataSourceName = jndiDataSourceName;
        this.serverType         = serverType;
    }

    // -----------------------------------------------------------Public Methods

    /**
     * insert an email in the fnbl_email_sentpop table 
     *
     * @param FID         IMAPFolder ID
     * @param messageID   header Message-ID
     * @param msg         Message
     * @param idSentSpace The GUID generator
     * @param source_uri  The SyncSource URI.
     * @param principalId The principal ID.
     * @return GUID String
     * @throws EntityException
     */
    public String addSentEmail(String FID,
                               String messageID,
                               Message msg,
                               DBIDGenerator idSentSpace,
                               String source_uri,
                               long principalId) throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("DAO start addItemInLocalDB folder " + FID);
        }

        String GUID = null;
        Connection connection = null;
        PreparedStatement ps = null;

        try {
            DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_INSERT_SENT);

            GUID = Utility.createPOPGUID(FID, ""+idSentSpace.next());

            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.setBytes(4, mail2db(msg));
            ps.setString(5, messageID);
            
            ps.executeUpdate();
            ps.close();
            ps = null;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        if (log.isTraceEnabled()) {
            log.trace("DAO end insertItem UID " + GUID);
        }

        return GUID;
    }


    /**
     * gets all Messages
     * NOTE: in the pop protocol this feature doesn't work
     * <br>SearchTerm st = new ReceivedDateTerm(ComparisonTerm.GE, filter.getTime());
     * <br>messages = f.search(st);
     * <br>so we have to use a manual filter
     *
     * @param f POP3Folder
     * @param filter EmailFilter
     * @param loc Locale
     * @return Message[]
     * @throws EntityException
     */
    /*
    public Message[] getAllEmailsInbox(POP3Folder f,
                                       EmailFilter filter,
                                       Locale loc)
            throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("DAO start getAllEmailsInbox - " + 
                       " filter ID: " + filter.getId() + 
                       " filter DATE: " + filter.getTime());
        }
        
        Message[] messages = null;

        try {

            
            if (filter.getId() != null) {
                String uid  = Utility.getKeyPart(filter.getId(),2);
                Message[] tmpmessages = f.getMessages();
                if (tmpmessages != null && tmpmessages.length > 0){
                    Message msg = getMessageByUID(f, tmpmessages, uid);
                    messages    = new Message[1];
                    messages[0] = msg;
                }
            } else {
                if (filter.getTime() != null) {
                    Message[] tmpmessages = f.getMessages();
                    messages = this.getTimeFilteredItems(tmpmessages,filter,loc);
                } else {
                    messages = f.getMessages();
                }
            }
        }
        catch (MessagingException me) {
            throw new EntityException(me);
        }

        if (log.isTraceEnabled()) {
            if (messages != null) {
                log.trace("DAO end getAllEmailsInbox - number of items " + messages.length);
            } else {
                log.trace("DAO end getAllEmailsInbox - number of items 0");
            }
        }            

        return messages;

    }
     */
    
    /**
     * gets all Messages
     * NOTE: in the pop protocol this feature doesn't work
     * <br>SearchTerm st = new ReceivedDateTerm(ComparisonTerm.GE, filter.getTime());
     * <br>messages = f.search(st);
     * <br>so we have to use a manual filter
     *
     * @param f POP3Folder
     * @return Message[]
     * @throws EntityException
     */
    public Message[] getAllEmailsInbox(POP3Folder f)
      throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("DAO start getAllEmailsInbox ");
        }
        
        Message[] messages;

        try {
             messages = f.getMessages();
        }
        catch (MessagingException me) {
            throw new EntityException(me);
        }

        if (log.isTraceEnabled()) {
            if (messages != null) {
                log.trace("DAO end getAllEmailsInbox - number of items " + messages.length);
            } else {
                log.trace("DAO end getAllEmailsInbox - number of items 0");
            }
        }

        return messages;

    }    
    
    /**
     * @param filter
     * @param source_uri  the SyncSource URI.
     * @param principalId The principal ID.
     * @return The items fulfilling the filter requirements as an Array of Strings.
     * @throws EntityException
     */
    public String[] getAllEmailsSent(EmailFilter filter,
                                     String source_uri,
                                     long principalId)
            throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("DAO start getAllEmailsSent ");
        }

        String[] ids = null;
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs;
        ArrayList rows = new ArrayList();;
        
        try {
            DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_SELECT_ALL_SENT);
                        
            ps.setString(1, source_uri);
            ps.setLong(2, principalId);
            ps.setString(3, Def.FOLDER_SENT_ID + Def.SEPARATOR_FIRST + "%");
            
            rs = ps.executeQuery();
            
            rows = this.createSentList(rs, filter.getMaxSentNum());
            
            ids = (String[]) rows.toArray(new String[0]);
            
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }
        if (ids != null) {
            if (log.isTraceEnabled()) {
                log.trace("number of items " + ids.length);
            }
        }
        
        if (log.isTraceEnabled()) {
            log.trace("DAO end getAllItems ");
        }

        return ids;

    }

    
    /**
     * @param msg Message
     * @throws EntityException
     */
    public void removeEmail(Message msg) throws EntityException {
        try {
            msg.setFlag(Flags.Flag.DELETED, true);
        } catch (MessagingException me) {
            throw new EntityException(me);
        }

    }

    /**
     * @param f   IMAPFolder
     * @param uid long
     * @throws EntityException
     */
    public void removeEmail(POP3Folder f, String uid)
      throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("POP DAO start removePopItem ");
        }
        
        try {
            Message[] messages = f.getMessages();            
            if (messages != null && messages.length > 0){
                Message msg = getMessageByUID(f, messages, uid);
                if (msg != null) {
                    msg.setFlag(Flags.Flag.DELETED, true);
                }
            }            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
    }    
    
    /**
     * @param f POP3Folder
     * @throws EntityException
     */
    public void removeAllEmail(POP3Folder f)
            throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("POP3 DAO start removeAllEmail ");
        }

        Message[] messages;
        try {

            messages = f.getMessages();

            for (int i = 0; i < messages.length; i++) {
                messages[i].setFlag(Flags.Flag.DELETED, true);
            }

        } catch (MessagingException e) {
            throw new EntityException(e);
        }

    }

    /**
     * serch a document using UID; for POP3  protocol UID is String
     * I have to use a custome getMessageByUID because 
     * there is not a folder.getMessageByUID method like in the imap protocol
     *
     * @param f   POP3Folder
     * @param uid String
     * @return Message
     * @throws EntityException
     */
    public Message getEmailInboxFromUID(POP3Folder f, 
                                        Message[] messages, 
                                        String uid)
      throws EntityException {
        Message msg = null;
        try {                   
            if (messages != null && messages.length > 0){
               msg = getMessageByUID(f, messages, uid);
            }            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        return msg;
    }

    /**
     * @param GUID        the Global Unique ID.
     * @param s           The session info to the backend.
     * @param source_uri  the SyncSource URI.
     * @param principalId The principal ID.
     * @return Message
     * @throws EntityException
     */
    public Message getEmailSentFromUID(String GUID,
                                       Session s,
                                       String source_uri,
                                       long principalId)
      throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("POPDAO start getItemByUIDInSent GUID: " + GUID);
        }

        Message message = null;
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs;

        try {
            DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_SELECT_SENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            rs = ps.executeQuery();
            while (rs.next()) {
                message = db2mail(getBytesFromColumn(1, rs), s);
            }
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        if (log.isTraceEnabled()) {
            log.trace("POPDAO end getItemByUIDInSent ");
        }

        return message;

    }

    /**
     * @param GUID        the Global Unique ID.
     * @param source_uri  the SyncSource URI.
     * @param principalId The principal ID.
     * @throws EntityException
     */
    public void removeEmail(String GUID,
                            String source_uri,
                            long principalId)
            throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("POP DAO start removeItemInSent UID " + GUID);
        }
        
        Connection connection = null;
        PreparedStatement ps = null;
        try {
            DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_DELETE_SENT);
            ps.setString(1, GUID);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            ps.executeUpdate();
            ps.close();
            ps = null;
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        if (log.isTraceEnabled()) {
            log.trace("POP DAO end removeItemInSent UID " + GUID);
        }

    }

    /**
     * serch a document using the header Message-ID
     *
     * @param message_id  String
     * @param s           Session
     * @param source_uri  String
     * @param principalId long
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage getEmailFromMessageID(String message_id,
                                             Session s,
                                             String source_uri,
                                             long principalId)
            throws EntityException {

        if (log.isTraceEnabled()) {
            log.trace("DAO start getItemByMessageIDInSent Header Message-ID: " +
                    message_id);
        }

        ItemMessage eisMessage = null;
        String GUID = null;
        Message javaMessage = null;
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs;

        try {
            DataSource dataSource = DataSourceTools.lookupDataSource(this.jndiDataSourceName);
            connection = dataSource.getConnection();
            ps = connection.prepareStatement(Query.SENT_SELECT_TWIN_SENT);
            ps.setString(1, message_id);
            ps.setString(2, source_uri);
            ps.setLong(3, principalId);
            rs = ps.executeQuery();
            while (rs.next()) {
                GUID = rs.getString(1);
                javaMessage = db2mail(getBytesFromColumn(2, rs), s);
            }
            eisMessage = new ItemMessage(GUID, "S", message_id, javaMessage,
                                         null, null, null, null, null, null,
                                         null, null, null, null, null);
        } catch (Exception e) {
            throw new EntityException(e);
        } finally {
            DBTools.close(connection, ps, null);
        }

        if (log.isTraceEnabled()) {
            log.trace("DAO end getItemByMessageIDInSent Header Message-ID: " +
                    message_id);
        }

        return eisMessage;
    }

    //----------------------------------------------------------- PRIVATE METHODS

    /**
     * used by the SENT/OUTBOX synchronization
     *
     * @param msg Message
     * @return byte[]
     * @throws Exception
     */
    private static byte[] mail2db(Message msg) throws Exception {
        OutputStream baos = new ByteArrayOutputStream();
        msg.writeTo(baos);
        String msgS = baos.toString();
        return msgS.getBytes();
    }

    /**
     * used by the SENT/OUTBOX synchronization
     *
     * @param b byte[]
     * @param s Session
     * @return Message
     * @throws Exception
     */
    private Message db2mail(byte[] b, Session s) throws Exception {
        InputStream is = new ByteArrayInputStream(b);
        return new MimeMessage(s, is);
    }

    /**
     *
     * get the email from the blob column in the DB
     *
     * @param  colIndex
     * @param  rs
     * @return email from the DB byte[]
     * @throws SQLException
     */
    private byte[] getBytesFromColumn(int colIndex, ResultSet rs)
            throws SQLException {

        Object obj = rs.getObject(colIndex);
        if (obj instanceof Blob) {
            return ((Blob) obj).getBytes(1, (int) ((Blob) obj).length());
        }
        if (obj instanceof Clob) {
            return ((Clob) obj).getSubString(1, (int) ((Clob) obj).length()).getBytes();
        }
        if (obj instanceof byte[]) {
            return (byte[]) obj;
        }
        if (obj instanceof String) {
            return ((String) obj).getBytes();
        }
        throw new ClassCastException("Unexpected type " + obj.getClass().getName() + ".");
    }


    /**
     * workaround.
     * In the POP folder there is not the method
     * getMessageByUID
     *
     * @param f POP3Folder
     * @param uid String
     * @return Message
     * @throws MessagingException
     */
    private Message getMessageByUID(POP3Folder f, Message[] messages, String uid)
     throws MessagingException{
        
        Message msgOut = null;
        Message msgTmp = null;
        String uidTmp  = null;
        boolean check  = false;

        //for (int i=0; i<messages.length; i++) {
        for (int i=(messages.length-1); i>=0; i--){
            msgTmp = messages[i];
            uidTmp = f.getUID(msgTmp);
            if (uidTmp != null){
                if (uidTmp.equalsIgnoreCase(uid)) {
                    check = true;
                    break;
                }
            }
        }
        
        if (check){
            msgOut = msgTmp;
        }

        return msgOut;
  }
   
   /**
    *
    */ 
   private ArrayList createSentList(ResultSet rs, int max) 
   throws SQLException{
       
        ArrayList rows  = new ArrayList();
        TreeMap rowsTmp = new TreeMap(new DescendingComparator());
      
        // create a sorted Map with all values
        String GUID = null;
        String sFMID = null;
        int FMID = 0;
        
        while (rs.next()) {
            GUID  = rs.getString(1);
            try {
               sFMID = Utility.getKeyPart(GUID, 2);
               FMID  = Integer.parseInt(sFMID);
               rowsTmp.put(new Integer(FMID), GUID);                
            } catch (Exception e){
               // do nothing
            }   
        }

        // get only max items
        int counter = 1;
        String id = null;
        Iterator values = rowsTmp.values().iterator();
        while (values.hasNext()) {
            id = (String)values.next();
            rows.add(id);
            counter++;
            if (counter > max){
                break;
            }
        }   
        
        return rows;
   }

}