/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager;


import com.funambol.email.exception.EntityException;
import com.funambol.email.exception.ParsingException;
import com.funambol.email.model.EmailFilter;
import com.funambol.email.model.FlagProperties;
import com.funambol.email.model.ItemMessage;
import com.funambol.email.model.InternalPart;
import com.funambol.email.pdi.mail.Ext;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;

import javax.mail.*;
import javax.mail.internet.*;
import java.util.*;

/**
 * <p>Filter class. </p>
 */

public class EntityManagerFilter extends EntityManager {
    
    /**
     * optimization of the QP
     */
    private boolean optimizeQP = true;
    
    //----------------------------------------------------------- PUBLIC METHODS
    
    
    /**
     * returns only header
     *
     * @param session opened mail server session
     * @param FID     parent folder id
     * @param FMID    mail id in the folder
     * @param msg     Message
     * @param status  status of the item; char
     * @param loc     Locale
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage getH(Session session,
            String FID,
            String FMID,
            Message msg,
            char status,
            Locale loc,
            EmailFilter filter)
            throws EntityException {
        
        ItemMessage im = null;
        Ext ext = new Ext();
        
        try {
            
            // save the flag
            FlagProperties fp = Utility.getFlags(msg);
            
            MimeMessage newMsg = new MimeMessage(session);
            
            // HEADER
            String[] h = MessageCreator.setNewHeaderIn(newMsg, msg, FID, FMID, 
                                                       Def.IS_DEVICE, optimizeQP);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);
            
            // CONTENT 
            try {    
                
                newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
                // get the info from original msg
                ArrayList parts = MessageParser.getAllPartsOfMessage(msg);
                ext = MessageCreator.getExtComplete(parts,headersDimension);
                
            } catch (Exception e){
                log.error("Generic error creating the content .. set the default content");
                newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);                
            }            
                        
            newMsg.saveChanges();
            
            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_MESSAGE_ID, mid);
            
            String encodingType = MessageCreator.getEncodingType(newMsg, optimizeQP);            

            im = setItemMessage(FMID, FID, newMsg, fp, loc, encodingType, ext, false);
            
            // re-set the flag on the server
            msg.setFlag(Flags.Flag.SEEN    , fp.getSeen());
            msg.setFlag(Flags.Flag.ANSWERED, fp.getAnswered());
            msg.setFlag(Flags.Flag.FLAGGED , fp.getFlagged());
            //msg.setFlag(Flags.Flag.FORWARDED, fp.getForwarded());
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (Utility.needsFlagOptimization(status, FID, filter)) {
            im.setStreamMessage("");
        }
        
        return im;
        
    }
    
    
    /**
     *
     * returns header and body with byte quatity limitation
     *
     *
     * @param session  opened mail server session
     * @param FID      parent folder id
     * @param FMID     mail id in the folder
     * @param msg      Message
     * @param status   status of the item; char
     * @param loc      Locale
     * @param numBytes int
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage getHB(Session session,
                             String FID,
                             String FMID,
                             Message msg,
                             char status,
                             Locale loc,
                             int numBytes,
                             EmailFilter filter)
      throws EntityException {
        
        ItemMessage im = null;
        Ext ext = new Ext();
        boolean isTextCalendar = false;
        
        boolean needOpt = Utility.needsFlagOptimization(status, FID, filter);
                                
        try {
            
            // save the flag
            FlagProperties fp = Utility.getFlags(msg);
            
            MimeMessage newMsg = new MimeMessage(session);
            
            // HEADER
            String[] h = MessageCreator.setNewHeaderIn(newMsg, msg, FID, FMID, 
                                                       Def.IS_DEVICE, optimizeQP);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);
                        
            // CONTENT
            try {
                if (needOpt){
                    
                    // set empty content (will be removed)
                    newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);  
                    
                } else {    
                    
                    // get just the body (eventually truncted)                    
                    ArrayList parts = MessageParser.getAllPartsOfMessage(msg);
                    InternalPart ip = MessageCreator.getHB_GetPart(parts, numBytes, ext, headersDimension);
                    // save the info about text/calendar email
                    isTextCalendar = ip.isTextCalendar();
                    MessageCreator.getHB_Content(newMsg, msg, ip);
                    
                }
            } catch (Exception e){
                log.error("Generic error creating the content .. set the default content");
                newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);                
            }
                        
            newMsg.saveChanges();
            
            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_MESSAGE_ID, mid);
            
            String encodingType = MessageCreator.getEncodingType(newMsg, optimizeQP);            
                       
            im = setItemMessage(FMID, FID, newMsg, fp, loc, encodingType, ext, isTextCalendar);
            
            // re-set the flag on the server
            msg.setFlag(Flags.Flag.SEEN    , fp.getSeen());
            msg.setFlag(Flags.Flag.ANSWERED, fp.getAnswered());
            msg.setFlag(Flags.Flag.FLAGGED , fp.getFlagged());
            //msg.setFlag(Flags.Flag.FORWARDED, fp.getForwarded());
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (needOpt){
            im.setStreamMessage("");
        }
        
        return im;
        
    }
    
    /**
     *
     * if numBytes = -1 it returns all parts of the message
     * (header, body, attachements) without byte quatity limitation.
     *
     * if messageSize <= numBytes it returns all parts of the message
     * (header, body, attachements)
     *
     * otherwise it returns all parts of the message
     * (header, body, attachements) but there is the byte number limitation
     *
     *
     * @param session  opened mail server session
     * @param FID      parent folder id
     * @param FMID     mail id in the folder
     * @param msg      Message
     * @param status   status of the item; char
     * @param loc      Locale
     * @param numBytes int
     * @return ItemMessage
     * @throws EntityException
     */
    public ItemMessage getHBA(Session session,
            String FID,
            String FMID,
            Message msg,
            char status,
            Locale loc,
            int numBytes,
            EmailFilter filter)
            throws EntityException {
        
        ItemMessage im = null;
        Ext ext = new Ext();        
        boolean isTextCalendar = false;
               
        boolean needOpt = Utility.needsFlagOptimization(status, FID, filter);
                
        try {
            
            // save the flag
            FlagProperties fp = Utility.getFlags(msg);
            
            int messageSize = msg.getSize();
                  
            MimeMessage newMsg = new MimeMessage(session);

            // HEADER
            String[] h = MessageCreator.setNewHeaderIn(newMsg, msg, FID, FMID, 
                                                       Def.IS_DEVICE, optimizeQP);
            String mid              = h[0];
            int    headersDimension = Integer.parseInt(h[1]);
            
            
            // CONTENT
            try {
                if (needOpt){
                    
                    newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);   
                    
                } else if ( (numBytes == -1) || (messageSize <= numBytes) ){
                    
                    // get all the mail without limitation
                    ArrayList parts = MessageParser.getAllPartsOfMessage(msg);
                    ArrayList ips = MessageCreator.getHBA_GetParts(parts, ext, headersDimension);
                    // save the info about text/calendar email
                    if (ips.size() == 1){
                       isTextCalendar = ((InternalPart)ips.get(0)).isTextCalendar();
                    }
                    MessageCreator.getHBA_Content(newMsg, msg, ips);                    
                    
                } else {
                    
                    // get mail with limitation                
                    ArrayList parts = MessageParser.getAllPartsOfMessage(msg);
                    ArrayList ips = MessageCreator.getHBA_GetParts(parts, numBytes, ext, headersDimension);
                    // save the info about text/calendar email
                    if (ips.size() == 1){
                       isTextCalendar = ((InternalPart)ips.get(0)).isTextCalendar();
                    }
                    MessageCreator.getHBA_Content(newMsg, msg, ips);                    
                    
                }
            } catch (Exception e){
                log.error("Generic error creating the content .. set the default content");
                newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);                
            }
            
            
            newMsg.saveChanges();

            // the save message command change the message-ID
            newMsg.setHeader(Def.HEADER_MESSAGE_ID, mid);

            String encodingType = MessageCreator.getEncodingType(newMsg, optimizeQP);            
                        
            im = setItemMessage(FMID, FID, newMsg, fp, loc, encodingType, ext, isTextCalendar);

            // re-set the flag on the server
            msg.setFlag(Flags.Flag.SEEN    , fp.getSeen());
            msg.setFlag(Flags.Flag.ANSWERED, fp.getAnswered());
            msg.setFlag(Flags.Flag.FLAGGED , fp.getFlagged());
            //msg.setFlag(Flags.Flag.FORWARDED, fp.getForwarded());
                        
        } catch (MessagingException me) {
            throw new EntityException(me);
        }
        
        // remove the message if needed (syncml dimension optimization)
        if (needOpt){
            im.setStreamMessage("");
        }
        
        return im;
    }    
   
}
