/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager;


import com.funambol.email.exception.EntityException;
import com.funambol.email.model.InternalPart;
import com.funambol.email.pdi.mail.Ext;
import com.funambol.email.util.Def;
import com.funambol.email.util.Html2Txt;
import com.funambol.email.util.Utility;
import com.funambol.email.util.UtilityDate;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import java.awt.datatransfer.DataFlavor;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import javax.mail.*;
import javax.mail.internet.*;
import java.util.*;
import javax.activation.DataHandler;

/**
 *
 * This class is used in order to create an email using the list 
 * of InternalPart items.
 * this class is an helper for the EntityManagerFilter class
 *
 * @version $Id: MessageCreator.java,v 1.13 2007/03/19 14:24:42 gbmiglia Exp $
 *
 */

public class MessageCreator {
    
    
    //---------------------------------------------------------------- CONSTANTS
    
    /**     */
    private static FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    
    //----------------------------------------------------------- PUBLIC METHODS
    
    /**
     *
     * this method checks if message contains non-ASCII characters
     * and set the correct encoding type
     * It's a public method because there is an external unit test
     *
     * @param newMsg Message
     * @param optimizeQP boolean
     * @return encodingType String
     */    
    public static String getEncodingType(MimeMessage newMsg, 
                                         boolean optimizeQP) 
      throws EntityException {        
       
        // default
        String encodingType = Def.ENCODE_QUOTED_PRINTABLE;        
         
        if (optimizeQP){
            
            String transfertEnc = Utility.getHeaderEncoding(newMsg);
            String contentType  = Utility.getHeaderCharset(newMsg);

            if (transfertEnc != null){
                if (transfertEnc.indexOf("quoted-printable") != -1 ||
                    transfertEnc.indexOf("QUOTED-PRINTABLE") != -1 ){
                    // don't apply a double encoding
                   return null;
                } 
            }

            if (contentType != null){
                if (contentType.indexOf("UTF-8") != -1 ||
                    contentType.indexOf("UTF8") != -1 ){
                    // don't apply a double encoding
                   return null;
                } 
            }

            // the header is already checked .. see setNewHeader
        }
        
        return encodingType;
        
    }               
    
    
    /**
     * create an Ext object with the following info
     * <br>
     * - truncated
     * <br>
     * - body size
     * <br>
     * - all info about attachments
     *
     * @param ips part list ArrayList
     * @param headersDimension int
     * @return ext Ext
     */
    public static Ext getExtComplete(ArrayList ips, int headersDimension)
     throws EntityException {
        
        Ext ext       = new Ext();
        int totalSize = 0;
        int itemSize  = 0;
        
        try {
            
            // truncated
            ext.setTruncated(true); 
                        
            // body & attachments
            ArrayList xvalList = new ArrayList();
            InternalPart item = null;
            String[] value = null;
            for (int i=0; i<ips.size(); i++){
                
                 item = (InternalPart)ips.get(i);
                 
                 itemSize = item.getSize();
                 
                 if (item.isTextBody()){
                     ext.setXvalBody(""+item.getSize());
                     totalSize = totalSize + itemSize;   
                 }
                 
                 // ext
                 if (item.getDisposition() != null && 
                     item.getDisposition().equals(MimeBodyPart.ATTACHMENT)){
                     value = new String[2];        
                     value[0] = item.getFileName();
                     value[1] = ""+item.getSize();
                     xvalList.add(value); 
                     totalSize = totalSize + itemSize;   
                 }

                 // ext
                 if (!item.isTextBody()  &&
                     item.getDisposition() != null && 
                     item.getDisposition().equals(MimeBodyPart.INLINE)){
                     value = new String[2];        
                     value[0] = item.getFileName();
                     value[1] = ""+item.getSize();
                     xvalList.add(value); 
                     totalSize = totalSize + itemSize;                        
                 }
                 
            }
            ext.setXvalAttach(xvalList);                        
            
            if (ext.isTruncated()){
               ext.setXsize("" +  (totalSize+headersDimension));
            }
            
        } catch (Exception e){
            throw new EntityException("Error creating the Ext property");
        }
              
        return ext;
        
    }
            
    
    
    /**
     *
     *
     *
     */
    public static ArrayList getHBA_GetParts(ArrayList ips, 
                                            int numBytes, 
                                            Ext ext,
                                            int headersDimension) 
      throws EntityException {
        
        InternalPart item                 = null;
        ArrayList    newMsgParts          = new ArrayList();
        boolean      isTextBodyAlreadyGot = false;        
        boolean      isHtmlBodyAlreadyGot = false;        
        int          htmlBodySize         = 0;
        String[]     value                = null;    
        ArrayList    xvalList             = new ArrayList();
        int          itemSize             = 0;
        int          totalSize            = 0;
        
        for (int i=0; i<ips.size(); i++){
                
             item = (InternalPart)ips.get(i);
             
             itemSize = item.getSize();
                              
             if (item.isTextBody()){
                 if (!isTextBodyAlreadyGot){
                     if ( numBytes < (totalSize + item.getSize()) ){
                         newMsgParts.add(truncBody(item, numBytes, ext, itemSize));
                     } else {
                         newMsgParts.add(item);                         
                     }
                     if (isHtmlBodyAlreadyGot){                     
                         totalSize = totalSize - htmlBodySize;
                         totalSize = totalSize + item.getSize();
                     } 
                     totalSize = totalSize + itemSize;
                     isTextBodyAlreadyGot = true;                 
                 } else{
                     // test 009
                     // it's a part in txt but is not the main body
                     item.setDisposition(MimeBodyPart.ATTACHMENT);
                     item.setFileName("Part00"+i+".txt");                     
                 }
             }

             if (item.isHtmlBody()){
                 if (!isTextBodyAlreadyGot){
                     if ( numBytes < (totalSize + item.getSize()) ){
                         newMsgParts.add(truncBody(item, numBytes, ext, itemSize));
                     } else {
                         newMsgParts.add(item);
                     }                 
                     totalSize = totalSize + itemSize;
                     isHtmlBodyAlreadyGot = true;                 
                     htmlBodySize = item.getSize();
                     totalSize = totalSize + item.getSize();                     
                 }
             }

             if (!item.isTextBody() &&
                 item.getDisposition() != null && 
                 item.getDisposition().equals(MimeBodyPart.INLINE)){

                  if ( numBytes < (totalSize + item.getSize()) ){
                     // don't get the attachment, but set the ext
                     ext.setTruncated(true);
                     value = new String[2];  
                     value[0] = item.getFileName();
                     value[1] = ""+item.getSize();
                     xvalList.add(value); 
                  } else {
                     // don't set the ext, but get the attachment
                     newMsgParts.add(item);
                     totalSize = totalSize + item.getSize();   
                  }
             }             

             if (item.getDisposition() != null && 
                 item.getDisposition().equals(MimeBodyPart.ATTACHMENT)){

                 if ( numBytes < (totalSize + item.getSize()) ){
                     // don't get the attachment, but set the ext
                     ext.setTruncated(true);     
                     value = new String[2];  
                     value[0] = item.getFileName();
                     value[1] = ""+item.getSize();
                     xvalList.add(value); 
                 } else {
                     // don't set the ext, but get the attachment
                     newMsgParts.add(item);
                     totalSize = totalSize + item.getSize();
                 }
             }                  
        }
        
        ext.setXvalAttach(xvalList);
        
        if (ext.isTruncated()){
           ext.setXsize("" +  (totalSize+headersDimension));
        }
        
        return newMsgParts;
        
    }
    
    /**
     *
     *
     *
     */
    public static ArrayList getHBA_GetParts(ArrayList ips,
                                            Ext ext,
                                            int headersDimension) 
      throws EntityException {
        
        InternalPart item                 = null;
        ArrayList    newMsgParts          = new ArrayList();
        boolean      isTextBodyAlreadyGot = false;        
        boolean      isHtmlBodyAlreadyGot = false;        
        int          itemSize             = 0;
        int          totalSize            = 0;
        
        for (int i=0; i<ips.size(); i++){
                
             item = (InternalPart)ips.get(i);
                              
             itemSize = item.getSize();
             
             // get the body
             if (item.isTextBody()){
                 if (!isTextBodyAlreadyGot){
                     newMsgParts.add(item);  
                     totalSize = totalSize + itemSize;
                     isTextBodyAlreadyGot = true;      
                 } else{
                     // test 009
                     // it's a part in txt but is not the main body
                     item.setDisposition(MimeBodyPart.ATTACHMENT);
                     item.setFileName("Part00"+i+".txt");                     
                 }
             }
             
             // get html only if there is no TXT body
             if (item.isHtmlBody()){
                 if (!isTextBodyAlreadyGot){
                     newMsgParts.add(item);
                     totalSize = totalSize + itemSize;
                 }
             }
             
             // @to be verified
             if (!item.isTextBody() && 
                 item.getDisposition() != null && 
                 item.getDisposition().equals(MimeBodyPart.INLINE)){
                 newMsgParts.add(item);
                 totalSize = totalSize + itemSize;
             } 
             
             // get all the attachments             
             if (item.getDisposition() != null && 
                 item.getDisposition().equals(MimeBodyPart.ATTACHMENT)){
                 newMsgParts.add(item);
                 totalSize = totalSize + itemSize;
             } 
        
        }         
        
        if (ext.isTruncated()){
           ext.setXsize("" +  (totalSize+headersDimension));
        }
        
        return newMsgParts;
        
    }

    /**
     * get just the InternalPart that contains the body 
     *
     * @param newMsg MimeMessage new Message
     * @param oldMsg Message old Message
     * @param ip InternalPart part of theemail
     */
    public static void getHB_Content(MimeMessage newMsg, 
                                     Message oldMsg, 
                                     InternalPart ip)
      throws EntityException {
        try {
            if (ip != null){
                String msgEnc = Utility.getHeaderEncoding(oldMsg);
                if ( ip.isHtmlBody() && !msgEnc.equalsIgnoreCase(Def.ENCODE_BASE64)){
                    try {
                        String newBody = convertHtmlBody(ip);
                        newMsg.setContent(newBody, Def.CONTENT_CONTENTTYPE);                    
                    } catch (Exception e){
                        // if there is an error returns the html body
                        newMsg.setContent(ip.getPartValue(), ip.getContentType());                    
                    }
                } else {
                    newMsg.setContent(ip.getPartValue(), ip.getContentType());                    
                }                        
            } else {
                newMsg.setContent(Def.CONTENT_BODY_ALERT, Def.CONTENT_CONTENTTYPE);
            }
        } catch (MessagingException me){
            throw new EntityException("Error setting the new content for the getHB method");
        }        
    }
    
    /**
     * get just the InternalPart that contains the body 
     *
     * @param newMsg MimeMessage new Message
     * @param oldMsg Message old Message
     * @param ips InternalPart part of theemail
     * @throws EntityException
     */
    public static void getHBA_Content(MimeMessage newMsg, 
                                      Message oldMsg, 
                                      ArrayList ips)
     throws EntityException {

        try {
            
            if ( ips.size() == 1){

                InternalPart ip = (InternalPart)ips.get(0);

                String msgEnc = Utility.getHeaderEncoding(oldMsg);

                if ( ip.isTextBody()){

                    newMsg.setContent(ip.getPartValue(), ip.getContentType());                                        
                                        
                } else if ( ip.isHtmlBody() && !msgEnc.equalsIgnoreCase(Def.ENCODE_BASE64)){
                    
                    try {
                        String newBody = convertHtmlBody(ip);
                        newMsg.setContent(newBody, Def.CONTENT_CONTENTTYPE);                    
                    } catch (Exception e){
                        // if there is an error returns the html body
                        newMsg.setContent(ip.getPartValue(), ip.getContentType());                    
                    }
                    
                } else {
                    
                    newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);                    
                    
                }                       

            } else if (ips.size() > 1){

                MimeMultipart multipart = new MimeMultipart();
                BodyPart part   = null;
                InternalPart ip = null;

                String msgEnc = Utility.getHeaderEncoding(oldMsg);
                
                for (int i = 0; i < ips.size(); i++) {

                    part = new MimeBodyPart();

                    ip = (InternalPart)ips.get(i);

                    if (ip.isTextBody()) {
                        
                        part.setContent(ip.getPartValue(), ip.getContentType());  
                        
                        if (ip.getFileName() != null){
                          part.setFileName(ip.getFileName());
                        }        
                        
                        part.setDisposition(ip.getDisposition());
                        
                    } else if ( ip.isHtmlBody() && !msgEnc.equalsIgnoreCase(Def.ENCODE_BASE64)){
                        
                        try {
                            String newBody = convertHtmlBody(ip);
                            part.setContent(newBody, Def.CONTENT_CONTENTTYPE);                    
                        } catch (Exception e){
                            // if there is an error returns the html body
                            part.setContent(ip.getPartValue(), ip.getContentType());                    
                        }
                                                                        
                        if (ip.getFileName() != null){
                          part.setFileName(ip.getFileName());
                        }
                       
                        part.setDisposition(ip.getDisposition());
                       
                    } else if (ip.getDisposition() != null && 
                        (ip.getDisposition().equals(MimeBodyPart.ATTACHMENT) ||
                          ip.getDisposition().equals(MimeBodyPart.INLINE))){
                        
                       if (ip.getFileName() != null){
                          part.setFileName(ip.getFileName());
                       }
                       
                       part.setDisposition(ip.getDisposition());
                       
                       
                       try {
                           DataHandler dh = ip.getDHandler();
                           checkDataHandlerContent(dh);
                           part.setDataHandler(dh); 
                       } catch (Exception me){
                           // do nothing 
                           log.error("Error setting the Content Part: " + 
                                   me.getMessage() + " for the attachment " + ip.getFileName());
                           String defContent = Def.CONTENT_BODY_ALERT;
                           if (ip.getFileName() != null){
                               defContent = "Error Handling Attachment " + ip.getFileName();
                           }
                           DataHandler dhNew = new DataHandler(defContent, Def.CONTENT_CONTENTTYPE);
                           part.setDataHandler(dhNew); 
                           if (ip.getFileName() != null){
                              part.setFileName("DefaultAttachment00"+i+".txt");
                           }
                       }
                                              
                    } else {
                        
                       part.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);                    
                       
                    }

                    multipart.addBodyPart(part);

                }
                
                newMsg.setContent(multipart);

            } else {
                // just in case of error
                newMsg.setContent(Def.CONTENT_BODY, Def.CONTENT_CONTENTTYPE);
            }
        } catch (Exception me){
            throw new EntityException("Error setting new content for the getHBA method");
        }

    }
    
    /**
     *
     * get just the InternalPart that contains the body 
     *
     * @param ips All parts of the emails
     * @param numBytes int
     * @param ext Ext
     * @param headersDimension int
     * @return InternalPart
     * @throws EntityException
     */
    public static InternalPart getHB_GetPart(ArrayList ips, 
                                             int numBytes, 
                                             Ext ext,
                                             int headersDimension)
      throws EntityException {
        
        // itemOut keeps just the body of the email
        InternalPart itemOut              = null;
        
        boolean      isTextBodyAlreadyGot = false;        
        boolean      isHtmlBodyAlreadyGot = false;        
        InternalPart item                 = null;
        String[]     value                = null; 
        ArrayList    xvalList             = new ArrayList();        
        int          totalSize            = 0;
        int          itemSize             = 0;
                
        for (int i=0; i<ips.size(); i++){
                
             item = (InternalPart)ips.get(i);
             
             itemSize = item.getSize();

             if (item.isTextBody()){
                 if (!isTextBodyAlreadyGot){
                     if (numBytes == -1){
                        itemOut = item;
                     } else {
                        itemOut = truncBody(item, numBytes, ext, itemSize);
                     }
                     totalSize = totalSize + itemSize;
                     isTextBodyAlreadyGot = true;                 
                 } else{
                     // test 009
                     // it's a part in txt but is not the main body
                     item.setDisposition(MimeBodyPart.ATTACHMENT);
                     item.setFileName("Part00"+i+".txt");                     
                 }
             }
             
             // get html only if there is no TXT body
             if (item.isHtmlBody()){
                 if (!isTextBodyAlreadyGot){
                     // no body already got, so get the HTML body
                     if (numBytes == -1){
                        itemOut = item;
                     } else {
                        itemOut = truncBody(item, numBytes, ext, itemSize);
                     }
                     totalSize = totalSize + itemSize;
                 }
             }
                          
             // ext
             if (item.getDisposition() != null && 
                 item.getDisposition().equals(MimeBodyPart.ATTACHMENT)){
                 value = new String[2];        
                 value[0] = item.getFileName();
                 value[1] = ""+item.getSize();
                 xvalList.add(value); 
                 totalSize = totalSize + itemSize;
             }
                         
             // ext
             if (!item.isTextBody()  &&
                 item.getDisposition() != null && 
                 item.getDisposition().equals(MimeBodyPart.INLINE)){
                 value = new String[2];        
                 value[0] = item.getFileName();
                 value[1] = ""+item.getSize();
                 xvalList.add(value); 
                 totalSize = totalSize + itemSize;
             }
             
        }
        
        ext.setXvalAttach(xvalList);
        
        if (ext.isTruncated()){
           ext.setXsize("" +  (totalSize+headersDimension));
        }

        return itemOut;
        
    }
       
    
    /**
     *
     */
    public static InternalPart truncBody(InternalPart item, 
                                   int numBytes, 
                                   Ext ext,
                                   int origSize)
      throws EntityException {

        try {
            String partContent = "";
            byte[] partBytes   = null;
            if (item.getPartValue() != null) {
                
                partContent = String.valueOf(item.getPartValue().toString());
                partBytes = partContent.getBytes();
                
                if (numBytes < partBytes.length) {
                    item.setPartValue(new String(partBytes, 0, numBytes));
                    item.setSize(numBytes);                    
                    ext.setTruncated(true);
                    ext.setXvalBody(""+(origSize-numBytes));
                }
            }             
        }catch (Exception e){
            throw new EntityException ("Error cutting the email body " + e.getMessage());
        }
                
        return item;
    }

    /**
     * convert an Html Part to Txt Part
     *
     * @param v Part
     * @return part in txt format String
     */
    public static String convertHtmlBody(InternalPart v)
      throws EntityException {
        String bodyhtml = "";
        String bodytext = "";        
        if (v.getPartValue() instanceof String){
            bodyhtml = (String)v.getPartValue();
            bodytext = Html2Txt.htmltotext(bodyhtml);
        }
        return bodytext;
    }
          
    /**
     *
     * set a new header but keep the old Message-ID for
     * a message that must be downloaded
     *
     * @param newMsg Message
     * @param oldMsg Message
     * @return header-id and header dimension
     * @throws EntityException
     */
    public static String[] setNewHeaderIn(Message newMsg,
                                          Message oldMsg,
                                          String FID,
                                          String FMID,
                                          boolean isDevice,
                                          boolean optimizeQP)
      throws EntityException {


        String   oldMessageID     = null;
        String   label            = "";
        String   date             = "";
        int      headersDimension = 0;
        int      headerlength     = 0;
        Header   h                = null;
        String   name             = null;
        String   value            = null;
        String   valueqp          = null;
        String[] out              = new String[2];

        
        try {

            String[] recs  = oldMsg.getHeader(Def.HEADER_RECEIVED);
            
            Enumeration en = oldMsg.getAllHeaders();
                        
            while (en.hasMoreElements()) {
                
                h = (Header) en.nextElement();
                
                name  = h.getName();
                value = h.getValue();
                headerlength = 0;

                if (name.equalsIgnoreCase(Def.HEADER_DATE)) {
                    // save the date in string format for the createDummyMessageID()
                    date = value;                    
                    // check if the date is parsable
                    if (!UtilityDate.isHeaderDateParsable(oldMsg)){
                        value = UtilityDate.getHeaderDateNowPatch();
                    }

                }
                
                // save the original Message-ID. it will be set in the caller method 
                if (name.equalsIgnoreCase(Def.HEADER_MESSAGE_ID)) {
                    oldMessageID = value;
                }
                
                // encode value (QP)
                value = encodeHeaderField(name, value, optimizeQP);                        
                
                if(isDevice){
                    if (isHeaderMinimum(name)){                        
                        label = name + ": " + value;
                        headerlength = label.getBytes().length;
                        if (!name.equals(Def.HEADER_RECEIVED)){
                            // doesn't save the received
                            newMsg.addHeader(name, value);
                            headersDimension = headersDimension + headerlength;
                        }                    
                    }                
                } else {
                    label = name + ": " + value;
                    headerlength = label.getBytes().length;
                    if (!name.equals(Def.HEADER_RECEIVED)){
                        // doesn't save the received
                        newMsg.addHeader(name, value);
                        headersDimension = headersDimension + headerlength;
                    }                                        
                }
                
            }
            
            // save the received
            String recValue = null;
            if(isDevice){
                if (recs != null){
                    int i = recs.length-1;
                    // encode value (QP)
                    recValue = encodeHeaderField(Def.HEADER_RECEIVED, recs[i], optimizeQP);
                    newMsg.addHeader(Def.HEADER_RECEIVED, recValue);         
                }                                
            } else {
                if (recs != null){
                    for (int i=recs.length-1; i>=0; i--){
                        // encode value (QP)
                        recValue = encodeHeaderField(Def.HEADER_RECEIVED, recs[i], optimizeQP);
                        newMsg.addHeader(Def.HEADER_RECEIVED, recValue);
                    }
                }                                
            }
            
            // set a message-id if the old message-id is null
            try {
                if (oldMessageID == null){
                    String guid = Utility.createIMAPGUID(FID,FMID,"");
                    oldMessageID = Utility.createDummyMessageID(guid, 
                                                                ""+headersDimension, 
                                                                date);
                }
                // the message-id is set in the caller method
            } catch (Exception e) {
                // do nothing
            }
        
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        }

        out[0] = oldMessageID;
        out[1] = String.valueOf(headersDimension);

        return out;

    }    
    
    /**
     *
     * set a new header but keep the old Message-ID for
     * a message that must be sent
     *
     * @param newMsg Message
     * @param oldMsg Message
     * @param from String
     * @param firstname String
     * @param lastname String
     * @throws EntityException
     */
    public static void setNewHeaderOut(Message newMsg,
                                       Message oldMsg,
                                       String from,
                                       String firstname,
                                       String lastname)
      throws EntityException {


        Header   h            = null;
        String   name         = null;
        String   value        = null;
        String   oldMessageID = null;
        boolean  hasFromField = false;
        
        try {

            String[] recs  = oldMsg.getHeader(Def.HEADER_RECEIVED);
            
            Enumeration en = oldMsg.getAllHeaders();
            
            hasFromField = Utility.hasHeaderFrom(oldMsg);
                        
            while (en.hasMoreElements()) {
                
                h = (Header) en.nextElement();
                
                name  = h.getName();
                value = h.getValue();

                if (name.equalsIgnoreCase(Def.HEADER_MESSAGE_ID)) {
                    oldMessageID = value;
                }
                                
                if (!name.equals(Def.HEADER_RECEIVED)){
                    // doesn't save the received
                    newMsg.addHeader(name, value);
                }                    
                                
            }
            
            // save the from field
            if (!hasFromField){
                InternetAddress fromAddress = new InternetAddress(from);
                fromAddress.setPersonal(firstname + " " + lastname);
                newMsg.setFrom(fromAddress);                
            }
            
            // save all the received fileds
            if (recs != null){
                for (int i=recs.length-1; i>=0; i--){
                   newMsg.addHeader(Def.HEADER_RECEIVED, recs[i]);                
                }
            }

        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (UnsupportedEncodingException me) {
            throw new EntityException(me);
        }

    }        
        
    /**
     *
     * set a new header but keep the old Message-ID for
     * a message that must be sent
     *
     * @param newMsg Message
     * @param oldMsg Message
     * @param from String
     * @param firstname String
     * @param lastname String
     * @throws EntityException
     */
    public static void setNewHeaderSent(Message newMsg,
                                        Message oldMsg,
                                        String from,
                                        String firstname,
                                        String lastname)
      throws EntityException {


        Header   h            = null;
        String   name         = null;
        String   value        = null;
        String   oldMessageID = null;
        boolean  hasFromField = false;
        
        try {
            
            Enumeration en = oldMsg.getAllHeaders();
            
            hasFromField = Utility.hasHeaderFrom(oldMsg);
                        
            while (en.hasMoreElements()) {
                
                h = (Header) en.nextElement();
                
                name  = h.getName();
                value = h.getValue();

                if (name.equalsIgnoreCase(Def.HEADER_MESSAGE_ID)) {
                    oldMessageID = value;
                }
                
                // save just minimum info
                if (name.equalsIgnoreCase(Def.HEADER_DATE) || 
                    name.equalsIgnoreCase(Def.HEADER_MESSAGE_ID) ||
                    name.equalsIgnoreCase(Def.HEADER_FROM) ||
                    name.equalsIgnoreCase(Def.HEADER_TO) ) {
                    newMsg.addHeader(name, value);
                }                   
                                
            }
            
            // 
            newMsg.addHeader(Def.HEADER_SUBJECT, "");
            
            // save the from field
            if (!hasFromField){
                InternetAddress fromAddress = new InternetAddress(from);
                fromAddress.setPersonal(firstname + " " + lastname);
                newMsg.setFrom(fromAddress);                
            }
            
        } catch (MessagingException me) {
            throw new EntityException(me);
        } catch (UnsupportedEncodingException me) {
            throw new EntityException(me);
        }

    }        
    
    /**
     *
     * create an exact copy of the content
     *
     * @param newMsg Message
     * @param oldMsg Message
     */    
    public static void setNewContent(Message newMsg, 
                                     Message oldMsg) 
      throws EntityException {
        try {
            DataHandler dh = oldMsg.getDataHandler();
            newMsg.setDataHandler(dh);
        } catch (Exception e){
            throw new EntityException("Error creating new Content");
        }
    }
    
    
    
    /**
     *
     * create a copy of the content with the signature Part
     * this method is used in the sending procedure
     *
     * @param newMsg Message
     * @param oldMsg Message
     * @param sign String
     */
   public static void setNewContent(Message newMsg, 
                                    Message oldMsg, 
                                    String sign) 
     throws Exception {
           
        ArrayList newContents = MessageParser.getAllPartsOfMessage(oldMsg);
        
        BodyPart part;
        MimeMultipart multipart = new MimeMultipart();
        
        // original parts
        for (int i=0; i<newContents.size(); i++) {
            
            InternalPart v = (InternalPart)newContents.get(i);
            
            part = new MimeBodyPart();
            
            part.setContent(v.getPartValue(), v.getContentType());
            
            if (newContents.size()>1){
                part.setDataHandler(v.getDHandler());
            }
            if (v.getDisposition() != null){
                part.setDisposition(v.getDisposition());
            }
            if (v.getFileName() != null){
                part.setFileName(v.getFileName());
            }
            
            multipart.addBodyPart(part);
            
        }
        
        // signature part
        if (sign != null && !sign.equals("")){
          MimeBodyPart mbpfun = new MimeBodyPart();
          mbpfun.setText(sign);
          mbpfun.setDisposition(MimeBodyPart.INLINE);
          mbpfun.setFileName("Funambol File.txt");
          multipart.addBodyPart(mbpfun);
        }
                                           
        newMsg.setContent(multipart);

   }
    
   
   //----------------------------------------------------------- Private Methods
   
   
   /**
    * replace non-parsable character
    */
   private static String encodeHeaderField(String name, 
                                           String value,
                                           boolean optimizeQP){
       
       String out = value;
              
       if (optimizeQP){
           
           StringBuffer outTmp = new StringBuffer();
           
           if (!name.equalsIgnoreCase(Def.HEADER_MESSAGE_ID)) {
               
               char[] chars = value.toCharArray();
               char chr;
               
               for (int i = 0; i < chars.length; i++) {
                   
                   chr = chars[i];
                   
                   if (chr < 0x20) {
                        if (chr != 0x9 && chr != 0xA && chr != 0xD) {
                             outTmp.append((char)' ');
                        } else {
                             outTmp.append((char)chr);
                        } 
                   } else {
                        if (chr  > 0x7F) {
                           if (chr  >= 0x3FFF) {
                                if (chr  > 0xD7FF && 
                                     (chr  < 0xE000 || 
                                      chr  == 0xFFFE ||
                                      chr  == 0xFFFF || 
                                      chr  > 0x10FFFF)) {
                                    out.replace(chr,' ');
                                } else {
                                    outTmp.append((char)chr);
                                }
                            } else {
                                outTmp.append((char)chr);
                            }                  
                        } else {
                            outTmp.append((char)chr);
                        }
                   } 
                   
               }
               
               out = outTmp.toString();
           }
           
       }                  
       
       return out;
   }
   
   /**
    *
    */
   private static boolean isHeaderMinimum(String name){
       boolean check = false;
       for (int i=0; i < Def.HEADER_MINIMUM.length; i++){
           if (name.equalsIgnoreCase(Def.HEADER_MINIMUM[i])){
               check = true;
               break;
           }
       }
       return check;
   }
   
     
   /**
    *
    * check about the content of the datahandler.
    * the Datahandler could contains the following error:
    * <p>
    * - some unparsable content type (mail with image in the body sent by 
    *   outlook 2002: "IMAGE/JPEG; name=image001.jpg; image/spj=")
    * <p>
    * - ...
    * <p>
    * In this way the system can insert a default Part in the email and the 
    * email can be sent to the client
    *
    */
   private static void checkDataHandlerContent(DataHandler dh) 
     throws Exception {
       try {
           DataFlavor[] df = dh.getTransferDataFlavors();       
           /*
           for (int y=0; y<df.length; y++){
               System.out.println("-----" + df[y].getMimeType());
               System.out.println("-----" + df[y].getHumanPresentableName());
           }
           */
       } catch (Exception e){
           throw new Exception(e.getMessage());
       }
   }
   
}
