/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.items.manager;


import com.funambol.email.exception.ParsingException;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;
import javax.mail.internet.MimeBodyPart;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.email.model.InternalPart;
import com.funambol.email.util.Def;
import com.funambol.email.util.ShowString;
import com.funambol.email.util.Utility;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.Reader;

/**
 *
 * This class is used in order to create a list of the Part of the 
 * single email. in the list there will be InternalPart items.
 * this class is an helper for the EntityManagerFilter class
 *
 * @version $Id*
 */
public class MessageParser {
    
    //---------------------------------------------------------------- CONSTANTS
        
    /**     */
    private static FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    //----------------------------------------------------------- PUBLIC METHODS
       
    /**
     * this method returns always a list of part
     * <br>
     * if the javaMail fails to get the main content the method 
     * returns a default InternalPart with the "CONTENT_BODY_ALERT" as 
     * the body value.
     * <br>
     * if the javaMail fails to get the single parts of the email 
     * the method check if it can return some parts. if no parts is 
     * taken the method return a default InternalPart with the 
     * "CONTENT_BODY_ALERT" as the body value.
     *
     * @param msg Message
     * @return list of email parts ArrayList
     *
     */
    public static ArrayList getAllPartsOfMessage(Message msg) {

        // contains all the message parts
        ArrayList ips          = new ArrayList();        
        boolean mainError      = false;
        boolean singleError    = false;
        Object mainContent     = null;            
                
        try {                                    
            mainContent = msg.getDataHandler().getContent();                                    
        } catch (MessagingException me){
            // if javaMail fails to get all the content we set an empty body
            mainError = true;
            log.error("Error getting the main content - Unsupported Encoding");
        } catch (IOException ioe){
            // if javaMail fails to get all the content we set an empty body
            mainError = true;
            log.error("Error getting the main content - Unsupported Encoding");
        }
                
        try {                                    
            Integer[] partIndex = { new Integer(1) };
            if (mainContent != null){
                if (mainContent instanceof Multipart) {                
                    getPartInfoMutli(ips, partIndex, msg, mainContent);                                
                } else {
                    InternalPart ip = getPartInfoSingle(msg, partIndex, mainContent);                
                    ips.add(ip);
                }           
            }                                    
        } catch (Exception me){
            // if javaMail fails to get single parts
            singleError = true;
            log.error("Error single email parts - Unsupported Encoding");
        }
              
        if (mainError){
            log.error("Create the default InternalPart - Unsupported Encoding");
            ips.clear();
            InternalPart defaultIP = new InternalPart();
            defaultIP.setPartValue(Def.CONTENT_BODY_ALERT);
            defaultIP.setIsTextBody(true);
            ips.add(defaultIP);
        }
        
        if (singleError){
            log.error("Create the default InternalPart - Unsupported Encoding ");
            if (ips.size() > 0){
                // there are some parts    
            } else {
                // no parts .. create a default part
                InternalPart defaultIP = new InternalPart();
                defaultIP.setPartValue(Def.CONTENT_BODY_ALERT);
                defaultIP.setIsTextBody(true);
                ips.add(defaultIP);
            }
        }
        
        // check if there is just one part and it's not a body but an attachment
        // (it happens with the Apple Mailer test 038 - 'volo apple')
        // removed the condition on the INLINE (test 042 - gmail)
        if ( ips.size() == 1){
            InternalPart ip = (InternalPart)ips.get(0);
            
            //if (ip.getDisposition() != null && 
            //    (ip.getDisposition().equals(MimeBodyPart.INLINE) || 
            //     ip.getDisposition().equals(MimeBodyPart.ATTACHMENT))){            
            if (ip.getDisposition() != null && 
                (ip.getDisposition().equals(MimeBodyPart.ATTACHMENT))){            

                // save the part
                InternalPart attachmentIP = new InternalPart();                
                attachmentIP.cloneIP(ip);                                
                // default txt body internalPart
                InternalPart defaultIP = new InternalPart();                
                defaultIP.setPartValue(Def.CONTENT_BODY);
                defaultIP.setIsTextBody(true);
                
                ips.clear();
                
                // add the body
                ips.add(defaultIP);
                // add the attachment
                ips.add(attachmentIP);
                
            }
        }        
        
        return ips;
        
    }
    
    
    /**
     *
     */
    private static InternalPart getPartInfoSingle(Part partInt,
                                                  Integer[] partIndex,
                                                  Object currentContent) 
    throws ParsingException {
        
        Object contentInt = null;
        int partsize = 0;

        if (currentContent instanceof String){            
            partsize = ((String)currentContent).getBytes().length;
        } else {
            partsize = -1;
        }
        
        try {
            contentInt = partInt.getContent();            
        } catch (Exception e){
            // javamail fails parsing the content
            try {
                contentInt = getTextBody(partInt);
            } catch (Exception ee){
                throw new ParsingException("Error parsing email content");
            }   
        }  

        InternalPart ip = null;
        try {
            ip = createIP(partInt, partsize, partIndex, currentContent, contentInt);
        } catch (Exception ee){
            throw new ParsingException("Error parsing email content");
        }   

        return ip;
    }
    
    /**
     *
     */
    private static void getPartInfoMutli(ArrayList allParts,
                                         Integer[] partIndex,
                                         Part part,
                                         Object currentContent) 
    throws MessagingException, ParsingException {
        
        Multipart multipart  = null;
        Object    contentInt = null;
        Part      partInt    = null;
        
        if (currentContent instanceof Multipart) {            
            
            multipart = (Multipart)currentContent;
            int num =  multipart.getCount();
            
            for (int i = 0; i<num; i++) {
                
                partInt = multipart.getBodyPart(i);
                try {
                  contentInt = partInt.getContent();
                } catch (Exception e){
                    // javamail fails parsing the content
                    try {
                        contentInt = getTextBody(partInt);
                    } catch (Exception ee){
                        throw new ParsingException("Error parsing email content");
                    }   
                }  
                
                getPartInfoMutli(allParts, partIndex, partInt, contentInt);
                
            }    
            
        } else {
            
            InternalPart ip = getPartInfoSingle(part, partIndex, currentContent);            
            allParts.add(ip);
            partIndex[0] = (partIndex[0] + 1);
            
        }        
    }    
    
    /**
     *
     */
    private static InternalPart createIP(Part part, 
                                         int partSize, 
                                         Integer[] partIndex, 
                                         Object currentContent,
                                         Object contentInt) 
     throws MessagingException{  
        
        InternalPart ip = new InternalPart();                        
                
        ip.setPartValue(contentInt);
        
        ip.setDHandler(part.getDataHandler());
        
        ip.setFileName(part.getFileName());
        
        if (partSize == -1){
           ip.setSize(part.getSize());        
        } else {
           ip.setSize(partSize);        
        }
        
        // isBodyText() handles text/plain and text/calendar
        ip.setIsTextBody(isBodyTxt(part, partIndex, ip));

        // isBodyHtml() handles text/html and set the fileName
        ip.setIsHtmlBody(isBodyHtml(part, partIndex, ip));
                        
        // hanldes the content type
        if  (ip.isTextBody() && (currentContent instanceof String)){
            
            // if body is a string then it sets UTF-8 for all 
            // @todo add condition about DEVICE
            ip.setContentType(Def.CONTENT_CONTENTTYPE);
            //  TEST VIEWER
            //new ShowString((String)currentContent, "Conversion Demo");                
            
        } else if (ip.getContentType().equals(Def.CONTENT_CONTENTTYPE_CALENDAR)){
            
            // patch for outlook calendar (it will be replace before sending
            // the email to the client, it is used the IsTextCalendar flag)
            ip.setContentType(Def.CONTENT_CONTENTTYPE);
            ip.setDHandler(null);
            ip.setIsTextCalendar(true);
            String calendar = Def.CONTENT_BODY_ALERT;
            try {
                calendar = getTextBody(part);
            } catch (Exception e){
                // do nothing... leave the alert body content
            }    
            ip.setPartValue(calendar);
            
        } else {
            
            // if body is not a string
            ip.setContentType(part.getContentType());
            
        }  
        
        // @to be test
        // in some cases the javaMail retuens disposition null
        // we can put the following fix but we have to test
        // by default is an attachement
        ip.setDisposition(part.getDisposition());
        if ((ip.getFileName() != null) && (ip.getDisposition() == null) ){
            ip.setDisposition(MimeBodyPart.ATTACHMENT);
        } 
        
        // test 027
        if ((ip.getFileName() == null)) {
            if (ip.getDisposition() != null && (ip.getDisposition().equals(MimeBodyPart.INLINE))){
               if ((!ip.isTextBody()) && (!ip.isHtmlBody())){
                  if (ip.getContentType().indexOf("text/plain") != -1 ||
                      ip.getContentType().indexOf("TEXT/PLAIN") != -1 ){                  
                         ip.setDisposition(MimeBodyPart.INLINE);
                         ip.setFileName("Part00"+partIndex[0]+".txt");
                  }
               }
            }
        }
                            
        return ip;
        
    }
    
    /**
     *
     */
    private static boolean isBodyTxt(Part part, 
                                     Integer[] partIndex,
                                     InternalPart ip) {
        
        boolean isTextBody = false;
        
        try {
            if (part.getContentType().indexOf("text/plain") != -1 ||
                part.getContentType().indexOf("TEXT/PLAIN") != -1 ){
                if (part.getFileName() == null){
                    if (part.getDisposition() == null){
                        isTextBody = true;
                    } else if (part.getDisposition().equals(MimeBodyPart.INLINE) &&
                               partIndex[0] == 1){
                        // test 027 ; test 020
                        isTextBody = true;
                    } else {
                        isTextBody = false;
                    }
                }    
            } else if (part.getContentType().indexOf("text/calendar") != -1 ||
                       part.getContentType().indexOf("TEXT/CALENDAR") != -1 ){
                // test 029 ; test 030
                // patch for outlook 
                isTextBody = true;
                ip.setContentType(Def.CONTENT_CONTENTTYPE_CALENDAR);
            } else {
                // do nothing
            }
        } catch (MessagingException me){
            log.error("Error verifying if the Part is a TXT Body part", me);
        }
        
        return isTextBody;
    }
    
    /**
     *
     */
    private static boolean isBodyHtml(Part part, 
                                      Integer[] partIndex, 
                                      InternalPart ip) {
        
        boolean isHtmlBody = false;
        try {
            if (part.getContentType().indexOf("text/html") != -1 ||
                part.getContentType().indexOf("TEXT/HTML") != -1 ){
                if (part.getFileName() == null){
                    if (part.getDisposition() == null){
                        isHtmlBody = true;
                    } else if (part.getDisposition().equals(MimeBodyPart.INLINE) &&
                               partIndex[0] == 1){
                        // test 027 ; test 020; test 028
                        isHtmlBody = true;
                    } else {
                        // test 027 ; test 020; test 028
                        ip.setFileName("Att00"+partIndex[0]+".htm");
                        isHtmlBody = false;
                    }
                }    
            }
        } catch (MessagingException me){
            log.error("Error verifying if the Part is a HTML Body part", me);
        }
        return isHtmlBody;
    }
    
     
    /**
     * Utility method for obtaining a string representation of a
     * Message's body
     */
    private static String getTextBody(Part part, String charset) 
      throws Exception {
        
        String out = null;
        
        try {
            StringBuffer buffer = new StringBuffer();
            InputStream is = part.getInputStream();
            InputStreamReader isr = new InputStreamReader(is, charset);
            Reader in = new BufferedReader(isr);
            int ch;
            while ((ch = in.read()) > -1) {                                
               buffer.append((char)ch);
            }
            out = buffer.toString();
        } catch (Exception e) {
            // do nothing
            throw new Exception("Error getting text using the charset: " + charset);
        }

        return out;
        
    }    
    
    /**
     * Utility method for obtaining a string representation of a
     * Message's body
     */
    private static String getTextBody(Part part) throws Exception {
        
        String       body  = null;
        InputStream  bis   = null;
        OutputStream bos   = null;
        ByteArrayOutputStream bodyOs = new ByteArrayOutputStream();
        
        byte[] block = new byte[1024];
        int read = 0;
        
        try {
            bis = part.getInputStream();
            bos = bodyOs;
            while ((read = bis.read(block)) > -1) {
                bos.write(block, 0, read);
            }
            bos.flush();
            body = bodyOs.toString();
        } catch(MessagingException me) {
            throw me;
        } finally {
            bis.close();
        }
        
        return body;
        
    }  
         
}
