/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport  ;


import java.util.Locale;
import java.util.Properties;
import javax.mail.Session;
import javax.mail.Store;
import javax.mail.Folder;
import com.sun.net.ssl.internal.ssl.Provider;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.util.Def;
import com.funambol.email.model.DefaultFolder;
import com.funambol.email.util.Utility;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;


/**
 *
 * <p>
 * Create a connection with a Mail Server
 * </p>
 *
 * @version $Id: CommonMailServerWrapper.java,v 1.23 2007/03/19 14:23:35 gbmiglia Exp $
 */
public abstract class CommonMailServerWrapper
        implements IMailServerWrapper
{

    // --------------------------------------------------------------- Constants

    /**     */
    protected FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);

    // -------------------------------------------------------------- Properties

    /**     */
    protected Session session;
    /**     */
    protected Store store ;
    /**     */
    protected DefaultFolder defaultFolder ;
    /**     */
    protected Folder mailDefaultFolder ;
    /**     */
    protected Locale locale;
    /**     */
    protected String serverType;
    /**     */
    protected String serverOut;
    /**     */
    protected String userAddress;


    /**
     * opens the connection with the mail server
     *
     *
     * @param serverType String
     * @param outgoingServer String
     * @param outgoingPort String
     * @param smtpAuth boolean
     * @param incomingServer String
     * @param incomingPort String
     * @param defaultFolder DefaultFolder
     * @param user String
     * @param password String
     * @param isSSLIn boolean
     * @param isSSLOut boolean
     * @param keystorePath String
     * @param keystorePassphrase String
     * @throws EmailAccessException
     */
    public abstract void openConnection(String  serverType,
                               String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder defaultFolder,
                               String  user,
                               String  password,
                               boolean isSSLIn,
                               boolean isSSLOut,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException ;

    /**
     *
     * @throws EmailAccessException
     */
    public abstract void releaseConnection()  throws EmailAccessException ;

    /**
     *
     *
     *
     * @param server String
     * @param incomingPort String
     * @param user String
     * @param password String
     * @param isSSLIn boolean
     * @param keystorePath String
     * @param keystorePassphrase String
     * @throws EmailAccessException
     */
    public void checkUser(String server,
                          String incomingPort,
                          String user,
                          String password,
                          boolean isSSLIn,
                          String keystorePath,
                          String keystorePassphrase)
     throws EmailAccessException{

      if (log.isTraceEnabled()) {
          log.trace("\nCONNECTION PARAMs"+
                     "\nserver       " + server+ "("+Utility.getMailServerIP(server)+")" +
                     "\nport         " + incomingPort+
                     "\nprotocol     " + this.getClass().getName()+
                     "\nuser         " + user+
                     "\nSSLIn        " + isSSLIn+
                     "\nKeystorePath " + keystorePath);
      }

      // contains for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;

      if (isSSLIn) {
        setConnectionSSLIn(props, server, incomingPort,
                           provider, keystorePath, keystorePassphrase);
      } else {
        setConnectionNormalIn(props, server, incomingPort);
      }

      try{
         this.session = Session.getInstance(props, null);
      } catch (Exception e) {
	 throw new EmailAccessException("Error Setting Session for Server (" + e.getMessage() + ")");
      }     
      
      boolean isOpen = multipleOpenConnection(server, user, password);      

      if (isOpen){
          try {
             this.store.close();
             this.store = null;
             this.session = null;
             props.clear();
          } catch (Exception e) {
                throw new EmailAccessException("Error closing store " + e.toString());
          }
          log.info("Check User operation done");
      } else {
          log.info("Error Checking User - Synchronization Interrupt");
      }

    }

    /**
     *
     * @param props Properties
     * @param incomingServer String
     * @param incomingPort String
     */
    protected abstract void setConnectionNormalIn(Properties props,
                                                  String incomingServer,
                                                  String incomingPort);

    /**
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param smtpAuth boolean
     */
    protected abstract void setConnectionNormalOut(Properties props,
                                                   String outgoingServer,
                                                   String outgoingPort,
                                                   boolean smtpAuth);
    /**
     *
     * @param props Properties
     * @param incomingServer String
     * @param incomingPort String
     * @param provider Provider
     * @param keystorePath String
     * @param kestorePassphrase String
     */
    protected abstract void  setConnectionSSLIn(Properties props,
                                                String incomingServer,
                                                String incomingPort,
                                                Provider provider,
                                                String keystorePath,
                                                String kestorePassphrase);
    
    /**
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param smtpAuth boolean
     * @param provider Provider
     * @param keystorePath String
     * @param kestorePassphrase String
     */
    protected abstract void  setConnectionSSLOut(Properties props,
                                                 String outgoingServer,
                                                 String outgoingPort,
                                                 boolean smtpAuth,
                                                 Provider provider,
                                                 String keystorePath,
                                                 String kestorePassphrase);
    
    /**
     *
     *
     * @return Session
     */
    public Session getSession(){
       return this.session;
    }

    /**
     *
     * @return Store
     */
    public Store getStore(){
       return this.store;
    }

    /**
     *
     * @return Folder
     */
    public Folder getMailDefaultFolder(){
       return this.mailDefaultFolder;
    }

    /**
     *
     * @return DefaultFolder
     */
    public DefaultFolder getDefaultFolder(){
       return this.defaultFolder;
    }


    /**
     *
     *
     * @return Locale
     */
    public Locale getLocale(){
       return this.locale;
    }
        
    /**
     * used in the creation of the Message-ID
     * see IMAP-based  getEmailFromUID
     *
     * @return serverOut
     */
    public String getServerOut(){
       return this.serverOut;
    }

    /**
     * used in the creation of the Message-ID
     * see IMAP-based  getEmailFromUID
     *
     * @return String
     */
    public String getUserAddress(){
       return this.userAddress;
    }

  // --------------------------------------------------------- PRIVATE METHODS

  /**
   *
   *
   */
  protected boolean multipleOpenConnection(String incomingServer,
                                           String user,
                                           String password)
    throws EmailAccessException {

      boolean isOpen = false;

      try {
          this.store = session.getStore();
          this.store.connect(incomingServer, user, password);
          isOpen = true;
      } catch (Exception e1) {
          try {
              Thread.sleep(3000);
              this.store = session.getStore();
              this.store.connect(incomingServer, user, password);
              isOpen = true;
          } catch (Exception e2) {
              try {
                 Thread.sleep(3000);
                 this.store = session.getStore();
                 this.store.connect(incomingServer, user, password);
                 isOpen = true;
              } catch (Exception e3) {
                 log.error("Error Opening Connection Store: ", e3);
                 throw new EmailAccessException("Error Opening Connection Store (" + 
                         e3.getMessage() + ")");
              }
          }                      
      }

      return isOpen;

    }

}
