/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport  ;

import java.util.Properties;
import java.security.Security;
import javax.mail.Session;
import com.sun.net.ssl.internal.ssl.Provider;
import com.sun.mail.imap.IMAPFolder;
import javax.mail.MessagingException;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.model.DefaultFolder;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.PasswordAuthentication;


/**
 *
 * <p>
 * Create a connection with a Mail Server
 * using the IMAP protocol
 * </p>
 *
 * @version $Id: ImapMailServerWrapper.java,v 1.31 2007/03/19 14:23:35 gbmiglia Exp $
 */
public class ImapMailServerWrapper
        extends CommonMailServerWrapper
        implements IMailServerWrapper
{

    // -------------------------------------------------------------- Properties

    /**
     * used in the getMessageByUID method. in the POP protocol 
     * there is no the method "folder.getMessageByUID(String)
     * At the end of the sync session the server calls the getSyncItem
     * In this method we open and close the folder every time.
     * Using this propreties we can leave the folder open for alle the
     * calls to the getSyncItem method.
     * In the EndSync we close this folder if it's open
     *
     */
    public Message[]  messagesInboxOpened  = null;
    
    public IMAPFolder folderInboxOpened    = null;
        
    // ---------------------------------------------------------- PUBLIC METHODS


    /**
     * opens the connection with the mail server
     *
     *
     * @param serverType String
     * @param outgoingServer SMPT address i.e. 192.168.0.1
     * @param outgoingPort 25 def.
     * @param smtpAuth boolean
     * @param incomingServer IMAP address i.e. 192.168.0.1
     * @param incomingPort 143 / 993 IMAP def.
     * @param df inbox, outbox, sent items,...
     * @param user String
     * @param password String
     * @param isSSLIn true/false
     * @param isSSLOut true/false
     * @param keystorePath String
     * @param keystorePassphrase String
     * @throws EmailAccessException
     */
    public void openConnection(String  serverType,
                               String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder df,
                               final String  user,
                               final String  password,
                               boolean isSSLIn,
                               boolean isSSLOut,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException {

      if (log.isTraceEnabled()) {
          log.trace("Connection Parameters ["+
                    "(protocol: "+this.getClass().getName()+ ") "+
                    "(out: "+outgoingServer+ "/"+Utility.getMailServerIP(outgoingServer) + ") " +
                    "(port: "+outgoingPort+ ") "+
                    "(smtpauth: "+smtpAuth+") "+
                    "(in: "+incomingServer+ "/"+Utility.getMailServerIP(incomingServer) + ") " +
                    "(port: " + incomingPort+") " +
                    "(user: "+user+ ") (SSL in: " + isSSLIn+ " - SSL out: "+isSSLIn+") "+
                    "(keystore: "+keystorePath+")]");
      }        

      // set info
      this.userAddress = user;
      this.serverType  = serverType;
      this.serverOut   = outgoingServer;

      // containe for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;

      if (isSSLIn){
        this.setConnectionSSLIn(props, incomingServer, incomingPort,
                                provider, keystorePath, keystorePassphrase);
      } else {
        this.setConnectionNormalIn(props, incomingServer, incomingPort);
      }
      if (isSSLOut){
        this.setConnectionSSLOut(props, outgoingServer, outgoingPort, smtpAuth, 
                              provider, keystorePath, keystorePassphrase);
      } else {
        this.setConnectionNormalOut(props, outgoingServer, outgoingPort, smtpAuth);
      }

      try{
         if (smtpAuth){
            Authenticator auth = new javax.mail.Authenticator() 
                 {
                    protected PasswordAuthentication getPasswordAuthentication() {
                          return new PasswordAuthentication(user, password);
                    }
                  };           
            this.session = Session.getInstance(props, auth);
         } else {
            this.session = Session.getInstance(props, null);
         }
      } catch (Exception e) {
         log.error("Error Setting Authentication for Server: ", e);
	 throw new EmailAccessException("Error Setting Authentication for Server (" + e.getMessage() + ")");
      }

      boolean isOpen = multipleOpenConnection(incomingServer, user, password);

      if (isOpen){
          try {
              this.mailDefaultFolder  = store.getDefaultFolder();
              this.defaultFolder      = df;
          } catch (Exception e) {
              log.error("Error getting Default Folder from Store: ", e);
              throw new EmailAccessException("Error getting Default Folder from Store " +
                      "(" + e.getMessage() + ")");                  
          }
          
          try {
              // open the Inbox folder
              this.folderInboxOpened = (IMAPFolder) 
                          this.mailDefaultFolder.getFolder(df.getInboxName());
              this.folderInboxOpened.open(javax.mail.Folder.READ_WRITE);  
          } catch (Exception e) {
              log.error("Error getting Folders: ", e);
              throw new EmailAccessException("Error getting Folders (" + e.getMessage() + ")");                  
          }
          if (log.isTraceEnabled()) {
              log.info("IMAP Connection opened");
          }
      } else {
          log.error("IMAP Error Opening Connection Store - Synchronization Interrupt");
      }

    }


    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException {
        if (log.isTraceEnabled()) {
            log.info("IMAP Start Connection Closing");
        }
        try {
            if (this.folderInboxOpened != null) {
                if (this.folderInboxOpened.isOpen()) {
                    this.folderInboxOpened.close(true);
                }
            }
            if (this.store != null){
                if (this.store.isConnected()){
                   this.store.close();
                }
            }
            this.store = null;
            this.session = null;
        } catch (Exception e) {
     	    throw new EmailAccessException("Error Closing Store " + e.toString());
        }

    }

    /**
     *
     * returns Inbox Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getInboxFolder() throws EmailAccessException {
       IMAPFolder inbox;
       try {
           inbox = (IMAPFolder)mailDefaultFolder.getFolder(
                                             defaultFolder.getInboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting Inbox Folder " +
                                          me.toString());
       }
       return inbox;
    }


    /**
     *
     * returns Outbox Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getOutboxFolder() throws EmailAccessException {
       IMAPFolder outbox;
       try {
           outbox = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getOutboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting outbox Folder " +
                                          me.toString());
       }
       return outbox;
    }


    /**
     *
     * returns Sent Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getSentFolder() throws EmailAccessException {
       IMAPFolder sent;
       try {
           sent = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getSentName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting sent Folder " +
                                          me.toString());
       }
       return sent;
    }

    /**
     *
     * returns Drafts Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getDraftsFolder() throws EmailAccessException {
       IMAPFolder drafts;
       try {
           drafts = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getDraftsName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting drafts Folder " +
                                          me.toString());
       }
       return drafts;
    }

    /**
     *
     * returns Trash Folder Instance
     *
     * @return IMAPFolder
     * @throws EmailAccessException
     */
    public IMAPFolder getTrashFolder() throws EmailAccessException {
       IMAPFolder trash;
       try {
           trash = (IMAPFolder)mailDefaultFolder.getFolder(
                                              defaultFolder.getTrashName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting trash Folder " +
                                          me.toString());
       }
       return trash;
    }

    //---------------------------------------------------------- PRIVATE METHODS

   /**
    *
    *
    * @param props Properties
    * @param incomingServer String
    * @param incomingPort String
    * @param provider Provider
    * @param keystorePath String
    * @param keystorePassphrase String
    */
   protected void setConnectionSSLIn(Properties props,
                                     String incomingServer,
                                     String incomingPort,
                                     Provider provider,
                                     String keystorePath,
                                     String keystorePassphrase)
   {

        if (log.isTraceEnabled()) {
            log.trace("IMAP set an SSL connection Input");
        }

        provider = new Provider();
        //System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
        Security.setProperty("javax.net.ssl.keyStore"          ,keystorePath);
        Security.setProperty("javax.net.ssl.trustStorePassword",keystorePassphrase);
	Security.addProvider(provider);
         
        props.setProperty("mail.imap.host"                   , incomingServer);
        props.setProperty("mail.imap.port"                   , incomingPort);
        props.setProperty("mail.store.protocol"              , "imap");
        
        props.setProperty("mail.imap.timeout"                , Def.TIMEOUT_STORE);
          
        props.setProperty("mail.imap.socketFactory.class"    , "javax.net.ssl.SSLSocketFactory");
    	props.setProperty("mail.imap.socketFactory.fallback" , "false");
        props.setProperty("mail.imap.socketFactory.port"     , incomingPort);

    }

   /**
    *
    *
    * @param props Properties
    * @param outgoingServer String
    * @param outgoingPort String
    * @param smtpAuth boolean
    * @param provider Provider
    * @param keystorePath String
    * @param keystorePassphrase String
    */
   protected void setConnectionSSLOut(Properties props,
                                      String outgoingServer,
                                      String outgoingPort,
                                      boolean smtpAuth,
                                      Provider provider,
                                      String keystorePath,
                                      String keystorePassphrase)
   {

        if (log.isTraceEnabled()) {
            log.trace("IMAP set an SSL connection Output");
        }

        provider = new Provider();
        //System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
        Security.setProperty("javax.net.ssl.keyStore",keystorePath);
        Security.setProperty("javax.net.ssl.trustStorePassword",keystorePassphrase);
	Security.addProvider(provider);
         
        props.setProperty("mail.transport.protocol" , "smtp");
        props.setProperty("mail.smtp.host" , outgoingServer);
        props.setProperty("mail.smtp.port" , outgoingPort);
        props.setProperty("mail.smtp.socketFactory.port", outgoingPort);        
        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }
        props.setProperty("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
        props.setProperty("mail.smtp.socketFactory.fallback", "false");
        props.setProperty("mail.smtp.socketFactory.port", outgoingPort);
         
    }
   
    /**
     *
     *
     * @param props Properties
     * @param incomingServer String
     * @param incomingPort String
     */
    protected void setConnectionNormalIn(Properties props,
                                         String incomingServer,
                                         String incomingPort){

        if (log.isTraceEnabled()) {
            log.trace("IMAP set a normal connection Input");
        }
        props.setProperty("mail.imap.host"      , incomingServer);
        props.setProperty("mail.imap.port"      , incomingPort);
        props.setProperty("mail.store.protocol" , "imap");
        props.setProperty("mail.imap.timeout"   , Def.TIMEOUT_STORE);
        
    }

    /**
     *
     *
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param smtpAuth boolean
     */
    protected void setConnectionNormalOut(Properties props,
                                          String outgoingServer,
                                          String outgoingPort,
                                          boolean smtpAuth){

        if (log.isTraceEnabled()) {
            log.trace("IMAP set a normal connection Output");
        }

        props.setProperty("mail.smtp.host", outgoingServer);
        props.setProperty("mail.smtp.port", outgoingPort);
        props.setProperty("mail.transport.protocol", "smtp");
        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }
        
    }
    
}
