/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.transport;


import java.util.Properties;
import java.security.Security;
import javax.mail.Session;
import com.sun.net.ssl.internal.ssl.Provider;
import com.sun.mail.pop3.POP3Folder;
import javax.mail.MessagingException;
import com.funambol.email.exception.EmailAccessException;
import com.funambol.email.model.DefaultFolder;
import com.funambol.email.util.Def;
import com.funambol.email.util.Utility;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.PasswordAuthentication;


/**
 *
 * <p>Create a connection with a Mail Server </p>
 *
 * @version $Id: PopMailServerWrapper.java,v 1.33 2007/03/19 14:23:35 gbmiglia Exp $
 */
public class PopMailServerWrapper
        extends CommonMailServerWrapper
        implements IMailServerWrapper
{

    // -------------------------------------------------------------- Properties

    /**
     * used in the getMessageByUID method. in the POP protocol 
     * there is no the method "folder.getMessageByUID(String)
     * At the end of the sync session the server calls the getSyncItem
     * In this method we open and close the folder every time.
     * Using this propreties we can leave the folder open for alle the
     * calls to the getSyncItem method.
     * In the EndSync we close this folder if it's open
     *
     */
    public Message[]  messagesInboxOpened  = null;
    
    public POP3Folder folderInboxOpened    = null;
   
    // ---------------------------------------------------------- PUBLIC METHODS


    /**
     * opens the connection with the mail server
     *
     *
     * @param serverType Server Type
     * @param outgoingServer SMPT address i.e. 192.168.0.1
     * @param outgoingPort 25 def.
     * @param smtpAuth boolean
     * @param incomingServer POP3 address i.e. 192.168.0.1
     * @param incomingPort 110 / 995 POP3 def.
     * @param df inbox, outbox, sent items,...
     * @param user String
     * @param password String
     * @param isSSLIn boolean
     * @param isSSLOut boolean
     * @param keystorePath String
     * @param keystorePassphrase String
     * @throws EmailAccessException
     */
    public void openConnection(String  serverType,
                               String  outgoingServer,
                               String  outgoingPort,
                               boolean smtpAuth,
                               String  incomingServer,
                               String  incomingPort,
                               DefaultFolder df,
                               final String  user,
                               final String  password,
                               boolean isSSLIn,
                               boolean isSSLOut,
                               String keystorePath,
                               String keystorePassphrase)
      throws EmailAccessException {

      
      if (log.isTraceEnabled()) {
          log.trace("Connection Parameters ["+
                    "(protocol: "+this.getClass().getName()+ ") "+
                    "(out: "+outgoingServer+ "/"+Utility.getMailServerIP(outgoingServer) + ") " +
                    "(port: "+outgoingPort+ ") "+
                    "(smtpauth: "+smtpAuth+") "+
                    "(in: "+incomingServer+ "/"+Utility.getMailServerIP(incomingServer) + ") " +
                    "(port: " + incomingPort+") " +
                    "(user: "+user+ ") (SSLin: " + isSSLIn+ " - SSLout: " + isSSLOut + ") "+
                    "(keystore: "+keystorePath+")]");
      }
      
      // set info
      this.userAddress = user;
      this.serverType  = serverType;
      this.serverOut   = outgoingServer;

      // containe for properties
      Properties props = new Properties();

      // provider for a SSL connection
      Provider provider = null;

      if (isSSLIn){
        this.setConnectionSSLIn(props, incomingServer, incomingPort,
                                provider, keystorePath, keystorePassphrase);
      } else {
        this.setConnectionNormalIn(props, incomingServer, incomingPort);
      }
      if (isSSLOut){
        this.setConnectionSSLOut(props, outgoingServer, outgoingPort, smtpAuth, 
                              provider, keystorePath, keystorePassphrase);
      } else {
        this.setConnectionNormalOut(props, outgoingServer, outgoingPort, smtpAuth);
      }

      try{

        if (smtpAuth){
           Authenticator auth = new javax.mail.Authenticator() 
                 {
                    protected PasswordAuthentication getPasswordAuthentication() {
                          return new PasswordAuthentication(user, password);
                    }
                  };           
           this.session = Session.getInstance(props, auth);
        } else {
           this.session = Session.getInstance(props, null);
        }
        
      } catch (Exception e) {
         log.error("Error Setting Authentication for Server: ", e);
	 throw new EmailAccessException("Error Setting Authentication for Server (" + e.getMessage() + ")");
      }

      boolean isOpen = multipleOpenConnection(incomingServer, user, password);

      if (isOpen){
          
          try {
              this.mailDefaultFolder = store.getDefaultFolder();
              this.defaultFolder     = df;
          } catch (Exception e) {
              log.error("Error getting Default Folder from Store: ", e);
              throw new EmailAccessException("Error getting Default Folder from Store " +
                      "(" + e.getMessage() + ")");                  
          }
          
          // open the Inbox folder
          try {
              this.folderInboxOpened = (POP3Folder) 
                          this.mailDefaultFolder.getFolder(df.getInboxName());              
              this.folderInboxOpened.open(javax.mail.Folder.READ_WRITE);                                    
          } catch (Exception e) {
              log.error("Error getting Folders: ", e);
              throw new EmailAccessException("Error getting Folders (" + e.getMessage() + ")");                  
          }
          if (log.isTraceEnabled()) {
              log.trace("POP3 Connection opened");
          }
      } else {
          log.error("POP3 Error Opening Connection Store - Synchronization Interrupt");
      }

    }



    /**
     * closes the connection with the mail server
     *
     * @throws EmailAccessException
     */
    public void releaseConnection() throws EmailAccessException {
         if (log.isTraceEnabled()) {
              log.trace("POP3 Start Connection Closing");
         }
         try {
            if (this.folderInboxOpened != null) {
                if (this.folderInboxOpened.isOpen()) {
                    this.folderInboxOpened.close(true);
                }
            }            
            if (this.store != null){
                if (this.store.isConnected()){
                   this.store.close();
                }
            }
            this.store = null;
            this.session = null;
        } catch (Exception e) {
     	    throw new EmailAccessException("Error Closing Store " + e.toString());
        }

    }


    /**
     *
     * returns Inbox Folder Instance
     *
     * @return POP3Folder
     * @throws EmailAccessException
     */
    public POP3Folder getInboxFolder() throws EmailAccessException {
       POP3Folder inbox;
       try {
           inbox = (POP3Folder)mailDefaultFolder.getFolder(
                                             defaultFolder.getInboxName());
       } catch (MessagingException me){
           throw new EmailAccessException("Error getting Inbox Folder " +
                                          me.toString());
       }
       return inbox;
    }

    //---------------------------------------------------------- PRIVATE METHODS

   /**
    *
    *
    *
    * @param props Properties
    * @param incomingServer String
    * @param incomingPort String
    * @param provider Provider
    * @param keystorePath String
    * @param keystorePassphrase String
    */
   protected void setConnectionSSLIn(Properties props,
                                     String incomingServer,
                                     String incomingPort,
                                     Provider provider,
                                     String keystorePath,
                                     String keystorePassphrase)
   {
       
        if (log.isTraceEnabled()) {
            log.trace("POP3 set an SSL connection Incoming");
        }
        
        provider = new Provider();
        //System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
        Security.setProperty("javax.net.ssl.keyStore",keystorePath);
        Security.setProperty("javax.net.ssl.trustStorePassword",keystorePassphrase);
	Security.addProvider(provider);
        
        props.setProperty("mail.store.protocol"              , "pop3");                
        props.setProperty("mail.pop3.host"                   , incomingServer);
        props.setProperty("mail.pop3.port"                   , incomingPort);        
        
        props.setProperty("mail.pop3.timeout"                , Def.TIMEOUT_STORE);                
        
    	props.setProperty("mail.pop3.socketFactory.class"    ,"javax.net.ssl.SSLSocketFactory");
    	props.setProperty("mail.pop3.socketFactory.fallback" ,"false");
    	props.setProperty("mail.pop3.socketFactory.port"     ,incomingPort);

    }

   
   /**
    *
    *
    *
    * @param props Properties
    * @param outgoingServer String
    * @param outgoingPort String
    * @param smtpAuth boolean
    * @param provider Provider
    * @param keystorePath String
    * @param keystorePassphrase String
    */
   protected void setConnectionSSLOut(Properties props,
                                      String outgoingServer,
                                      String outgoingPort,
                                      boolean smtpAuth,
                                      Provider provider,
                                      String keystorePath,
                                      String keystorePassphrase)
   {
       
        if (log.isTraceEnabled()) {
            log.trace("POP3 set an SSL connection Outgoing");
        }
        
        provider = new Provider();
        //System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");
        Security.setProperty("javax.net.ssl.keyStore",keystorePath);
        Security.setProperty("javax.net.ssl.trustStorePassword",keystorePassphrase);
	Security.addProvider(provider);
        
        props.setProperty("mail.transport.protocol" , "smtp");
        props.setProperty("mail.smtp.host" , outgoingServer);
        props.setProperty("mail.smtp.port" , outgoingPort);
        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }
        props.setProperty("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
        props.setProperty("mail.smtp.socketFactory.fallback", "false");
        props.setProperty("mail.smtp.socketFactory.port", outgoingPort);

    }
      
    /**
     *
     *
     *
     * @param props Properties
     * @param incomingServer String
     * @param incomingPort String
     */
    protected void setConnectionNormalIn(Properties props,
                                         String incomingServer,
                                         String incomingPort){
        
        if (log.isTraceEnabled()) {
            log.trace("POP3 set a normal connection Input");
        }        
        props.setProperty("mail.pop3.host"      , incomingServer);
        props.setProperty("mail.pop3.port"      , incomingPort);
        props.setProperty("mail.store.protocol" , "pop3");
        props.setProperty("mail.imap.timeout"   , Def.TIMEOUT_STORE);
        
    }
    /**
     *
     *
     *
     * @param props Properties
     * @param outgoingServer String
     * @param outgoingPort String
     * @param smtpAuth boolean
     */
    protected void setConnectionNormalOut(Properties props,
                                          String outgoingServer,
                                          String outgoingPort,
                                          boolean smtpAuth){
        
        if (log.isTraceEnabled()) {
            log.trace("POP3 set a normal connection Output");
        }        
        props.setProperty("mail.smtp.host", outgoingServer);
        props.setProperty("mail.smtp.port", outgoingPort);
        props.setProperty("mail.transport.protocol", "smtp");
        if (smtpAuth){
           props.setProperty("mail.smtp.auth", "true");
        }
        
    }

}
