/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.util;

import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import java.security.spec.KeySpec;
import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESedeKeySpec;
import org.apache.commons.codec.binary.Base64;

/**
 * @version $Id: TokenUtility.java,v 1.8 2007/03/19 14:23:14 gbmiglia Exp $
 */
public class TokenUtility {

    // ------------------------------------------------------------ Public Methods

    /**     */
    protected static FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
    
    // ------------------------------------------------------------ Public Methods

    
    /**
     * 
     */ 
    public static String getToken() {
        
        String d   = "Omnia Gallia in tres partes divida est";
        byte[] tok = null;
        try {
            tok = new TokenProvider().get();
            d   = new String(tok);
            log.debug("encryption key found; using fnbl key");
        } catch (java.lang.UnsatisfiedLinkError jlu){
            log.debug("encryption key not found; using default key " + jlu.toString());
        } catch (Exception e){
            log.debug("encryption key not found; using default key " + e.toString());
        }        
        return d;
    }
    
    /**
     * Encrypt a password.
     *
     * @param plain_text the password to encrypt
     * @param encryptionKey the key to use in the encryption phase
     * @return the encrypted password.
     * @exception Exception
     */
    public static String encryptPassword(String plain_text, 
                                         String encryptionKey) 
       throws Exception {
        
        String encryptedPassword = null;
                
        try {
                  
            encryptionKey = checkEncryptionKey(encryptionKey);        
            
            byte[] bytes = encryptionKey.getBytes(Def.PWD_ENCODE_UTF8);            
            KeySpec keySpec = new DESedeKeySpec(bytes);            
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(Def.PWD_ALGORITHM);            
            Cipher cipher = Cipher.getInstance(Def.PWD_ALGORITHM);                        
            
            SecretKey key = keyFactory.generateSecret(keySpec);            
            cipher.init(Cipher.ENCRYPT_MODE, key);            
            
            
            byte[] plain_bytes  = plain_text.getBytes(Def.PWD_ENCODE_UTF8);            
            byte[] cipher_bytes = cipher.doFinal(plain_bytes);
            byte[] encpwd_bytes = Base64.encodeBase64(cipher_bytes);
                    
            encryptedPassword =  new String(encpwd_bytes);
            
        }
        catch (Exception e) {
             throw new Exception(e);
        }
        
        return encryptedPassword;
    }

    /**
     * Decrypt a password.
     *
     * @param encryptedPassword  the encrypted password, needing to be decrypted
     * @param encryptionKey  
     * @return the original, plain text password
     * @exception Exception
     */
    public static String decryptPassword(String encryptedPassword, 
                                          String encryptionKey) 
        throws Exception {

        String plain_text = null;

        
        try {
            
            encryptionKey = checkEncryptionKey(encryptionKey);        
           
            byte[] bytes = encryptionKey.getBytes(Def.PWD_ENCODE_UTF8);            
            KeySpec keySpec = new DESedeKeySpec(bytes);            
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance(Def.PWD_ALGORITHM);            
            Cipher cipher = Cipher.getInstance(Def.PWD_ALGORITHM);
            
            SecretKey key = keyFactory.generateSecret(keySpec);            
            cipher.init(Cipher.DECRYPT_MODE, key);
            
            byte[] encpwd_bytes = encryptedPassword.getBytes();            
            byte[] cipher_bytes = Base64.decodeBase64(encpwd_bytes);            
            byte[] plain_bytes  = cipher.doFinal(cipher_bytes);
            
            plain_text = new String(plain_bytes);
            
        }
        catch (Exception e) {
             throw new Exception(e);
        }
        
        return plain_text;
    }

    //----------------------------------------------------------- Private Method
    
    /**
     * insert chars to reach lenght 32
     */
    private static String checkEncryptionKey(String key){
        String key32 = "";
        int l = key.length();
        if (l < 32){
            key32 = key;
            for (int i=0; i<(32-l); i++){
              key32 = key32 + " ";
            }
        } else {
            key32 = key;
        }
        //System.out.println("|"+key32+"|");
        //System.out.println("key32 lenght " + key32.length());
        return key32;
        
    }
    
}
