/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.email.util;


import java.util.Date;
import java.util.Locale;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.mail.Message;
import javax.mail.MessagingException;
import com.funambol.email.exception.EntityException;
import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import javax.mail.internet.MailDateFormat;

/**
 * <p>Utility class</p>
 *
 * @version $Id: UtilityDate.java,v 1.13 2007/03/16 10:38:02 gbmiglia Exp $
 */

public class UtilityDate {

    //------------------------------------------------------------------ CONSTANTS
    
    /**     */
    protected static FunambolLogger log = FunambolLoggerFactory.getLogger(Def.LOGGER_NAME);
        
    
    // ------------------------------------------------------------ Public Methods


    /**
     * Returns an origination date in UTC format
     * <p>
     * Available Date Parser
     * <p>
     * pat_1: Exchange 2003 (eng): wed, 6 Oct 2005 09:46:47 +0200
     * <p>
     * pat_2: Exchange 2000 (eng): 6 Oct 2005 09:46:47 +0200
     * <p>
     * pat_3: BB client          : Wed Dec 28 20:05:53 GMT 2005
     * <p>
     * pat_4: ??                 : Wed Dec 28 20:05:53 2005
     *
     * @param message Message
     * @param loc     Locale
     * @return date in the header Date
     */
    public static Date getHeaderDate(Message message, Locale loc) 
      throws EntityException {

        Date origDate = null;
        String[] values = null;

        String subj = "";
        try {
            subj = message.getSubject();
        } catch (MessagingException me2){
            // do nothing
        }
        
        try {
            values = message.getHeader(Def.HEADER_DATE);
        } catch (MessagingException me) {
            throw new EntityException(
                    "Error getting Header Date for email: [" + subj + "]", me); 
        }

        if ((values != null) && (values.length > 0)) {
            origDate = dateStringToDate(values[0], subj);             
        }

        return origDate;
    }   
    
    /**
     *
     * @param message Message
     * @return String
     */
    public static Date getHeaderReceived(Message message) {

        try {
            Date d = message.getReceivedDate();
            return d;
        }
        catch (Exception e) {
            // do nothing; we just want to return null
        }
        return null;
    }    
    
    
    /**
     * Returns true if the date is parsable
     *
     * @param message Message
     * @return true if the header date is parsable
     */
    public static boolean isHeaderDateParsable(Message message) {

        boolean check = true;
        
        try {
            String[] values = message.getHeader(Def.HEADER_DATE);            
            if ((values != null) && (values.length > 0)) {
                String s = values[0];
                MailDateFormat mailDateFormat = new MailDateFormat();
                Date origDate = mailDateFormat.parse(s);
                if (origDate == null) {
                    check = false;
                }            
            } else {
                // there is not the Date
                check = false;
            }
        } catch (Exception me) {
            check = false;
            // do nothing
        }

        return check;
    }   
    
    
    
    /**
     *
     * @param message Message
     * @return String
     */
    public static Date getDateForTimeFilter(Message message, Locale loc) {

        Date d = null;
        try {
            d = getHeaderReceived(message);
            if (d == null){
                // this method returns always a date
                d = getHeaderDate (message,loc);
            }
        } catch (Exception e) {
            // do nothing; we just want to return null
        }
        return d;
    }    

        
    /**
     *
     * this is a fix for all strange date format;
     * it returns the NOW value.
     *
     */
   public static String  getHeaderDateNowPatch(){      
       java.util.Date d = new java.util.Date(System.currentTimeMillis());
       DateFormat utcf = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);
       String out = utcf.format(d);
       return out;
       
   }
     
    /**
     * this method always returns a Date value.
     * if the parser returns a null value, it applies the getHeaderDatePatch3().
     * the getHeaderDatePatch3() method returns the NOW value.
     * it's public for testing purpose
     *
     */
    public static Date dateStringToDate(String s, String subj) 
    throws EntityException {

        Date origDate = null;
        
        try {
            MailDateFormat mailDateFormat = new MailDateFormat();
            origDate = mailDateFormat.parse(s);
        } catch (Exception e1) {
            throw new EntityException(
                    "Error parsing Header Date for email: [" + subj + "]", e1);                        
        }
        
        try {
            if (origDate == null) {
                s = getHeaderDateNowPatch();
                origDate = formatter(s);
            }
        } catch (Exception e1) {
            throw new EntityException(
                    "Error parsing Header Date for email: [" + subj + "]", e1);                        
        }

        
        return origDate;
    }
    
    
    /**
     *
     */
    public static Date formatter(String s) 
    throws EntityException {

       DateFormat pat_1 = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss Z", Locale.US);
       DateFormat pat_2 = new SimpleDateFormat("dd MMM yyyy HH:mm:ss Z", Locale.US);
       DateFormat pat_3 = new SimpleDateFormat("EEE MMM dd HH:mm:ss Z yyyy", Locale.US);
       DateFormat pat_4 = new SimpleDateFormat("EEE MMM dd HH:mm:ss yyyy", Locale.US);
            
       Date origDate = null;
       try {
            origDate = pat_1.parse(s);
       } catch (Exception e1) {
            try {
                origDate = pat_2.parse(s);
            } catch (Exception e2) {
                try {
                    origDate = pat_3.parse(s);
                } catch (Exception e3) {
                    try {
                        origDate = pat_4.parse(s);
                    } catch (Exception e4) {
                        throw new EntityException("Error parsing Header Date", e4);                                    
                    }
                }
            }
       }
       return origDate;
    }    
    
}
